/*
  Copyright (C) 2009 Pot Etienne

  This file is part of Vocab++.

  Vocab++ is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Vocab++ is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Vocab++.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "dialog_ajouter_question.hpp"

#define TAILLE_MAXIMUM_SCROLL_AREA 150

Dialog_ajouter_question::Dialog_ajouter_question(QStringList *c_liste_tags, QStringList *c_liste_tags_defaut, QList<QDockWidget*> *c_claviers_virtuels, QWidget *parent) : QDialog(parent)
{
    liste_tags = c_liste_tags;
    liste_tags_defaut = c_liste_tags_defaut;

    champ_question = new QLineEdit;
    champ_reponse = new QLineEdit;
    champ_question->installEventFilter(this);
    champ_reponse->installEventFilter(this);
    dernier_champ_a_avoir_le_focus = champ_question;
    champ_question_regexp = new QLineEdit;
    champ_reponse_regexp = new QLineEdit;
    widget_tags = new Widget_tags(liste_tags);
    liste_deroulante_difficulte = new QComboBox;
    liste_deroulante_difficulte->addItems(QStringList() << trUtf8("Très facile")
                                                        << trUtf8("Facile")
                                                        << trUtf8("Moyen")
                                                        << trUtf8("Difficile")
                                                        << trUtf8("Très difficile"));
    liste_deroulante_difficulte->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum));
    connect(champ_question,              SIGNAL(textChanged(QString)),         this, SLOT(modifie_question()));
    connect(champ_reponse,               SIGNAL(textChanged(QString)),         this, SLOT(modifie_reponse()));
    connect(champ_question_regexp,       SIGNAL(textChanged(QString)),         this, SLOT(modifie_question_regexp()));
    connect(champ_reponse_regexp,        SIGNAL(textChanged(QString)),         this, SLOT(modifie_reponse_regexp()));
    connect(widget_tags->champ_tags,     SIGNAL(textChanged(QString)),         this, SLOT(modifie_tags()));
    connect(liste_deroulante_difficulte, SIGNAL(currentIndexChanged(int)),     this, SLOT(modifie_difficulte()));

    bouton_question_regexp_calculer  = new QPushButton(trUtf8("Calculer"));
    bouton_reponse_regexp_calculer   = new QPushButton(trUtf8("Calculer"));
    bouton_difficulte_par_defaut     = new QPushButton(trUtf8("Défaut"));
    bouton_tags_par_defaut           = new QPushButton(trUtf8("Défaut"));
    bouton_ajouter_tags_par_defaut   = new QPushButton(trUtf8("Ajouter défaut"));
    connect(bouton_question_regexp_calculer, SIGNAL(clicked()),                 this, SLOT(question_regexp_calculer()));
    connect(bouton_reponse_regexp_calculer,  SIGNAL(clicked()),                 this, SLOT(reponse_regexp_calculer()));
    connect(bouton_difficulte_par_defaut,    SIGNAL(clicked()),                 this, SLOT(difficulte_par_defaut()));
    connect(bouton_tags_par_defaut,          SIGNAL(clicked()),                 this, SLOT(tags_par_defaut()));
    connect(bouton_ajouter_tags_par_defaut,  SIGNAL(clicked()),                 this, SLOT(ajouter_tags_par_defaut()));
    connect(widget_tags,                     SIGNAL(liste_tags_afficher(bool)), this, SLOT(liste_tags_afficher()));

    bouton_question_reinitialiser        = new QPushButton(trUtf8("<"));
    bouton_reponse_reinitialiser         = new QPushButton(trUtf8("<"));
    bouton_question_regexp_reinitialiser = new QPushButton(trUtf8("<"));
    bouton_reponse_regexp_reinitialiser  = new QPushButton(trUtf8("<"));
    bouton_difficulte_reinitialiser      = new QPushButton(trUtf8("<"));
    bouton_tags_reinitialiser            = new QPushButton(trUtf8("<"));
    connect(bouton_question_reinitialiser,        SIGNAL(clicked()), this, SLOT(question_reinitialiser()));
    connect(bouton_reponse_reinitialiser,         SIGNAL(clicked()), this, SLOT(reponse_reinitialiser()));
    connect(bouton_question_regexp_reinitialiser, SIGNAL(clicked()), this, SLOT(question_regexp_reinitialiser()));
    connect(bouton_reponse_regexp_reinitialiser,  SIGNAL(clicked()), this, SLOT(reponse_regexp_reinitialiser()));
    connect(bouton_difficulte_reinitialiser,      SIGNAL(clicked()), this, SLOT(difficulte_reinitialiser()));
    connect(bouton_tags_reinitialiser,            SIGNAL(clicked()), this, SLOT(tags_reinitialiser()));

    bouton_plus_d_option     = new QPushButton(trUtf8("Plus d'options"));
    bouton_plus_d_option->setCheckable(true);
    bouton_valider           = new QPushButton("");// (!) Remarque : le texte est calculé au moment de l'execution
    bouton_annuler           = new QPushButton(trUtf8("Annuler"));
    bouton_reinitialiser     = new QPushButton(trUtf8("Réinitialiser"));//efface tous les champs
    connect(bouton_plus_d_option, SIGNAL(clicked(bool)), this, SLOT(plus_d_option_afficher(bool)));
    connect(bouton_valider,       SIGNAL(clicked()),     this, SLOT(valider()));
    connect(bouton_annuler,       SIGNAL(clicked()),     this, SLOT(reject()));
    connect(bouton_reinitialiser, SIGNAL(clicked()),     this, SLOT(reinitialiser()));

    bouton_claviers_virtuels = new QPushButton(trUtf8("C"));// TODO : Remplacer par une icone
    QMenu *menu_claviers_virtuels = new QMenu;
    for(int i = 0 ; i < c_claviers_virtuels->size() ; ++i)
    {
        claviers_virtuels.append(new Clavier_virtuel(static_cast<Clavier_virtuel*>(c_claviers_virtuels->at(i)->widget())->get_nom(),
                                                     static_cast<Clavier_virtuel*>(c_claviers_virtuels->at(i)->widget())->get_touches_clavier(),
                                                     this, Qt::Tool));
        QAction *action = menu_claviers_virtuels->addAction(claviers_virtuels.last()->get_nom());
        action->setCheckable(true);
        connect(action, SIGNAL(toggled(bool)), claviers_virtuels.last(), SLOT(setVisible(bool)));
        connect(claviers_virtuels.last(), SIGNAL(clavier_ferme(bool)), action, SLOT(setChecked(bool)));

        connect(claviers_virtuels.last(), SIGNAL(touche_appuyee(QString)), this, SLOT(ajouter_caractere_special(QString)));
    }
    bouton_claviers_virtuels->setMenu(menu_claviers_virtuels);

    QHBoxLayout *layout_question = new QHBoxLayout;
    layout_question->addWidget(champ_question);
    layout_question->addWidget(bouton_question_reinitialiser);
    QHBoxLayout *layout_reponse = new QHBoxLayout;
    layout_reponse->addWidget(champ_reponse);
    layout_reponse->addWidget(bouton_reponse_reinitialiser);
    QHBoxLayout *layout_question_regexp = new QHBoxLayout;
    layout_question_regexp->addWidget(champ_question_regexp);
    layout_question_regexp->addWidget(bouton_question_regexp_calculer);
    layout_question_regexp->addWidget(bouton_question_regexp_reinitialiser);
    QHBoxLayout *layout_reponse_regexp = new QHBoxLayout;
    layout_reponse_regexp->addWidget(champ_reponse_regexp);
    layout_reponse_regexp->addWidget(bouton_reponse_regexp_calculer);
    layout_reponse_regexp->addWidget(bouton_reponse_regexp_reinitialiser);
    QHBoxLayout *layout_difficulte = new QHBoxLayout;
    layout_difficulte->addWidget(liste_deroulante_difficulte);
    layout_difficulte->addWidget(bouton_difficulte_par_defaut);
    layout_difficulte->addWidget(bouton_difficulte_reinitialiser);
    QHBoxLayout *layout_tags = new QHBoxLayout;
    layout_tags->addWidget(widget_tags);
    widget_tags->layout_champ_tags->insertWidget(1, bouton_tags_par_defaut);
    widget_tags->layout_champ_tags->insertWidget(2, bouton_ajouter_tags_par_defaut);
    widget_tags->layout_champ_tags->addWidget(bouton_tags_reinitialiser);

    QFormLayout *layout_champs_options = new QFormLayout;
    layout_champs_options->addRow(trUtf8("Regexp de la question : "), layout_question_regexp);
    layout_champs_options->addRow(trUtf8("Regexp de la réponse : "), layout_reponse_regexp);
    layout_champs_options->addRow(trUtf8("Difficulté : "), layout_difficulte);
    layout_champs_options->addRow(trUtf8("Tags : "), widget_tags);
    layout_champs_options->setMargin(0);

    cadre_option = new QWidget;
    cadre_option->setLayout(layout_champs_options);

    QHBoxLayout *layout_boutons = new QHBoxLayout;
    layout_boutons->addStretch();
    layout_boutons->addWidget(bouton_plus_d_option);
    layout_boutons->addSpacing(30);
    layout_boutons->addWidget(bouton_valider);
    layout_boutons->addWidget(bouton_annuler);
    layout_boutons->addWidget(bouton_reinitialiser);
    layout_boutons->addWidget(bouton_claviers_virtuels);

    QFormLayout *layout_champs = new QFormLayout;
    layout_champs->addRow(trUtf8("Question : "), layout_question);
    layout_champs->addRow(trUtf8("Réponse : "), layout_reponse);
    layout_champs->addRow(cadre_option);
    layout_champs->addRow(layout_boutons);

    setLayout(layout_champs);

    setMinimumWidth(750);
    widget_tags->afficher_liste_tags(false);
    plus_d_option_afficher(false);//on cache les options "more" et redimentionne la fenetre

    bouton_valider->setDefault(true);
}

QString Dialog_ajouter_question::calculer_regexp(QString pattern)
{
    // Le contenu des parentèses est facultatif
    QRegExp regexp_parenteses("\\((.*)\\)");
    regexp_parenteses.setMinimal(true);
    pattern.replace(regexp_parenteses, "(?:\\(\\1\\))?");

    // On peut utiliser des abréviations
    QStringList liste_abreviations;
    liste_abreviations << "jdm|jemandem"
                       << "jdn|jemanden"
                       << "etw|etwas"
                       << "qqn|quelqu'un"
                       << "qqch|quelque chose"
                       << "ds|dans"
                       << "sbd|somebody"
                       << "swh|somewhere"
                       << "sth|something";
    for(int i = 0 ; i < liste_abreviations.size() ; ++i)
    {
        QRegExp regexp_abreviations("\\b(" + liste_abreviations.at(i) + ")\\b");
        pattern.replace(regexp_abreviations, "(?:" + liste_abreviations.at(i) + ")");
    }

    // Les déterminant sont facultatifs
    QRegExp regexp_determinant("^(der|die|das|ein|eine|le|la|les|l'|un|une|des|the|a|an|to) ");

    // Autorisation de plusieurs mots
    QStringList liste_possibilitees = pattern.split(", ", QString::SkipEmptyParts);
    pattern = "(?:";
    for(int i = 0 ; i < liste_possibilitees.size() ; ++i)
    {
        liste_possibilitees[i].replace(regexp_determinant, "(?:\\1 )?");
        pattern += liste_possibilitees.at(i) +"|";
    }
    pattern.remove(pattern.lastIndexOf("|"), 1);
    pattern += ")";

    // Pour finir, on délimite le debut et la fin
    pattern = "^(?:(?:^|(?!^), )" + pattern + ")+$";

    return pattern;
}

void Dialog_ajouter_question::liste_tags_afficher()
{
    // (!) Remarque : la taille de cadre_option est déjà actualisée(par widget_tags) donc on peut directement appeler setFixedHeight
    setFixedHeight(sizeHint().height());
}

void Dialog_ajouter_question::plus_d_option_afficher(bool enfonce)
{
    cadre_option->setVisible(enfonce);
    setFixedHeight(sizeHint().height());
}

void Dialog_ajouter_question::question_reinitialiser()
{
    champ_question->setText(question_a_ajouter->question);
    bouton_question_reinitialiser->setEnabled(false);
    champ_question->setFocus();
    modifie();
}

void Dialog_ajouter_question::reponse_reinitialiser()
{
    champ_reponse->setText(question_a_ajouter->reponse);
    bouton_reponse_reinitialiser->setEnabled(false);
    champ_reponse->setFocus();
    modifie();
}

void Dialog_ajouter_question::question_regexp_reinitialiser()
{
    champ_question_regexp->setText(question_a_ajouter->question_regexp.pattern());
    bouton_question_regexp_reinitialiser->setEnabled(false);
    champ_question_regexp->setFocus();
    modifie();
}

void Dialog_ajouter_question::reponse_regexp_reinitialiser()
{
    champ_reponse_regexp->setText(question_a_ajouter->reponse_regexp.pattern());
    bouton_reponse_regexp_reinitialiser->setEnabled(false);
    champ_reponse_regexp->setFocus();
    modifie();
}

void Dialog_ajouter_question::difficulte_reinitialiser()
{
    liste_deroulante_difficulte->setCurrentIndex(question_a_ajouter->difficulte);
    bouton_difficulte_reinitialiser->setEnabled(false);
    liste_deroulante_difficulte->setFocus();
    modifie();
}

void Dialog_ajouter_question::tags_reinitialiser()
{
    if(question_a_ajouter->date_ajout.isNull())
    {
        question_a_ajouter->tags = *liste_tags_defaut;
    }
    widget_tags->from_string_list(question_a_ajouter->tags);//widget_tags s'occuppe tous seul d'actualiser le cochage

    bouton_tags_reinitialiser->setEnabled(false);
    widget_tags->champ_tags->setFocus();
    modifie();
}

void Dialog_ajouter_question::modifie()
{
    if(bouton_question_reinitialiser->isEnabled() ||
       bouton_reponse_reinitialiser->isEnabled() ||
       bouton_question_regexp_reinitialiser->isEnabled() ||
       bouton_reponse_regexp_reinitialiser->isEnabled() ||
       bouton_difficulte_reinitialiser->isEnabled() ||
       bouton_tags_reinitialiser->isEnabled())
    {
        bouton_reinitialiser->setEnabled(true);
    }
    else
    {
        bouton_reinitialiser->setEnabled(false);
    }
}

void Dialog_ajouter_question::modifie_question()
{
    bouton_question_reinitialiser->setEnabled(true);
    modifie();
}

void Dialog_ajouter_question::modifie_reponse()
{
    bouton_reponse_reinitialiser->setEnabled(true);
    modifie();
}

void Dialog_ajouter_question::modifie_question_regexp()
{
    bouton_question_regexp_reinitialiser->setEnabled(true);
    modifie();
}

void Dialog_ajouter_question::modifie_reponse_regexp()
{
    bouton_reponse_regexp_reinitialiser->setEnabled(true);
    modifie();
}

void Dialog_ajouter_question::modifie_difficulte()
{
    bouton_difficulte_reinitialiser->setEnabled(true);
    modifie();
}

void Dialog_ajouter_question::modifie_tags()
{
    bouton_tags_reinitialiser->setEnabled(true);
    modifie();
}

void  Dialog_ajouter_question::reinitialiser()
{
    question_reinitialiser();
    reponse_reinitialiser();
    question_regexp_reinitialiser();
    reponse_regexp_reinitialiser();
    difficulte_reinitialiser();
    tags_reinitialiser();
    // pas besoin d'appeler "modifie()" car il est appelé par les fonctions précédentes
}

void Dialog_ajouter_question::question_regexp_calculer()
{
    champ_question_regexp->setText(calculer_regexp(champ_question->text()));
}

void Dialog_ajouter_question::reponse_regexp_calculer()
{
    champ_reponse_regexp->setText(calculer_regexp(champ_reponse->text()));
}

void Dialog_ajouter_question::difficulte_par_defaut()
{
    liste_deroulante_difficulte->setCurrentIndex(QSettings().value("Autre/difficulte_defaut", DEFAUT_DIFFICULTE_DEFAUT).toInt());
}

void Dialog_ajouter_question::tags_par_defaut()
{
    widget_tags->champ_tags->clear();
    ajouter_tags_par_defaut();
}

void Dialog_ajouter_question::ajouter_tags_par_defaut()
{
    QStringList chaines_tags = widget_tags->to_string_list();
    for(int i = 0 ; i < liste_tags_defaut->size() ; ++i)
    {
        if(!chaines_tags.contains(liste_tags_defaut->at(i)))
        {
            chaines_tags << liste_tags_defaut->at(i);//si le tag n'existe pas
        }
    }
    widget_tags->from_string_list(chaines_tags);
}

void  Dialog_ajouter_question::valider()
{
    bool valide = true;
    // Vérification(à l'envers pour le focus)
    if(champ_reponse->text().isEmpty())
    {
        champ_reponse->setFocus();
        valide = false;
    }
    if(champ_question->text().isEmpty())
    {
        champ_question->setFocus();
        valide = false;
    }

    if(valide)
    {
        // Création/modification de la question
        question_a_ajouter->question = champ_question->text();
        question_a_ajouter->reponse = champ_reponse->text();
        if(bouton_question_regexp_reinitialiser->isEnabled() || champ_question_regexp->text().isEmpty())//Si on n'a pas modifier la regexp ou que l'on à laissé un champ vide
        {
            question_regexp_calculer();
        }
        if(bouton_reponse_regexp_reinitialiser->isEnabled() || champ_reponse_regexp->text().isEmpty())
        {
            reponse_regexp_calculer();
        }
        question_a_ajouter->question_regexp.setPattern(champ_question_regexp->text());
        question_a_ajouter->reponse_regexp.setPattern(champ_reponse_regexp->text());

        if(question_a_ajouter->date_ajout.isNull())
        {
            question_a_ajouter->date_ajout = QDateTime::currentDateTime();
        }
        question_a_ajouter->difficulte = liste_deroulante_difficulte->currentIndex();
        // Rajoute les tags à la question
        question_a_ajouter->tags = widget_tags->to_string_list();// malgrès que la chaine soit passée dans le validateur, si l'utilisateur à laissé les champs vides on ne doit pas prendre en compte les blancs
        // Rajoute les tags à la liste
        for(int i = 0 ; i < question_a_ajouter->tags.size() ; ++i)
        {
            for(int j = i + 1 ; j < question_a_ajouter->tags.size() ; ++j)// si la question est en double dans les questions suivantes
            {
                if(question_a_ajouter->tags.at(i) == question_a_ajouter->tags.at(j))
                {
                    question_a_ajouter->tags.removeAt(j);
                    j--;// la liste comporte un élément de moins
                }
            }
        }
        for(int i = 0 ; i < question_a_ajouter->tags.size() ; ++i)
        {
            bool existe_deja = false;
            for(int j = 0 ; j < liste_tags->size() ; ++j)
            {
                if(question_a_ajouter->tags.at(i) == liste_tags->at(j))
                {
                    existe_deja = true;
                    break;
                }
            }
            if(!existe_deja)
            {
                widget_tags->ajouter_tag(question_a_ajouter->tags.at(i));
            }
        }
        // Toutes les cases sont décochées au momment ou on appelle réinitialiser

        accept();
    }
}

void Dialog_ajouter_question::ajouter_caractere_special(QString caractere)
{
    static_cast<QLineEdit*>(dernier_champ_a_avoir_le_focus)->setText(static_cast<QLineEdit*>(dernier_champ_a_avoir_le_focus)->text() + caractere);
}

bool Dialog_ajouter_question::eventFilter(QObject *object, QEvent *event)
{
    if(object == champ_question)
    {
        if(event->type() == QEvent::FocusIn)
        {
            dernier_champ_a_avoir_le_focus = champ_question;
        }
    }
    else if(object == champ_reponse)
    {
        if(event->type() == QEvent::FocusIn)
        {
            dernier_champ_a_avoir_le_focus = champ_reponse;
        }
    }
    return QDialog::eventFilter(object, event);
}

int Dialog_ajouter_question::exec(Question *c_question_a_ajouter)
{
    question_a_ajouter = c_question_a_ajouter;
    if(question_a_ajouter->date_ajout.isNull())// si la question n'a jamias été ajoutée
    {
        setWindowTitle(trUtf8("Ajouter une question"));// elle n'existe pas encore donc on l'ajoute
    }
    else
    {
        setWindowTitle(trUtf8("Modifier une question"));
    }
    if(question_a_ajouter->date_ajout.isNull())
    {
        bouton_valider->setText("Ajouter");
    }
    else
    {
        bouton_valider->setText("Modifier");
    }
    reinitialiser();
    champ_question->setFocus();
    return QDialog::exec();
}
