/*
  Copyright (C) 2009 Pot Etienne

  This file is part of Vocab++.

  Vocab++ is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Vocab++ is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Vocab++.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "dialog_filtrer.hpp"

Widget_filtre::Widget_filtre(int c_numero_du_fitre, QStringList *c_liste_tags)
{
    liste_tags = c_liste_tags;
    numero_du_fitre = c_numero_du_fitre;

    texte = new QLabel(trUtf8("Filtre : "));
    choix_filtre = new QComboBox;
    choix_filtre->addItems(QStringList() << trUtf8("date d'ajout")
                                         << trUtf8("date de dernière fois posée")
                                         << trUtf8("score")
                                         << trUtf8("score maximum")
                                         << trUtf8("nombre de fois posée")
                                         << trUtf8("nombre de fois réussi")
                                         << trUtf8("difficulté")
                                         << trUtf8("tags"));
    superieur_inferieur = new QComboBox;
    superieur_inferieur->setSizeAdjustPolicy(QComboBox::AdjustToContents);
    valeur = new QWidget;
    bouton_moin = new QPushButton("-");
    bouton_moin->setFixedSize(bouton_moin->sizeHint());

    connect(choix_filtre, SIGNAL(currentIndexChanged(int)), this, SLOT(choix_change(int)));
    connect(bouton_moin, SIGNAL(clicked()), this, SLOT(bouton_moin_clique()));

    QHBoxLayout *layout_choix = new QHBoxLayout;
    layout_choix->addWidget(texte);
    layout_choix->addWidget(choix_filtre);
    layout_choix->addWidget(superieur_inferieur);
    layout_choix->addStretch();

    layout_filtre = new QVBoxLayout;
    layout_filtre->addLayout(layout_choix);
    choix_change(0);

    QHBoxLayout *layout_principal = new QHBoxLayout;
    layout_principal->addLayout(layout_filtre);
    layout_principal->addWidget(bouton_moin);

    setLayout(layout_principal);
}

void Widget_filtre::choix_change(int choix)
{
    layout_filtre->removeWidget(valeur);
    delete valeur;
    superieur_inferieur->clear();
    if(choix == FILTRE_DATE_AJOUT || choix == FILTRE_DATE_DERNIERE_FOIS_POSEE)
    {
        superieur_inferieur->addItems(QStringList() << trUtf8("Avant le")
                                                    << trUtf8("Après le"));
        valeur = new QDateTimeEdit(QDateTime::currentDateTime());
    }
    else if(choix == FILTRE_SCORE || choix == FILTRE_SCORE_MAXIMUM || choix == FILTRE_NOMBRE_DE_FOIS_POSEE || choix == FILTRE_NOMBRE_DE_FOIS_REUSSI)
    {
        superieur_inferieur->addItems(QStringList() << trUtf8("Inferieur à")
                                                    << trUtf8("Égal à")
                                                    << trUtf8("Superieur à"));
        valeur = new Widget_int_edit;
        static_cast<Widget_int_edit*>(valeur)->set_maximum(100);
    }
    else if(choix == FILTRE_DIFFICULTEE)
    {
        superieur_inferieur->addItems(QStringList() << trUtf8("Plus facile que")
                                                    << trUtf8("De difficulté")
                                                    << trUtf8("Plus difficile que"));
        valeur = new QComboBox;
        static_cast<QComboBox*>(valeur)->addItems(QStringList() << trUtf8("Très facile")
                                                                << trUtf8("Facile")
                                                                << trUtf8("Moyen")
                                                                << trUtf8("Difficile")
                                                                << trUtf8("Très difficile"));
        static_cast<QComboBox*>(valeur)->setCurrentIndex(QSettings().value("Autre/difficulte_defaut", DEFAUT_DIFFICULTE_DEFAUT).toInt());
    }
    else if(choix == FILTRE_TAGS)
    {
        superieur_inferieur->addItems(QStringList() << trUtf8("Contenant")
                                                    << trUtf8("Ne contenant pas"));
        valeur = new Widget_tags(liste_tags);
        static_cast<Widget_tags*>(valeur)->afficher_liste_tags(false);
    }
    layout_filtre->addWidget(valeur);
}

void Widget_filtre::bouton_moin_clique()
{
    delete texte;
    delete choix_filtre;
    delete superieur_inferieur;// Contient/Ne contient pas
    delete valeur;
    delete bouton_moin;

    emit enlever_widget(numero_du_fitre);
}

Dialog_filtrer::Dialog_filtrer(QList<Filtre> *c_liste_filtres, QStringList *c_liste_tags, QWidget *parent) : QDialog(parent)
{
    liste_tags = c_liste_tags;
    liste_filtres = c_liste_filtres;

    bouton_plus = new QPushButton("+");
    connect(bouton_plus, SIGNAL(clicked()), this, SLOT(ajouter_filtre()));

    bouton_valider = new QPushButton(trUtf8("Filtrer"));
    bouton_annuler = new QPushButton(trUtf8("Annuler"));
    connect(bouton_valider, SIGNAL(clicked()), this, SLOT(valider()));
    connect(bouton_annuler, SIGNAL(clicked()), this, SLOT(reject()));

    QHBoxLayout *layout_bouton = new QHBoxLayout;
    layout_bouton->addStretch();
    layout_bouton->addWidget(bouton_plus);
    layout_bouton->addSpacing(30);
    layout_bouton->addWidget(bouton_valider);
    layout_bouton->addWidget(bouton_annuler);

    layout_principal = new QVBoxLayout;
    layout_principal->addLayout(layout_bouton);

    setLayout(layout_principal);
    setMinimumWidth(500);

    bouton_valider->setDefault(true);

    reinitialiser();
}

void Dialog_filtrer::reinitialiser()
{
    // TODO : Suppression des filtres courants
    for(int i = 0 ; i < liste_filtres->size() ; ++i)
    {
        ajouter_filtre();
        liste_widget.last()->choix_filtre->setCurrentIndex(liste_filtres->at(i).type_de_filtre);
        liste_widget.last()->superieur_inferieur->setCurrentIndex(liste_filtres->at(i).mode_de_filtrage);
        if(liste_filtres->at(i).type_de_filtre == FILTRE_DATE_AJOUT ||
           liste_filtres->at(i).type_de_filtre == FILTRE_DATE_DERNIERE_FOIS_POSEE)
        {
            static_cast<QDateTimeEdit*>(liste_widget.last()->valeur)->setDateTime(liste_filtres->at(i).valeur.toDateTime());
        }
        else if(liste_filtres->at(i).type_de_filtre == FILTRE_SCORE ||
                liste_filtres->at(i).type_de_filtre == FILTRE_SCORE_MAXIMUM ||
                liste_filtres->at(i).type_de_filtre == FILTRE_NOMBRE_DE_FOIS_POSEE ||
                liste_filtres->at(i).type_de_filtre == FILTRE_NOMBRE_DE_FOIS_REUSSI)
        {
            static_cast<Widget_int_edit*>(liste_widget.last()->valeur)->set_valeur(liste_filtres->at(i).valeur.toInt());
        }
        else if(liste_filtres->at(i).type_de_filtre == FILTRE_DIFFICULTEE)
        {
            static_cast<QComboBox*>(liste_widget.last()->valeur)->setCurrentIndex(liste_filtres->at(i).valeur.toInt());
        }
        else if(liste_filtres->at(i).type_de_filtre == FILTRE_TAGS)
        {
            static_cast<Widget_tags*>(liste_widget.last()->valeur)->from_string_list(liste_filtres->at(i).valeur.toStringList());
        }
    }
}

void Dialog_filtrer::valider()
{
    liste_filtres->clear();// On efface tous les anciens filtres
    for(int i = 0 ; i < liste_widget.size() ; ++i)
    {
        liste_filtres->append(Filtre(liste_widget.at(i)->choix_filtre->currentIndex(), liste_widget.at(i)->superieur_inferieur->currentIndex()));

        if(liste_filtres->last().type_de_filtre == FILTRE_DATE_AJOUT ||
           liste_filtres->last().type_de_filtre == FILTRE_DATE_DERNIERE_FOIS_POSEE)
        {
            liste_filtres->last().valeur = static_cast<QDateTimeEdit*>(liste_widget.at(i)->valeur)->dateTime();
        }
        else if(liste_filtres->last().type_de_filtre == FILTRE_SCORE ||
                liste_filtres->last().type_de_filtre == FILTRE_SCORE_MAXIMUM ||
                liste_filtres->last().type_de_filtre == FILTRE_NOMBRE_DE_FOIS_POSEE ||
                liste_filtres->last().type_de_filtre == FILTRE_NOMBRE_DE_FOIS_REUSSI)
        {
            liste_filtres->last().valeur = static_cast<Widget_int_edit*>(liste_widget.at(i)->valeur)->valeur();
        }
        else if(liste_filtres->last().type_de_filtre == FILTRE_DIFFICULTEE)
        {
            liste_filtres->last().valeur = static_cast<QComboBox*>(liste_widget.at(i)->valeur)->currentIndex();
        }
        else if(liste_filtres->last().type_de_filtre == FILTRE_TAGS)
        {
            liste_filtres->last().valeur = static_cast<Widget_tags*>(liste_widget.at(i)->valeur)->to_string_list();
        }
    }
    accept();
}

void Dialog_filtrer::ajouter_filtre()
{
    liste_widget.append(new Widget_filtre(liste_widget.size(), liste_tags));
    layout_principal->insertWidget(liste_widget.size()-1, liste_widget.last());
    connect(liste_widget.last(), SIGNAL(enlever_widget(int)), this, SLOT(enlever_filtre(int)));
}

void Dialog_filtrer::enlever_filtre(int numero)
{
    layout_principal->removeWidget(liste_widget.at(numero));
    delete liste_widget.at(numero);
    liste_widget.removeAt(numero);
    for(int i = numero ; i < liste_widget.size() ; ++i)
    {
        liste_widget.at(i)->numero_du_fitre--;// tous ceux après celui que l'on a supprimé sont décalés
    }
}
