/*
  Copyright (C) 2009 Pot Etienne

  This file is part of Vocab++.

  Vocab++ is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Vocab++ is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Vocab++.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DIALOG_FILTRER_HPP
#define DIALOG_FILTRER_HPP

#include <QtGui>
#include <iostream>

#include "question.hpp"
#include "valeurs_par_defaut.hpp"
#include "widget_tags.hpp"

enum type_de_filtre
{
    FILTRE_DATE_AJOUT = 0,
    FILTRE_DATE_DERNIERE_FOIS_POSEE = 1,
    FILTRE_SCORE = 2,
    FILTRE_SCORE_MAXIMUM = 3,
    FILTRE_NOMBRE_DE_FOIS_POSEE = 4,
    FILTRE_NOMBRE_DE_FOIS_REUSSI = 5,
    FILTRE_DIFFICULTEE = 6,
    FILTRE_TAGS = 7
};

enum mode_filtre_date
{
    MODE_FILTRE_AVANT_LE = 0,
    MODE_FILTRE_APRES_LE = 1
};

enum mode_filtre_int
{
    MODE_FILTRE_INFERIEUR_A = 0,
    MODE_FILTRE_EGAL_A = 1,
    MODE_FILTRE_SUPPERIEUR_A = 2
};

enum mode_filtre_difficultee
{
    MODE_FILTRE_PLUS_FACILE_QUE = 0,
    MODE_FILTRE_DE_DIFFICULTEE = 1,
    MODE_FILTRE_PLUS_DIFICILE_QUE = 2
};

enum mode_filtre_tags
{
    MODE_FILTRE_CONTENANT = 0,
    MODE_FILTRE_NE_CONTENANT_PAS = 1,
};

class Filtre
{
public:
    Filtre(int c_type_de_filtre, int c_mode_de_filtrage) : type_de_filtre(c_type_de_filtre), mode_de_filtrage(c_mode_de_filtrage) {}
    int type_de_filtre;
    int mode_de_filtrage;
    QVariant valeur;
};

class Widget_int_edit : public QWidget
{
public:
    Widget_int_edit()
    {
        slider = new QSlider(Qt::Horizontal);
        spinbox = new QSpinBox;
        connect(spinbox, SIGNAL(valueChanged(int)), slider, SLOT(setValue(int)));
        connect(slider, SIGNAL(valueChanged(int)), spinbox, SLOT(setValue(int)));

        QHBoxLayout *layout_principal = new QHBoxLayout;
        layout_principal->addWidget(slider);
        layout_principal->addWidget(spinbox);

        setLayout(layout_principal);
    }

    int valeur()
    {
        return slider->value();
    }

    void set_valeur(int nouvelle_valeur)
    {
        slider->setValue(nouvelle_valeur);
        // La connection se charge de modifier le spinbox
    }

    void set_maximum(int maximum)
    {
        slider->setMaximum(maximum);
        spinbox->setMaximum(maximum);
    }

    void set_minimum(int minimum)
    {
        slider->setMinimum(minimum);
        spinbox->setMinimum(minimum);
    }


private:
    QSlider *slider;
    QSpinBox *spinbox;
};

class Widget_filtre : public QWidget
{
    Q_OBJECT

public:
    Widget_filtre(int c_numero_du_fitre, QStringList *c_liste_tags);

    QLabel *texte;
    QComboBox *choix_filtre;
    QComboBox *superieur_inferieur;// Contient/Ne contient pas
    QWidget *valeur;
    QPushButton *bouton_moin;
    int numero_du_fitre;

private slots:
    void bouton_moin_clique();
    void choix_change(int);

signals:
    void enlever_widget(int);

private:
    QStringList *liste_tags;
    QVBoxLayout *layout_filtre;
};

class Dialog_filtrer : public QDialog
{
    Q_OBJECT

public:
    Dialog_filtrer(QList<Filtre> *c_liste_filtres, QStringList *c_liste_tags, QWidget *parent = 0);

private slots:
    void reinitialiser();
    void valider();
    void ajouter_filtre();
    void enlever_filtre(int);

private:
    QStringList *liste_tags;
    QList<Filtre> *liste_filtres;

    QList<Widget_filtre*> liste_widget;

    QPushButton *bouton_plus;

    QPushButton *bouton_valider;
    QPushButton *bouton_annuler;

    QVBoxLayout *layout_principal;
};

#endif
