/*
  Copyright (C) 2009 Pot Etienne

  This file is part of Vocab++.

  Vocab++ is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Vocab++ is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Vocab++.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "dialog_gerer_claviers_virtuels.hpp"

Dialog_gerer_claviers_virtuels::Dialog_gerer_claviers_virtuels(QList<QDockWidget*> *c_claviers_virtuels, QWidget *parent) : QDialog(parent)
{
    setWindowTitle(trUtf8("Gerer les  claviers virtuels"));

    layout_liste_claviers_virtuels = new QVBoxLayout;
    layout_liste_claviers_virtuels->addStretch();

    mapper_modifier = new QSignalMapper(this);
    mapper_supprimer = new QSignalMapper(this);
    connect(mapper_modifier, SIGNAL(mapped(QWidget*)), this, SLOT(modifier_clavier(QWidget*)));
    connect(mapper_supprimer, SIGNAL(mapped(QWidget*)), this, SLOT(supprimer_clavier(QWidget*)));

    docks_claviers_virtuels = c_claviers_virtuels;

    cadre_liste = new QFrame;
    cadre_liste->setFrameShape(QFrame::StyledPanel);
    cadre_liste->setFrameShadow(QFrame::Raised);
    texte_liste = new QLabel(trUtf8("Liste des claviers virtuels"));
    aire_liste_claviers = new QScrollArea;
    aire_liste_claviers->setFixedHeight(150);
    widget_liste_claviers = new QWidget;
    widget_liste_claviers->setLayout(layout_liste_claviers_virtuels);
    aire_liste_claviers->setWidget(widget_liste_claviers);
    aire_liste_claviers->setWidgetResizable(true);
    bouton_ajouter = new QPushButton(trUtf8("Nouveau un clavier"));
    connect(bouton_ajouter, SIGNAL(clicked()), this, SLOT(nouveau_clavier()));

    QHBoxLayout *layout_bouton_ajouter = new QHBoxLayout;
    layout_bouton_ajouter->addStretch();
    layout_bouton_ajouter->addWidget(bouton_ajouter);

    QVBoxLayout *layout_liste_claviers_virtuels = new QVBoxLayout;
    layout_liste_claviers_virtuels->addWidget(texte_liste);
    layout_liste_claviers_virtuels->addWidget(aire_liste_claviers);
    layout_liste_claviers_virtuels->addLayout(layout_bouton_ajouter);

    cadre_liste->setLayout(layout_liste_claviers_virtuels);

    cadre_edition = new QFrame;
    cadre_edition->setFrameShape(QFrame::StyledPanel);
    cadre_edition->setFrameShadow(QFrame::Raised);
    champ_nom = new QLineEdit;
    champ_touche = new QTextEdit;
    champ_touche->setAcceptRichText(false);
    champ_touche->setFixedHeight(150);
    bouton_reinitialiser_champs = new QPushButton(trUtf8("Réinitialiser"));
    bouton_modifier = new QPushButton;
    connect(bouton_reinitialiser_champs, SIGNAL(clicked()), this, SLOT(reinitialiser_modification()));
    connect(bouton_modifier, SIGNAL(clicked()), this, SLOT(valider_modification()));

    QHBoxLayout *layout_bouton_modifier = new QHBoxLayout;
    layout_bouton_modifier->addStretch();
    layout_bouton_modifier->addWidget(bouton_reinitialiser_champs);
    layout_bouton_modifier->addWidget(bouton_modifier);

    QVBoxLayout *layout_edition = new QVBoxLayout;
    layout_edition->addWidget(champ_nom);
    layout_edition->addWidget(champ_touche);
    layout_edition->addLayout(layout_bouton_modifier);

    cadre_edition->setLayout(layout_edition);
    cadre_edition->hide();

    bouton_defaut = new QPushButton(trUtf8("Defaut"));
    bouton_reinitialiser = new QPushButton(trUtf8("Réinitialiser"));
    bouton_valider = new QPushButton(trUtf8("Valider"));
    bouton_annuler = new QPushButton(trUtf8("Annuler"));
    connect(bouton_defaut, SIGNAL(clicked()), this, SLOT(defaut()));
    connect(bouton_reinitialiser, SIGNAL(clicked()), this, SLOT(reinitialiser()));
    connect(bouton_valider, SIGNAL(clicked()), this, SLOT(valider()));
    connect(bouton_annuler, SIGNAL(clicked()), this, SLOT(reject()));

    QHBoxLayout *layout_bouton = new QHBoxLayout;
    layout_bouton->addWidget(bouton_defaut);
    layout_bouton->addStretch();
    layout_bouton->addWidget(bouton_reinitialiser);
    layout_bouton->addSpacing(20);
    layout_bouton->addWidget(bouton_valider);
    layout_bouton->addWidget(bouton_annuler);

    QVBoxLayout *layout_principal = new QVBoxLayout;
    layout_principal->addWidget(cadre_liste);
    layout_principal->addWidget(cadre_edition);
    layout_principal->addLayout(layout_bouton);

    setLayout(layout_principal);
    setFixedHeight(sizeHint().height());

    reinitialiser();
}

void Dialog_gerer_claviers_virtuels::modifier_clavier(QWidget *widget)
{
    clavier_actuel = static_cast<Widget_edition_clavier_virtuel*>(widget)->contenu;
    champ_nom->setText(clavier_actuel->nom);
    champ_touche->setText(clavier_actuel->touches);
    bouton_modifier->setText(trUtf8("Modifier"));
    cadre_edition->show();
    setFixedHeight(sizeHint().height());
}

void Dialog_gerer_claviers_virtuels::supprimer_clavier(QWidget *widget)
{
    int place_clavier = claviers_virtuels.indexOf(static_cast<Widget_edition_clavier_virtuel*>(widget)->contenu);

    if(static_cast<Widget_edition_clavier_virtuel*>(widget)->contenu == clavier_actuel)
    {
        clavier_actuel = 0;
        cadre_edition->hide();
        setFixedHeight(sizeHint().height());
    }
    for(int i = 0 ; ligne_clavier_virtuel.size() ; ++i)
    {
        if(ligne_clavier_virtuel.at(i)->contenu == claviers_virtuels.at(place_clavier))
        {
            layout_liste_claviers_virtuels->removeWidget(ligne_clavier_virtuel.at(i));
            delete ligne_clavier_virtuel[i];
            ligne_clavier_virtuel.removeAt(i);
            break;
        }
    }

    if(place_clavier < docks_claviers_virtuels->size())
    {
        claviers_virtuels.at(place_clavier)->supprimer = true;
    }
    else
    {
        claviers_virtuels.removeAt(place_clavier);
    }
}

void Dialog_gerer_claviers_virtuels::nouveau_clavier()
{
    clavier_actuel = 0;
    champ_nom->clear();
    champ_touche->clear();
    bouton_modifier->setText(trUtf8("Ajouter"));
    cadre_edition->show();
    setFixedHeight(sizeHint().height());
}

void Dialog_gerer_claviers_virtuels::valider_modification()
{
    if(!champ_nom->text().isEmpty() && !champ_touche->toPlainText().isEmpty())
    {
        if(clavier_actuel == 0)
        {
            clavier_actuel = new Contenu_clavier;
            claviers_virtuels.append(clavier_actuel);

            ligne_clavier_virtuel.append(new Widget_edition_clavier_virtuel(claviers_virtuels.last()));
            layout_liste_claviers_virtuels->insertWidget(layout_liste_claviers_virtuels->count()-1, ligne_clavier_virtuel.last());//ajoute le widget juste avant le séparateur

            connect(ligne_clavier_virtuel.last()->bouton_modifer, SIGNAL(clicked()), mapper_modifier, SLOT(map()));
            mapper_modifier->setMapping(ligne_clavier_virtuel.last()->bouton_modifer, ligne_clavier_virtuel.last());

            connect(ligne_clavier_virtuel.last()->bouton_supprimer, SIGNAL(clicked()), mapper_supprimer, SLOT(map()));
            mapper_supprimer->setMapping(ligne_clavier_virtuel.last()->bouton_supprimer, ligne_clavier_virtuel.last());
        }
        else
        {
            clavier_actuel->modifier = true;
        }
        clavier_actuel->nom = champ_nom->text();
        clavier_actuel->touches = champ_touche->toPlainText();

        for(int i = 0 ; i < ligne_clavier_virtuel.size() ; ++i)
        {
            if(ligne_clavier_virtuel.at(i)->contenu == clavier_actuel)
            {
                ligne_clavier_virtuel.at(i)->text_nom->setText(clavier_actuel->nom);
                break;
            }
        }

        cadre_edition->hide();
        setFixedHeight(sizeHint().height());
        clavier_actuel = 0;
    }
    // TODO : Redimentionement en largeur
}

void Dialog_gerer_claviers_virtuels::reinitialiser_modification()
{
    if(clavier_actuel == 0)
    {
        champ_nom->clear();
        champ_touche->clear();
    }
    else
    {
        champ_nom->setText(clavier_actuel->nom);
        champ_touche->setText(clavier_actuel->touches);
    }
}

void Dialog_gerer_claviers_virtuels::defaut()
{
}

void Dialog_gerer_claviers_virtuels::reinitialiser()
{
    while(!ligne_clavier_virtuel.isEmpty())
    {
        delete ligne_clavier_virtuel.at(0);
        ligne_clavier_virtuel.removeFirst();
    }
    while(!claviers_virtuels.isEmpty())
    {
        delete claviers_virtuels.at(0);
        claviers_virtuels.removeFirst();
    }
    for(int i = 0 ; i < docks_claviers_virtuels->size() ; ++i)
    {
        claviers_virtuels.append(new Contenu_clavier());
        claviers_virtuels.last()->nom = static_cast<Clavier_virtuel*>(docks_claviers_virtuels->at(i)->widget())->get_nom();
        claviers_virtuels.last()->touches = static_cast<Clavier_virtuel*>(docks_claviers_virtuels->at(i)->widget())->get_touches_clavier();
        ligne_clavier_virtuel.append(new Widget_edition_clavier_virtuel(claviers_virtuels.last()));
        layout_liste_claviers_virtuels->insertWidget(layout_liste_claviers_virtuels->count()-1, ligne_clavier_virtuel.last());

        connect(ligne_clavier_virtuel.last()->bouton_modifer, SIGNAL(clicked()), mapper_modifier, SLOT(map()));
        mapper_modifier->setMapping(ligne_clavier_virtuel.last()->bouton_modifer, ligne_clavier_virtuel.last());

        connect(ligne_clavier_virtuel.last()->bouton_supprimer, SIGNAL(clicked()), mapper_supprimer, SLOT(map()));
        mapper_supprimer->setMapping(ligne_clavier_virtuel.last()->bouton_supprimer, ligne_clavier_virtuel.last());
    }
}

void Dialog_gerer_claviers_virtuels::valider()
{
    int i = 0;
    for( ; i < docks_claviers_virtuels->size() ; ++i)//On modifie d'abord les anciens claviers
    {
        if(claviers_virtuels.at(i)->supprimer)
        {
            delete (*docks_claviers_virtuels)[i];
            docks_claviers_virtuels->removeAt(i);
            delete claviers_virtuels[i];
            claviers_virtuels.removeAt(i);
            --i;
        }
        else if(claviers_virtuels.at(i)->modifier)
        {

        }
    }
    for( ; i < claviers_virtuels.size() ; ++i)//Puis on rajoute les nouveaux
    {
    }

    accept();
}
