/*
  Copyright (C) 2009 Pot Etienne

  This file is part of Vocab++.

  Vocab++ is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Vocab++ is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Vocab++.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "dialog_gerer_tags.hpp"

Dialog_gerer_tags::Dialog_gerer_tags(QStringList *c_liste_tags, QList<Tag_modifie> *c_modifications_tag, QWidget *parent) : QDialog(parent)
{
    liste_tags = c_liste_tags;
    modifications_tag = c_modifications_tag;
    model_liste_tags = new QStandardItemModel;
    connect(model_liste_tags, SIGNAL(itemChanged(QStandardItem*)), this, SLOT(ligne_editee(QStandardItem*)));

    setWindowTitle("Gerer les tags");

    cadre_liste_tags = new QFrame;
    cadre_liste_tags->setFrameShape(QFrame::StyledPanel);
    cadre_liste_tags->setFrameShadow(QFrame::Raised);
    texte_liste_tags = new QLabel(trUtf8("Liste des tags"));
    vue_liste_tags = new QListView;
    vue_liste_tags->setModel(model_liste_tags);
    vue_liste_tags->setAlternatingRowColors(true);
    vue_liste_tags->setSelectionMode(QAbstractItemView::ExtendedSelection);
    bouton_ajouter = new QPushButton(trUtf8("Ajouter"));
    bouton_supprimer = new QPushButton(trUtf8("Supprimer les tags séléctionnés"));
    connect(bouton_ajouter, SIGNAL(clicked()), this, SLOT(ajouter()));
    connect(bouton_supprimer, SIGNAL(clicked()), this, SLOT(supprimer()));

    bouton_reinitialiser = new QPushButton(trUtf8("Réinitialiser"));
    bouton_defaire = new QPushButton(trUtf8("<-"));
    bouton_refaire = new QPushButton(trUtf8("->"));
    bouton_valider = new QPushButton(trUtf8("Valider"));
    bouton_annuler = new QPushButton(trUtf8("Annuler"));
    connect(bouton_reinitialiser, SIGNAL(clicked()), this, SLOT(reinitialiser()));
    connect(bouton_defaire, SIGNAL(clicked()), this, SLOT(defaire()));
    connect(bouton_refaire, SIGNAL(clicked()), this, SLOT(refaire()));
    connect(bouton_valider, SIGNAL(clicked()), this, SLOT(accept()));
    connect(bouton_annuler, SIGNAL(clicked()), this, SLOT(reject()));

    QHBoxLayout *layout_bouton_ajouter = new QHBoxLayout;
    layout_bouton_ajouter->addStretch();
    layout_bouton_ajouter->addWidget(bouton_ajouter);
    layout_bouton_ajouter->addWidget(bouton_supprimer);

    QVBoxLayout *layout_liste_tags = new QVBoxLayout;
    layout_liste_tags->addWidget(texte_liste_tags);
    layout_liste_tags->addWidget(vue_liste_tags);
    layout_liste_tags->addLayout(layout_bouton_ajouter);

    cadre_liste_tags->setLayout(layout_liste_tags);

    QHBoxLayout *layout_bouton = new QHBoxLayout;
    layout_bouton->addStretch();
    layout_bouton->addWidget(bouton_reinitialiser);
    layout_bouton->addWidget(bouton_defaire);
    layout_bouton->addWidget(bouton_refaire);
    layout_bouton->addSpacing(30);
    layout_bouton->addWidget(bouton_valider);
    layout_bouton->addWidget(bouton_annuler);

    QVBoxLayout *layout_principal = new QVBoxLayout;
    layout_principal->addWidget(cadre_liste_tags);
    layout_principal->addLayout(layout_bouton);

    setLayout(layout_principal);

    reinitialiser();
}

void Dialog_gerer_tags::keyReleaseEvent(QKeyEvent *event)
{
    if(event->key() == Qt::Key_Delete)//si on appuie sur la touche "suppr"
    {
        supprimer();
    }
}

void Dialog_gerer_tags::ligne_editee(QStandardItem *item)
{
    modifications_tag->append(Tag_modifie(item->data().toString(), item->text()));
    item->setData(item->text());// pour la prochaine fois que l'on modifie le texte
}

void Dialog_gerer_tags::ajouter()
{
    bool ok;
    QString tags_a_ajouter = QInputDialog::getText(this, "Tags à ajouter", "Entrez les tags à ajouter séparés par une virgule", QLineEdit::Normal, QString(), &ok);
    if(ok)
    {
        QStringList liste_tags_a_ajouter = Widget_tags::valider_chaine(tags_a_ajouter).split(", ", QString::SkipEmptyParts);
        for(int i = 0 ; i < liste_tags_a_ajouter.size() ; ++i)
        {
            bool existe_deja = false;
            for(int j = 0 ; j < model_liste_tags->rowCount() ; ++j)
            {
                if(model_liste_tags->item(j)->text() == liste_tags_a_ajouter.at(i))
                {
                    existe_deja = true;
                    // TODO ? : Message d'erreur ????(rajoute dans une liste qu'il affiche à la fin ???)
                    break;
                }
            }
            if(!existe_deja)
            {
                QStandardItem *item = new QStandardItem(liste_tags_a_ajouter.at(i));
                item->setData(liste_tags_a_ajouter.at(i));// Sauvegarde du texte
                model_liste_tags->appendRow(item);
                modifications_tag->append(Tag_modifie("", liste_tags_a_ajouter.at(i)));
            }
        }
        model_liste_tags->sort(Qt::AscendingOrder);
    }
}

void Dialog_gerer_tags::supprimer()
{
    QModelIndexList liste_selections = (vue_liste_tags->selectionModel())->selectedRows();

    QList<int> lignes_a_supprimer;

    for(int i = 0 ; i < liste_selections.size() ; ++i)
    {
        modifications_tag->append(Tag_modifie(liste_selections.at(i).data(Qt::DisplayRole).toString(), ""));
        delete model_liste_tags->itemFromIndex(liste_selections.at(i));
        lignes_a_supprimer.append(liste_selections.at(i).row());
    }
    qSort(lignes_a_supprimer.begin(), lignes_a_supprimer.end(), qGreater<int>());
    for(int i = 0 ; i < lignes_a_supprimer.size() ; ++i)
    {
        model_liste_tags->removeRow(lignes_a_supprimer.at(i));
    }
}

void Dialog_gerer_tags::reinitialiser()
{
    // On vide la liste
    for(int i = 0 ; i < model_liste_tags->rowCount() ; ++i)
    {
        delete model_liste_tags->item(i);
    }
    model_liste_tags->clear();

    // Réinitialise les modifications
    modifications_tag->clear();
    bouton_defaire->setEnabled(false);
    bouton_refaire->setEnabled(false);

    // On puis on la remplis de nouveau
    for(int i = 0 ; i < liste_tags->size() ; ++i)
    {
        QStandardItem *item = new QStandardItem(liste_tags->at(i));
        item->setData(liste_tags->at(i));// Sauvegarde du texte
        model_liste_tags->appendRow(item);
    }
    model_liste_tags->sort(Qt::AscendingOrder);
}

void Dialog_gerer_tags::defaire()
{
}

void Dialog_gerer_tags::refaire()
{
}
