/*
  Copyright (C) 2009 Pot Etienne

  This file is part of Vocab++.

  Vocab++ is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Vocab++ is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Vocab++.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "dialog_reinitialiser.hpp"

Dialog_reinitialiser::Dialog_reinitialiser(QList<Question*> *c_liste_questions, QList<Question*> *c_liste_questions_filtrees, QWidget *parent) : QDialog(parent)
{
    liste_questions = c_liste_questions;
    liste_questions_filtrees = c_liste_questions_filtrees;

    // Proppriétées générales de la fenetre
    setWindowTitle(trUtf8("Réinitialiser"));

    check_box_score_reinitialise                    = new QCheckBox(trUtf8("Réinitialiser le score"));
    check_box_score_maximum_reinitialise            = new QCheckBox(trUtf8("Réinitialiser le score maximum"));
    check_box_date_derniere_fois_posee_reinitialise = new QCheckBox(trUtf8("Réinitialiser la date de dernière fois posée"));
    check_box_nombre_de_fois_posee_reinitialise     = new QCheckBox(trUtf8("Réinitialiser le nombre de fois posée"));
    check_box_nombre_de_fois_reussi_reinitialise    = new QCheckBox(trUtf8("Réinitialiser le nombre de fois réussi"));
    check_box_date_ajout                            = new QCheckBox(trUtf8("Réinitialiser la date d'ajout"));
    check_box_difficulte_reinitialise               = new QCheckBox(trUtf8("Réinitialiser la difficulté"));
    check_box_tags_reinitialise                     = new QCheckBox(trUtf8("Réinitialiser les tags"));

    settings.beginGroup("Reinitialiser");
                       check_box_score_reinitialise->setChecked(settings.value("score_reinitialise",                    DEFAUT_SCORE_REINITIALISE).toBool());
               check_box_score_maximum_reinitialise->setChecked(settings.value("score_maximum_reinitialise",            DEFAUT_SCORE_MAXIMUM_REINITIALISE).toBool());
    check_box_date_derniere_fois_posee_reinitialise->setChecked(settings.value("date_derniere_fois_posee_reinitialise", DEFAUT_DATE_DERNIERE_FOIS_POSEE_REINITIALISE).toBool());
        check_box_nombre_de_fois_posee_reinitialise->setChecked(settings.value("nombre_de_fois_posee_reinitialise",     DEFAUT_NOMBRE_DE_FOIS_POSEE_REINITIALISE).toBool());
       check_box_nombre_de_fois_reussi_reinitialise->setChecked(settings.value("nombre_de_fois_reussi_reinitialise",    DEFAUT_NOMBRE_DE_FOIS_REUSSI_REINITIALISE).toBool());
    settings.endGroup();
    connect(check_box_nombre_de_fois_posee_reinitialise, SIGNAL(toggled(bool)), check_box_nombre_de_fois_reussi_reinitialise, SLOT(setDisabled(bool)));

    connect(check_box_score_reinitialise,                    SIGNAL(toggled(bool)), this, SLOT(actualiser_bouton_tout_cocher(bool)));
    connect(check_box_score_maximum_reinitialise,            SIGNAL(toggled(bool)), this, SLOT(actualiser_bouton_tout_cocher(bool)));
    connect(check_box_date_derniere_fois_posee_reinitialise, SIGNAL(toggled(bool)), this, SLOT(actualiser_bouton_tout_cocher(bool)));
    connect(check_box_nombre_de_fois_posee_reinitialise,     SIGNAL(toggled(bool)), this, SLOT(actualiser_bouton_tout_cocher(bool)));
    connect(check_box_nombre_de_fois_reussi_reinitialise,    SIGNAL(toggled(bool)), this, SLOT(actualiser_bouton_tout_cocher(bool)));
    connect(check_box_difficulte_reinitialise,               SIGNAL(toggled(bool)), this, SLOT(actualiser_bouton_tout_cocher(bool)));
    connect(check_box_tags_reinitialise,                     SIGNAL(toggled(bool)), this, SLOT(actualiser_bouton_tout_cocher(bool)));

    check_box_liste_questions_filtrees_reinitialise = new QCheckBox(trUtf8("Réinitialiser aussi pour les questions filtrées"));

    bouton_tout_cocher = new QPushButton(trUtf8("Tout cocher"));
    bouton_tout_cocher->setCheckable(true);
    actualiser_bouton_tout_cocher(true);

    connect(bouton_tout_cocher, SIGNAL(clicked(bool)), this, SLOT(bouton_tout_cocher_clique(bool)));
    bouton_valider = new QPushButton(trUtf8("Valider"));
    bouton_annuler = new QPushButton(trUtf8("Annuler"));
    connect(bouton_valider, SIGNAL(clicked()), this, SLOT(accept()));
    connect(bouton_annuler, SIGNAL(clicked()), this, SLOT(reject()));

    QFrame *ligne_horizontale1 = new QFrame;
    ligne_horizontale1->setFrameShape(QFrame::HLine);
    QFrame *ligne_horizontale2 = new QFrame;
    ligne_horizontale2->setFrameShape(QFrame::HLine);

    //on place les elements dans la fenetre
    QHBoxLayout *layout_nombre_de_fois_reussi_reinitialise = new QHBoxLayout;
    layout_nombre_de_fois_reussi_reinitialise->addSpacing(40);
    layout_nombre_de_fois_reussi_reinitialise->addWidget(check_box_nombre_de_fois_reussi_reinitialise);

    QHBoxLayout *layout_bouton_tout_cocher = new QHBoxLayout;
    layout_bouton_tout_cocher->addStretch();
    layout_bouton_tout_cocher->addWidget(bouton_tout_cocher);

    QVBoxLayout *layout_contenu = new QVBoxLayout;
    layout_contenu->addWidget(check_box_score_reinitialise);
    layout_contenu->addWidget(check_box_score_maximum_reinitialise);
    layout_contenu->addWidget(ligne_horizontale1);
    layout_contenu->addWidget(check_box_nombre_de_fois_posee_reinitialise);
    layout_contenu->addLayout(layout_nombre_de_fois_reussi_reinitialise);
    layout_contenu->addWidget(ligne_horizontale2);
    layout_contenu->addWidget(check_box_date_derniere_fois_posee_reinitialise);
    layout_contenu->addLayout(layout_bouton_tout_cocher);

    QFrame *cadre_contenu = new QFrame;
    cadre_contenu->setFrameShape(QFrame::StyledPanel);
    cadre_contenu->setFrameShadow(QFrame::Raised);
    cadre_contenu->setLayout(layout_contenu);

    QVBoxLayout *layout_contenu_rare = new QVBoxLayout;
    layout_contenu_rare->addWidget(check_box_date_ajout);
    layout_contenu_rare->addWidget(check_box_difficulte_reinitialise);
    layout_contenu_rare->addWidget(check_box_tags_reinitialise);

    QFrame *cadre_contenu_rare = new QFrame;
    cadre_contenu_rare->setFrameShape(QFrame::StyledPanel);
    cadre_contenu_rare->setFrameShadow(QFrame::Raised);
    cadre_contenu_rare->setLayout(layout_contenu_rare);

    QHBoxLayout *layout_boutons = new QHBoxLayout;
    layout_boutons->addStretch();
    layout_boutons->addWidget(bouton_valider);
    layout_boutons->addWidget(bouton_annuler);

    QVBoxLayout *layout_principal = new QVBoxLayout;
    layout_principal->addWidget(cadre_contenu);
    layout_principal->addWidget(cadre_contenu_rare);
    layout_principal->addWidget(check_box_liste_questions_filtrees_reinitialise);
    layout_principal->addLayout(layout_boutons);

    setLayout(layout_principal);

    setFixedSize(sizeHint());
}

bool Dialog_reinitialiser::reinitialiser(QList<Question*> *liste_questions,
                                         QList<Question*> *liste_questions_filtrees,
                                         bool liste_questions_filtrees_reinitialise,
                                         bool score_reinitialise,
                                         bool score_maximum_reinitialise,
                                         bool date_derniere_fois_posee_reinitialise,
                                         bool nombre_de_fois_posee_reinitialise,
                                         bool nombre_de_fois_reussi_reinitialise,
                                         bool date_ajout_reinitialise,
                                         bool difficulte_reinitialise,
                                         bool tags_reinitialise)
{
    if(nombre_de_fois_posee_reinitialise)
    {
        nombre_de_fois_reussi_reinitialise = true;
    }
    QDateTime date_actuelle = QDateTime::currentDateTime();

    for(int i = 0 ; i < liste_questions->size() ; ++i)
    {
        if(score_reinitialise)
        {
            liste_questions->at(i)->score = 0;
        }
        if(score_maximum_reinitialise)
        {
            liste_questions->at(i)->score_recors = liste_questions->at(i)->score;
        }
        if(date_derniere_fois_posee_reinitialise)
        {
            liste_questions->at(i)->date_derniere_fois_posee = QDateTime();
        }
        if(nombre_de_fois_posee_reinitialise)
        {
            liste_questions->at(i)->nombre_de_fois_posee = 0;
        }
        if(nombre_de_fois_reussi_reinitialise)
        {
            liste_questions->at(i)->nombre_de_fois_reussi = 0;
        }
        if(date_ajout_reinitialise)
        {
            liste_questions->at(i)->date_ajout = date_actuelle;
        }
        if(difficulte_reinitialise)
        {
            liste_questions->at(i)->difficulte = QSettings().value("Autre/difficulte_defaut", DEFAUT_DIFFICULTE_DEFAUT).toInt();
        }
        if(tags_reinitialise)
        {
            liste_questions->at(i)->tags.clear();
        }
    }
    if(liste_questions_filtrees_reinitialise)
    {
        for(int i = 0 ; i < liste_questions_filtrees->size() ; ++i)
        {
            if(score_reinitialise)
            {
                liste_questions_filtrees->at(i)->score = 0;
            }
            if(score_maximum_reinitialise)
            {
                liste_questions_filtrees->at(i)->score_recors = liste_questions_filtrees->at(i)->score;
            }
            if(date_derniere_fois_posee_reinitialise)
            {
                liste_questions_filtrees->at(i)->date_derniere_fois_posee = QDateTime();
            }
            if(nombre_de_fois_posee_reinitialise)
            {
                liste_questions_filtrees->at(i)->nombre_de_fois_posee = 0;
            }
            if(nombre_de_fois_reussi_reinitialise)
            {
                liste_questions_filtrees->at(i)->nombre_de_fois_reussi = 0;
            }
            if(date_ajout_reinitialise)
            {
                liste_questions_filtrees->at(i)->date_ajout = date_actuelle;
            }
            if(difficulte_reinitialise)
            {
                liste_questions_filtrees->at(i)->difficulte = QSettings().value("Autre/difficulte_defaut", DEFAUT_DIFFICULTE_DEFAUT).toInt();
            }
            if(tags_reinitialise)
            {
                liste_questions_filtrees->at(i)->tags.clear();
            }
        }
    }
    if(score_reinitialise ||
       date_derniere_fois_posee_reinitialise ||
       nombre_de_fois_posee_reinitialise ||
       nombre_de_fois_reussi_reinitialise ||
       date_ajout_reinitialise ||
       difficulte_reinitialise ||
       tags_reinitialise)
    {
        return true;
    }
    return false;
}

bool Dialog_reinitialiser::exec()
{
    if(QDialog::exec())
    {
        settings.beginGroup("Reinitialiser");
        if(settings.value("reprendre_derniere_configuration_utilisee", DEFAUT_REPRENDRE_DERNIERE_CONFIGURATION_UTILISEE).toBool())
        {
            settings.setValue("score_reinitialise",                                       check_box_score_reinitialise->isChecked());
            settings.setValue("score_maximum_reinitialise",                       check_box_score_maximum_reinitialise->isChecked());
            settings.setValue("date_derniere_fois_posee_reinitialise", check_box_date_derniere_fois_posee_reinitialise->isChecked());
            settings.setValue("nombre_de_fois_posee_reinitialise",         check_box_nombre_de_fois_posee_reinitialise->isChecked());
            settings.setValue("nombre_de_fois_reussi_reinitialise",       check_box_nombre_de_fois_reussi_reinitialise->isChecked());
            // Pour les tags et la difficulté : il faut cocher les cases à chaques fois
        }
        settings.endGroup();

        return reinitialiser(liste_questions,
                             liste_questions_filtrees,
                             check_box_liste_questions_filtrees_reinitialise->isChecked(),
                             check_box_score_reinitialise->isChecked(),
                             check_box_score_maximum_reinitialise->isChecked(),
                             check_box_date_derniere_fois_posee_reinitialise->isChecked(),
                             check_box_nombre_de_fois_posee_reinitialise->isChecked(),
                             check_box_nombre_de_fois_reussi_reinitialise->isChecked(),
                             check_box_date_ajout->isChecked(),
                             check_box_difficulte_reinitialise->isChecked(),
                             check_box_tags_reinitialise->isChecked());
    }
    return false;
}

void Dialog_reinitialiser::actualiser_bouton_tout_cocher(bool checked)
{
    if(checked)
    {
        if(check_box_score_reinitialise->isChecked() &&
           check_box_score_maximum_reinitialise->isChecked() &&
           check_box_date_derniere_fois_posee_reinitialise->isChecked() &&
           check_box_nombre_de_fois_posee_reinitialise->isChecked())// si il est coché, check_box_nombre_de_fois_reussi_reinitialise l'est forcement
        {
            bouton_tout_cocher->setChecked(true);
            bouton_tout_cocher->setText(trUtf8("Tout décocher"));
        }
    }
    else
    {
        if(!check_box_score_reinitialise->isChecked() &&
           !check_box_score_maximum_reinitialise->isChecked() &&
           !check_box_date_derniere_fois_posee_reinitialise->isChecked() &&
           !check_box_nombre_de_fois_posee_reinitialise->isChecked() &&
           !check_box_nombre_de_fois_reussi_reinitialise->isChecked())
        {
            bouton_tout_cocher->setChecked(false);
            bouton_tout_cocher->setText(trUtf8("Tout cocher"));
        }
    }
}

void Dialog_reinitialiser::bouton_tout_cocher_clique(bool checked)
{
    check_box_score_reinitialise->setChecked(checked);
    check_box_score_maximum_reinitialise->setChecked(checked);
    check_box_date_derniere_fois_posee_reinitialise->setChecked(checked);
    check_box_nombre_de_fois_posee_reinitialise->setChecked(checked);
    check_box_nombre_de_fois_reussi_reinitialise->setChecked(checked);

    actualiser_bouton_tout_cocher(checked);
}
