/*
  Copyright (C) 2009 Pot Etienne

  This file is part of Vocab++.

  Vocab++ is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Vocab++ is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Vocab++.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "dialog_trier.hpp"

bool tris_croissant(const Question *q1, const Question *q2)
{
     return q1->valeur_tris < q2->valeur_tris;
}

bool tris_decroissant(const Question *q1, const Question *q2)
{
     return q1->valeur_tris < q2->valeur_tris;
}

Dialog_trier::Dialog_trier(QList<Question*> *c_liste_questions, QWidget *parent) : QDialog(parent)
{
    liste_questions = c_liste_questions;

    liste_deroulante_mode_de_tris = new QComboBox;
    liste_deroulante_mode_de_tris->addItems(QStringList() << trUtf8("Question")
                                                          << trUtf8("Réponse")
                                                          << trUtf8("Date d'ajout")
                                                          << trUtf8("Date de dernière fois posée")
                                                          << trUtf8("Score")
                                                          << trUtf8("Score Maximum")
                                                          << trUtf8("Nombre de fois posée")
                                                          << trUtf8("Nombre de fois réussi")
                                                          << trUtf8("Difficulté"));
    texte_croissant_decroissant = new QLabel(trUtf8("par ordre : "));
    liste_deroulante_croissant_decroissant = new QComboBox;
    liste_deroulante_croissant_decroissant->addItems(QStringList() << trUtf8("Croissant") << trUtf8("Décroissant"));

    slider_hasard = new QSlider(Qt::Horizontal);
    slider_hasard->setMaximum(100);
    texte_hasard_valeur = new QSpinBox;
    texte_hasard_valeur->setMaximum(100);
    texte_hasard = new QLabel("%");
    connect(slider_hasard, SIGNAL(valueChanged(int)), texte_hasard_valeur, SLOT(setValue(int)));
    connect(texte_hasard_valeur, SIGNAL(valueChanged(int)), slider_hasard, SLOT(setValue(int)));

    bouton_trier = new QPushButton(trUtf8("Trier"));
    bouton_annuler = new QPushButton(trUtf8("Annuler"));
    connect(bouton_trier, SIGNAL(clicked()), this, SLOT(trier()));
    connect(bouton_annuler, SIGNAL(clicked()), this, SLOT(reject()));

    QHBoxLayout *layout_tris = new QHBoxLayout;
    layout_tris->addWidget(liste_deroulante_mode_de_tris);
    layout_tris->addWidget(texte_croissant_decroissant);
    layout_tris->addWidget(liste_deroulante_croissant_decroissant);

    QHBoxLayout *layout_marge = new QHBoxLayout;

    QHBoxLayout *layout_hasard = new QHBoxLayout;
    layout_hasard->addWidget(slider_hasard);
    layout_hasard->addWidget(texte_hasard_valeur);
    layout_hasard->addWidget(texte_hasard);

    QHBoxLayout *layout_boutons = new QHBoxLayout;
    layout_boutons->addStretch();
    layout_boutons->addWidget(bouton_trier);
    layout_boutons->addWidget(bouton_annuler);

    QFormLayout *layout_principal = new QFormLayout;
    layout_principal->addRow(trUtf8("Trier par : "), layout_tris);
    layout_principal->addRow(trUtf8("Marge : "), layout_marge);
    layout_principal->addRow(trUtf8("Hasard : "), layout_hasard);
    layout_principal->addRow(layout_boutons);

    setLayout(layout_principal);
    setFixedSize(sizeHint());

    // TODO : CONCEPTION DE L'INTERFACE, DES FONCTIONNALITÉES(modes de tris) ET INTERACTIONS AVEC LA FENETRE PRINCIPALE
}

void Dialog_trier::trier()
{
    for(int i = 0 ; i < liste_questions->size() ; ++i)
    {
        switch(liste_deroulante_mode_de_tris->currentIndex())
        {
            case TRIS_QUESTION:
                liste_questions->at(i)->valeur_tris = liste_questions->at(i)->question.toLower();// "toLower "pour ne pas prendre la case en compte
            break;
            case TRIS_REPONSE:
                liste_questions->at(i)->valeur_tris = liste_questions->at(i)->reponse.toLower();
            break;
            case TRIS_DATE_AJOUT:
                liste_questions->at(i)->valeur_tris = liste_questions->at(i)->date_ajout.toString("yyyyMMddhhmm");
            break;
            case TRIS_DATE_DERNIERE_FOIS_POSEE:
                liste_questions->at(i)->valeur_tris = liste_questions->at(i)->date_derniere_fois_posee.toString("yyyyMMddhhmm");
            break;
            case TRIS_SCORE:
                liste_questions->at(i)->valeur_tris = QString::number(liste_questions->at(i)->score, 100);//on met en base 100 pour qu'il n'y ai qu'un seul carractère pour le trie par ordre alphabetique
            break;
            case TRIS_SCORE_MAXIMUM:
                liste_questions->at(i)->valeur_tris = QString::number(liste_questions->at(i)->score_recors, 100);
            break;
            case TRIS_NOMBRE_DE_FOIS_POSEE:
                liste_questions->at(i)->valeur_tris = QString::number(liste_questions->at(i)->nombre_de_fois_posee, 100);
            break;
            case TRIS_NOMBRE_DE_FOIS_REUSSI:
                liste_questions->at(i)->valeur_tris = QString::number(liste_questions->at(i)->nombre_de_fois_reussi, 100);
            break;
            case TRIS_DIFFICULTEE:
                liste_questions->at(i)->valeur_tris = liste_questions->at(i)->difficulte;
            break;
        }
    }

    qSort(liste_questions->begin(), liste_questions->end(), (QVector<bool (*)(const Question *q1, const Question *q2)>() << tris_croissant << tris_decroissant).at(liste_deroulante_croissant_decroissant->currentIndex()));

    for(int i = 0 ; i < liste_questions->size() ; ++i)
    {
        liste_questions->at(i)->valeur_tris.clear();//pour la memoire
    }
    accept();
}
