/*
  Copyright (C) 2009 Pot Etienne

  This file is part of Vocab++.

  Vocab++ is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Vocab++ is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Vocab++.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QtGui/QApplication>
#include <QTextCodec>
#include <QTranslator>
#include <QLocale>
#include <QLibraryInfo>

#include "mainwindow.hpp"

int main(int argc, char *argv[])
{
    std::cout << "Vocab++  Copyright (C) 2009  Pot Etienne" << std::endl
              << "This program comes with ABSOLUTELY NO WARRANTY" << std::endl
              << "This is free software, and you are welcome to redistribute it" << std::endl
              << "under certain conditions." << std::endl;

    // Charge l'application
    QApplication app(argc, argv);

    // Lance le programme depuis l'endoit ou est l'exécutable(et donc les ressources qui sont indiquées avec le chemin relatif)
    chdir(QCoreApplication::applicationDirPath().toStdString().c_str());

    // Charge le splash screen et l'affiche
    QSplashScreen splash(QPixmap("data/images/splash.png"));
    splash.show();
    splash.showMessage(QObject::trUtf8("Chargement de la fenetre principale"), Qt::AlignRight | Qt::AlignTop, Qt::white);

    // Définis les caractéristiques de l'application courante
    // Evite d'avoir à indiquer à chaque fois les paramètres au chargement de QSettings
    QCoreApplication::setOrganizationName("Bubonik Software");
    QCoreApplication::setOrganizationDomain("bubonik-software.org");
    QCoreApplication::setApplicationName("Vocab++");
    QCoreApplication::setApplicationVersion("0.3.4");

    // Charge le traducteur pour avoir les boites de dialogues et autres en francais
    QTranslator translator;
    translator.load(QString("qt_") + QLocale::system().name(), QLibraryInfo::location(QLibraryInfo::TranslationsPath));
    app.installTranslator(&translator);

    // Définis l'encodage(pour les accents)
    QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF-8"));//pour avoir l'encodage en UTF-8( /!\ ne marche pas avec tr(employer trUTF8 à la place))

    // Pour pouvoir utiliser le hasard
    qsrand(QTime(0,0,0).secsTo(QTime::currentTime()));

    // Lance la fenetre principale
    MainWindow window_vocab;
    window_vocab.show();

    // Centre la fenetre sur l'ecran
    QRect surface_bureau = QDesktopWidget().screenGeometry();
    window_vocab.move(QPoint(surface_bureau.width()/2  - window_vocab.width()/2,
                             surface_bureau.height()/2 - window_vocab.height()/2));//centre la fenetre sur l'ecran

    // Termine le splash screen
    splash.finish(&window_vocab);

    // Lance le programme et récupere le code d'erreur s'il existe
    return app.exec();
}
