/*
  Copyright (C) 2009 Pot Etienne

  This file is part of Vocab++.

  Vocab++ is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Vocab++ is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Vocab++.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "mainwindow.hpp"

#define FORMAT_DATE "dd.MM.yyyy-hh:mm"
#define COULEUR_SUPPRIMEE QColor(214, 54, 0)
#define COULEUR_FILTREE QColor(9, 29, 125)
#define ECRIRE_UN_FICHIER(liste_questions) \
    out << "#" << liste_questions.at(i)->question << "# #" << liste_questions.at(i)->reponse << "# "; \
    out << "#" << liste_questions.at(i)->question_regexp.pattern() << "# #" << liste_questions.at(i)->reponse_regexp.pattern() << "# "; \
    out << liste_questions.at(i)->date_ajout.toString(FORMAT_DATE) << " " << liste_questions.at(i)->date_derniere_fois_posee.toString(FORMAT_DATE) << " "; \
    out << liste_questions.at(i)->score << " " << liste_questions.at(i)->score_recors << " " << liste_questions.at(i)->nombre_de_fois_posee << " " << liste_questions.at(i)->nombre_de_fois_reussi << " "; \
    out << liste_questions.at(i)->difficulte << " "; \
    out << "#"; \
    if(!liste_questions.at(i)->tags.isEmpty()) \
    { \
        out << liste_questions.at(i)->tags.first(); \
    } \
    for(int j = 1 ; j < liste_questions.at(i)->tags.size() ; ++j) \
    { \
        out << ", " << liste_questions.at(i)->tags.at(j); \
    } \
    out << "#" << endl;

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
{
    bouton_je_me_suis_trompe_autorise = false;
    liste_couleurs.append(QColor(192, 0,   0));//rouge(raté)
    liste_couleurs.append(QColor(192, 164, 0));//jaune(moyen)
    liste_couleurs.append(QColor(142, 160, 0));//jaune-vert(assez bien)
    liste_couleurs.append(QColor(99,  160, 0));//vert-jaune(bien)
    liste_couleurs.append(QColor(40,  160, 0));//vert(très bien)
    liste_couleurs.append(QColor(17,  109, 0));//vert foncé(parfais)

    read_settings();

    creer_widget_principal();
    creer_actions();
    creer_menus();
    creer_menu_contextuel();
    creer_docks();
    creer_barres_outils();
    creer_barre_etat();

    modifier_liste(false);//on n'a pas encore eu le temps de modifier la liste actuelle

    QStringList args = QCoreApplication::arguments();// on récupere les arguments passé en parametre du nom de fichier
    if(args.size() > 1)// si l'application possede plus d'un argument
    {
        for(int i = 1 ; i < args.size() ; ++i)
        {
            charger_liste(args.at(i));
        }
        if(args.size() == 2)
        {
            // /!\ /!\ /!\ ATTENTION : Quand on va réouvrir le programme, on ouvrira sur une liste vide si l'argument est invalide
            nom_fichier_actuel = args.at(1);
        }
        else
        {
            nom_fichier_actuel = "";// si on a ouvert plusieurs fichiers, il fusionnent en un seul
        }
    }
    else// si on lance l'application normalement
    {
        nom_fichier_actuel = settings.value("Chargement/nom_fichier_actuel", DEFAUT_NOM_FICHIER_ACTUEL).toString();
        charger_liste(nom_fichier_actuel);//si fichier_actuel est vide, charge une liste vide, si fichier_actuel n'existe pas, affiche une erreur, sinon, charge le fichier
    }
    actualiser_titre();

    edition_autorisee = true;

    calculer_taille_maximum();// TODO : à revoir
    actualiser_taille(true);
    poser_nouvelle_question();//on pose une question
}

void MainWindow::read_settings()
{
    nom_repertoire_actuel = settings.value("Chargement/repertoire_defaut", DEFAUT_REPERTOIRE_DEFAUT).toString();
    repertoire_dynamique = settings.value("Chargement/repertoire_defaut_dynamique", DEFAUT_REPERTOIRE_DEFAUT_DYNAMIQUE).toBool();
}

void MainWindow::write_settings()
{
}

void MainWindow::calculer_taille_maximum()
{
    //largeur_ecran = QDesktopWidget().screenGeometry().width();
    //setMaximumWidth(largeur_ecran);
}

bool MainWindow::charger_liste(QString nom_fichier_chargement)
{
    if(nom_fichier_chargement == "")
    {
        return false;//on démare avec une liste vide
    }

    QFile fichier_chargement(nom_fichier_chargement);
    if(!fichier_chargement.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        QMessageBox::critical(this, trUtf8("Erreur"), trUtf8("Impossible d'ouvrir le fichier : ") + nom_fichier_chargement + trUtf8("<br />Vérifiez que vous possedez les droit d'accès neccessaire et que le fichier existe réelement."));
        return false;
    }
    //sinon, on tente de lire le fichier : on ajoute chaque question à la liste courrante

    long nombre_de_question_avant = liste_questions.size() + liste_questions_interdites.size();
    long taille_liste = liste_questions.size();

    bool pous_tous = false;
    bool accepter = true;
    bool doublon = false;

    Question *nouvelle_question = NULL;
    QRegExp expression("^#(.+)# #(.+)# #(.+)# #(.+)# ([0-9]{2}\\.[0-9]{2}\\.[0-9]{4}\\-[0-9]{2}:[0-9]{2}) ([0-9]{2}\\.[0-9]{2}\\.[0-9]{4}\\-[0-9]{2}:[0-9]{2})? ([0-9]+) ([0-9]+) ([0-9]+) ([0-9]+) (.) #(.+)#$");
    //QRegExp expression("^#(.+)# #(.+)# #(.+)# #(.+)#$");
    expression.setMinimal(true);

    QTextStream in(&fichier_chargement);
    for(QString ligne = in.readLine() ; !ligne.isNull() ; ligne = in.readLine())// on lit chaque ligne tant que l'on est pas arrivé au bout du fichier
    {
        nouvelle_question = new Question();

        int position = expression.indexIn(ligne);
        if(position > -1)//si on la ligne est valide
        {
            nouvelle_question->question = expression.cap(1);
            nouvelle_question->reponse = expression.cap(2);
            nouvelle_question->question_regexp.setPattern(expression.cap(3));
            nouvelle_question->reponse_regexp.setPattern(expression.cap(4));
            nouvelle_question->date_ajout = QDateTime::fromString(expression.cap(5), FORMAT_DATE);
            nouvelle_question->date_derniere_fois_posee = QDateTime::fromString(expression.cap(6), FORMAT_DATE);
            nouvelle_question->score = expression.cap(7).toInt();
            nouvelle_question->score_recors = expression.cap(8).toInt();
            nouvelle_question->nombre_de_fois_posee = expression.cap(9).toInt();
            nouvelle_question->nombre_de_fois_reussi = expression.cap(10).toInt();
            nouvelle_question->difficulte = expression.cap(11).toUtf8()[0];// convertit la chaine en carractère (pas très propre)
            nouvelle_question->tags = expression.cap(12).split(", ");//chargement des tags
            for(int j = 0 ; j < nouvelle_question->tags.size() ; ++j)
            {
                bool existe_deja = false;
                for(int k = 0 ; k < liste_tags.size() ; ++k)
                {
                    if(nouvelle_question->tags.at(j) == liste_tags.at(k))
                    {
                        existe_deja = true;
                        break;
                    }
                }
                if(!existe_deja)
                {
                    liste_tags.append(nouvelle_question->tags.at(j));
                }
            }

            doublon = false;
            // Vérifications des doublons
            for(int i = 0 ; i < liste_questions.size() ; ++i)
            {
                if(liste_questions.at(i)->question == nouvelle_question->question &&
                   liste_questions.at(i)->reponse == nouvelle_question->reponse)// si la question existe déjà
                {
                    doublon = true;
                    break;// Ca ne sert plus à rien de continuer à parcourir la boucle : on a trouvé une question identique
                }
            }
            for(int i = 0 ; i < liste_questions_interdites.size() ; ++i)
            {
                if(liste_questions_interdites.at(i)->question == nouvelle_question->question &&
                   liste_questions_interdites.at(i)->reponse == nouvelle_question->reponse)// si la question existe déjà
                {
                    doublon = true;
                    break;// Ca ne sert plus à rien de continuer à parcourir la boucle : on a trouvé une question identique
                }
            }
            if(doublon)
            {
                if(!pous_tous)
                {
                    QMessageBox::StandardButton bouton_reponse = QMessageBox::information(this, trUtf8("Doublon"), nouvelle_question->question + trUtf8(" > ") + nouvelle_question->reponse + trUtf8(" ce trouve déjà dans la liste.<br />Voulez vous conserver les deux?"), QMessageBox::Yes | QMessageBox::YesToAll | QMessageBox::No | QMessageBox::NoToAll);
                    if(bouton_reponse == QMessageBox::No)
                    {
                        accepter = false;
                    }
                    else if(bouton_reponse == QMessageBox::NoToAll)
                    {
                        accepter = false;
                        pous_tous = true;
                    }
                    else if(bouton_reponse == QMessageBox::Yes)
                    {
                        accepter = true;
                    }
                    else if(bouton_reponse == QMessageBox::YesToAll)
                    {
                        accepter = true;
                        pous_tous = true;
                    }
                    else
                    {
                        accepter = false;
                    }
                }
            }
            if(!doublon || (doublon && accepter))
            {
                if(!questions_ajoutees_a_la_fin)
                {
                    liste_questions.prepend(nouvelle_question);
                }
                else
                {
                    liste_questions.insert(taille_liste, nouvelle_question);
                }
            }
        }
    }
    fichier_chargement.close();

    actualiser_filtre();
    poser_nouvelle_question();

    if(nombre_de_question_avant < liste_questions.size() + liste_questions_interdites.size())// Si on à rajouté des questions
    {
        if(repertoire_dynamique)
        {
            //le dossier de la liste devient le dossier des boites de dialogues
            nom_repertoire_actuel = nom_fichier_actuel;//convertis automatiquement le nom de fichier en dossier au moment du chargement
        }
        return true;
    }
    else
    {
        QMessageBox::information(this, trUtf8("Information"), trUtf8("Aucune question du fichier ") + nom_fichier_chargement + trUtf8(" n'a été rajoutée à la liste.<br />Raisons possible : toutes les questions était des doublons ou le fichier était invalide."));
        return true;
    }
}

bool MainWindow::sauvegarder_liste(QString nom_fichier_sauvegarde)
{
    if(nom_fichier_sauvegarde == "")
    {
        return false;//on ne vas pas plus loin
    }
    //sinon, on tente d'écrire dans le fichier

    // Rajoute l'extention à la fin du fichier si elle n'existe pas encore
    if(!nom_fichier_sauvegarde.endsWith(".vocab"))
    {
        nom_fichier_sauvegarde += ".vocab";
    }

    QFile fichier_sauvegarde(nom_fichier_sauvegarde);
    if(!fichier_sauvegarde.open(QIODevice::WriteOnly | QIODevice::Text))
    {
        return false;
    }
    QTextStream out(&fichier_sauvegarde);
    for(int i = liste_questions.size()-1 ; i >= 0 ; --i)//pour chaque element de la liste
    {
        ECRIRE_UN_FICHIER(liste_questions)
    }
    for(int i = liste_questions_interdites.size()-1 ; i >= 0 ; --i)//pour chaque element de la liste
    {
        ECRIRE_UN_FICHIER(liste_questions_interdites)
    }

    fichier_sauvegarde.close();

    return true;// On quitte avec succès
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    if(avertir_si_modifier())// si on quitte réelement le programme(en sauvegardant si nessessaire)
    {
        // On sauvegarde les données
        switch (settings.value("Chargement/demarage_reprendre_liste_en_cour", DEFAUT_DEMARAGE_REPRENDRE_LISTE_EN_COUR).toInt())
        {
            case REPRENDRE_LISTE:
                //on charge le dernier fichier utilisé(donc ne modifie pas le fichier actuel)
            break;
            case NOUVELLE_LISTE:
                nom_fichier_actuel = "";//on chargera un fichier vide la prochaine fois
            break;
            case LISTE_PERSONNALISEE:
                nom_fichier_actuel = settings.value("Chargement/nom_fichier_actuel", DEFAUT_NOM_FICHIER_ACTUEL).toString();//on garde le nom de fichier enregistré dans les options
            break;
            default:
                nom_fichier_actuel = "";
            break;
        }
        settings.setValue("Chargement/nom_fichier_actuel", nom_fichier_actuel);
        settings.setValue("Chargement/repertoire_defaut",nom_repertoire_actuel);
        event->accept();
    }
    else
    {
        event->ignore();
    }
}

void MainWindow::resizeEvent(QResizeEvent *event)
{
}

void MainWindow::keyReleaseEvent(QKeyEvent *event)
{
    if(event->key() == Qt::Key_Delete && !champ_reponse->hasFocus())//si on appuie sur la touche "suppr"
    {
        supprimer_questions_selectionnees();
    }
}

bool MainWindow::avertir_si_modifier()// DONE
{
    if(liste_modifiee)
    {
        switch(QMessageBox::warning(this , trUtf8("Enregistrer la liste"), trUtf8("La liste courante à été modifiée.<br />Voulez vous la sauvegarder ?"), QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel))
        {
            case QMessageBox::Save:
                return enregistrer_liste();//on enregistre la liste avant de continuer(si une erreur se produit, on revient a la liste actuel(return false)
            break;
            case QMessageBox::Discard:
                return true;//on continue sans sauvegarder
            break;
            default:
                return false;//on abandone
            break;
        }
    }
    else
    {
        return true;//on peut continuer sans risqué de perdre des données
    }
}

void MainWindow::actualiser_header_verticals()
{
    for(int i = 0 ; i < tableau_principal->rowCount() ; ++i)
    {
        QTableWidgetItem *titre = new QTableWidgetItem;
        titre->setText(QString::number(tableau_principal->rowCount() - i));
        tableau_principal->setVerticalHeaderItem(i, titre);
    }
}

void MainWindow::actualiser_titre()
{
    if(nom_fichier_actuel.isEmpty())
    {
        setWindowTitle(trUtf8("Sans titre"));
    }
    else
    {
        setWindowTitle(QString(nom_fichier_actuel).remove(0, QString(nom_fichier_actuel).lastIndexOf("/") + 1));
    }
    if(liste_modifiee)
    {
        setWindowTitle(windowTitle() + "*");
    }
    setWindowTitle(windowTitle() + " - Vocab++");
}

void MainWindow::vider_liste_completement()
{
    while(!liste_questions.isEmpty())
    {
        delete liste_questions.first();//on libere la memoire
        liste_questions.removeFirst();
    }
    while(!liste_questions_interdites.isEmpty())
    {
        delete liste_questions_interdites.first();
        liste_questions_interdites.removeFirst();
    }
    liste_tags.clear();
    liste_tags_defaut.clear();
    edition_autorisee = false;
    for(int i = 0 ; i < liste_questions_tableau.size() ; ++i)
    {
        delete tableau_principal->item(i, 0);
        delete tableau_principal->item(i, 1);
        delete tableau_principal->item(i, 2);//on efface le tableau

        liste_questions_tableau[i] = NULL;
    }
    edition_autorisee = true;
    poser_nouvelle_question();//on actualise l'affichage de la question
}

void MainWindow::actualiser(bool juste)
{
    edition_autorisee = false;

    // Décalage des lignes du tableau affiché
    delete tableau_principal->item(0, 0);
    delete tableau_principal->item(0, 1);
    delete tableau_principal->item(0, 2);//on suprime les dernieres cases du tableau
    tableau_principal->removeRow(0);//puis on suprime la ligne pour tout décaler
    tableau_principal->insertRow(liste_questions_tableau.size()-1);//on rajoute une ligne à la fin qui contient notre réponse et la solution
    if(!inverse)
    {
        tableau_principal->setItem(liste_questions_tableau.size()-1, 0, new QTableWidgetItem((liste_questions.first())->question));
        tableau_principal->setItem(liste_questions_tableau.size()-1, 1, new QTableWidgetItem((liste_questions.first())->reponse));
    }
    else
    {
        tableau_principal->setItem(liste_questions_tableau.size()-1, 0, new QTableWidgetItem((liste_questions.first())->reponse));
        tableau_principal->setItem(liste_questions_tableau.size()-1, 1, new QTableWidgetItem((liste_questions.first())->question));
    }
    tableau_principal->setItem(liste_questions_tableau.size()-1, 2, new QTableWidgetItem(champ_reponse->text()));

    // Décalage des lignes du tableau virtuel
    liste_questions_tableau.pop_front();// suppression de la première ligne du tableau
    liste_questions_tableau.push_back(liste_questions.first());// on rajoute à la fin la question actuelle
    liste_questions_tableau_inverse.pop_front();
    liste_questions_tableau_inverse.push_back(inverse);
    derniere_question_score_avant = liste_questions.first()->score;
    derniere_question_score_maximum_avant = liste_questions.first()->score_recors;

    // Actualisation du résultat
    liste_questions.first()->date_derniere_fois_posee = QDateTime::currentDateTime();
    liste_questions.first()->nombre_de_fois_posee++;
    if(juste)
    {
        if(liste_questions.first()->score < SCORE_MAX)
        {
            liste_questions.first()->score++;
        }
        // sinon, on garde le score maximum
        liste_questions.first()->nombre_de_fois_reussi++;

        if(liste_questions.first()->score_recors < liste_questions.first()->score)
        {
            liste_questions.first()->score_recors = liste_questions.first()->score;
        }

        // On replace la question dans la liste
        liste_questions.removeFirst();//on supprime la question actuele pour passer à la suivante
        liste_questions.insert(/**FORMULE ICI !!!!*/liste_questions_tableau.last()->score * 3, liste_questions_tableau.last());//on replace un peu plus loin la question
        // TODO : Faire attention a ce que la question ne se retrouve pas en première place(en prenant en compte le hasard)
    }
    else
    {
        //on repose la question(donc on la laisse en file d'attente)
        liste_questions.first()->score = 0;
    }
    // Actualisation des couleurs
    if(liste_questions_tableau.last()->score < SCORE_COULEUR_MAX)
    {
        tableau_principal->item(liste_questions_tableau.size()-1, 0)->setBackgroundColor(liste_couleurs[liste_questions_tableau.last()->score]);
        tableau_principal->item(liste_questions_tableau.size()-1, 1)->setBackgroundColor(liste_couleurs[liste_questions_tableau.last()->score]);
        tableau_principal->item(liste_questions_tableau.size()-1, 2)->setBackgroundColor(liste_couleurs[liste_questions_tableau.last()->score]);
    }
    else
    {
        tableau_principal->item(liste_questions_tableau.size()-1, 0)->setBackgroundColor(liste_couleurs[SCORE_COULEUR_MAX]);
        tableau_principal->item(liste_questions_tableau.size()-1, 1)->setBackgroundColor(liste_couleurs[SCORE_COULEUR_MAX]);
        tableau_principal->item(liste_questions_tableau.size()-1, 2)->setBackgroundColor(liste_couleurs[SCORE_COULEUR_MAX]);
    }

    if(difficultee_calculee_automatiquement)
    {
        // TODO
    }
    if(filtre_actualise_en_permanace)
    {
        // TODO
    }

    liste_questions_tableau_score.pop_front();
    liste_questions_tableau_score.push_back(liste_questions_tableau.last()->score);

    tableau_principal->scrollToBottom();

    bouton_je_me_suis_trompe_autorise = false;//l'utilisateur à appuyé sur le bouton "valider" donc il ne peut plus appuyé sur le bouton "je me suis trompé"
    modifier_liste(true);// le score est modifié
    poser_nouvelle_question();
    actualiser_taille();
    actualiser_header_verticals();

    edition_autorisee = true;
}

void MainWindow::actualiser_taille(bool taille_modifiee)
{
    for(int i = 0 ; i < 3 ; ++i)
    {
        int taille_precedente = tableau_principal->columnWidth(i);
        tableau_principal->resizeColumnToContents(i);
        int taille_acuelle = tableau_principal->columnWidth(i);
        if(taille_precedente > taille_acuelle)
        {
            tableau_principal->setColumnWidth(i, taille_precedente);
            taille_modifiee = true;
        }
    }

    if(taille_modifiee)
    {
        int nouvelle_taille = tableau_principal->horizontalHeader()->length() + tableau_principal->verticalHeader()->size().width();
        if(!tableau_principal->verticalScrollBar()->isHidden())
        {
            nouvelle_taille += tableau_principal->verticalScrollBar()->sizeHint().width();
        }
        tableau_principal->setFixedWidth(nouvelle_taille);
    }
}

void MainWindow::actualiser_filtre()
{
    edition_autorisee = false;
    // On regarde si on a maintenant des questions qui peuvent etre posées
    for(int i = 0 ;  i < liste_questions_interdites.size() ; ++i)
    {
        bool filtrer = true;
        for(int j = 0 ; j < liste_filtres.size() ; ++j)
        {
            if(liste_filtres.at(j).type_de_filtre == FILTRE_DATE_AJOUT ||
               liste_filtres.at(j).type_de_filtre == FILTRE_DATE_DERNIERE_FOIS_POSEE)
            {
                QDateTime *valeur_question_filtree;
                if(liste_filtres.at(j).type_de_filtre == FILTRE_DATE_AJOUT)
                {
                    valeur_question_filtree = &liste_questions_interdites.at(i)->date_ajout;
                }
                else if(liste_filtres.at(j).type_de_filtre == FILTRE_DATE_DERNIERE_FOIS_POSEE)
                {
                    valeur_question_filtree = &liste_questions_interdites.at(i)->date_derniere_fois_posee;
                }

                if((liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_AVANT_LE &&
                    *valeur_question_filtree > liste_filtres.at(j).valeur.toDateTime()) ||
                    (liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_APRES_LE &&
                     *valeur_question_filtree < liste_filtres.at(j).valeur.toDateTime()))
                {
                    filtrer = false;
                    break;// la question ne sera pas filtrée : cela ne sert à rien de vérifier les autres filtres
                }
            }
            else if(liste_filtres.at(j).type_de_filtre == FILTRE_SCORE ||
                    liste_filtres.at(j).type_de_filtre == FILTRE_SCORE_MAXIMUM ||
                    liste_filtres.at(j).type_de_filtre == FILTRE_NOMBRE_DE_FOIS_POSEE ||
                    liste_filtres.at(j).type_de_filtre == FILTRE_NOMBRE_DE_FOIS_REUSSI)
            {
                int *valeur_question_filtree;
                if(liste_filtres.at(j).type_de_filtre == FILTRE_SCORE)
                {
                    valeur_question_filtree = &liste_questions_interdites.at(i)->score;
                }
                else if(liste_filtres.at(j).type_de_filtre == FILTRE_SCORE_MAXIMUM)
                {
                    valeur_question_filtree = &liste_questions_interdites.at(i)->score_recors;
                }
                else if(liste_filtres.at(j).type_de_filtre == FILTRE_NOMBRE_DE_FOIS_POSEE)
                {
                    valeur_question_filtree = &liste_questions_interdites.at(i)->nombre_de_fois_posee;
                }
                else if(liste_filtres.at(j).type_de_filtre == FILTRE_NOMBRE_DE_FOIS_REUSSI)
                {
                    valeur_question_filtree = &liste_questions_interdites.at(i)->nombre_de_fois_reussi;
                }

                if((liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_INFERIEUR_A &&
                   *valeur_question_filtree > liste_filtres.at(j).valeur.toInt()) ||
                   (liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_EGAL_A &&
                   *valeur_question_filtree != liste_filtres.at(j).valeur.toInt()) ||
                   (liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_EGAL_A &&
                   *valeur_question_filtree < liste_filtres.at(j).valeur.toInt()))
                {
                    filtrer = false;
                    break;
                }
            }
            else if(liste_filtres.at(j).type_de_filtre == FILTRE_DIFFICULTEE)
            {
                if((liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_PLUS_FACILE_QUE &&
                    liste_filtres.at(j).valeur.toInt() < liste_questions_interdites.at(i)->difficulte) ||
                   (liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_DE_DIFFICULTEE &&
                    liste_filtres.at(j).valeur.toInt() != liste_questions_interdites.at(i)->difficulte) ||
                   (liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_PLUS_DIFICILE_QUE &&
                    liste_filtres.at(j).valeur.toInt() > liste_questions_interdites.at(i)->difficulte))
                {
                    filtrer = false;
                    break;
                }
            }
            else if(liste_filtres.at(j).type_de_filtre == FILTRE_TAGS)
            {
                for(int k = 0 ; k < liste_questions_interdites.at(i)->tags.size() ; ++k)
                {
                    if((liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_CONTENANT &&
                        !liste_filtres.at(j).valeur.toStringList().contains(liste_questions_interdites.at(i)->tags.at(k))) ||
                       (liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_NE_CONTENANT_PAS &&
                        liste_filtres.at(j).valeur.toStringList().contains(liste_questions_interdites.at(i)->tags.at(k))))
                    {
                        filtrer = false;
                        break;
                    }
                }
            }
        }
        if(!filtrer || liste_filtres.isEmpty())// si la question n'est plus à filtrer
        {
            // Actualisation des couleurs
            for(int j = 0 ; j < liste_questions_tableau.size() ; ++j)
            {
                std::cout << "aa\n";
                if(liste_questions_tableau.at(j) == liste_questions_interdites.at(i))
                {
                    if(liste_questions_tableau_score.at(j) < SCORE_COULEUR_MAX)
                    {
                        tableau_principal->item(j, 0)->setBackgroundColor(liste_couleurs[liste_questions_tableau_score.at(j)]);
                        tableau_principal->item(j, 1)->setBackgroundColor(liste_couleurs[liste_questions_tableau_score.at(j)]);
                        tableau_principal->item(j, 2)->setBackgroundColor(liste_couleurs[liste_questions_tableau_score.at(j)]);
                    }
                    else
                    {
                        tableau_principal->item(j, 0)->setBackgroundColor(liste_couleurs[SCORE_COULEUR_MAX]);
                        tableau_principal->item(j, 1)->setBackgroundColor(liste_couleurs[SCORE_COULEUR_MAX]);
                        tableau_principal->item(j, 2)->setBackgroundColor(liste_couleurs[SCORE_COULEUR_MAX]);
                    }
                }
            }

            liste_questions.append(liste_questions_interdites.at(i));
            liste_questions_interdites.removeAt(i);
            i--;//la liste comporte un élément de moins
        }
    }
    // Suppression des question de la liste actuelles qui doivent etre filtree
    for(int i = 0 ;  i < liste_questions.size() ; ++i)
    {
        bool filtrer = true;
        for(int j = 0 ; j < liste_filtres.size() ; ++j)
        {
            if(liste_filtres.at(j).type_de_filtre == FILTRE_DATE_AJOUT ||
               liste_filtres.at(j).type_de_filtre == FILTRE_DATE_DERNIERE_FOIS_POSEE)
            {
                QDateTime *valeur_question_filtree;
                if(liste_filtres.at(j).type_de_filtre == FILTRE_DATE_AJOUT)
                {
                    valeur_question_filtree = &liste_questions.at(i)->date_ajout;
                }
                else if(liste_filtres.at(j).type_de_filtre == FILTRE_DATE_DERNIERE_FOIS_POSEE)
                {
                    valeur_question_filtree = &liste_questions.at(i)->date_derniere_fois_posee;
                }

                if((liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_AVANT_LE &&
                    *valeur_question_filtree > liste_filtres.at(j).valeur.toDateTime()) ||
                    (liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_APRES_LE &&
                     *valeur_question_filtree < liste_filtres.at(j).valeur.toDateTime()))
                {
                    filtrer = false;
                    break;// la question ne sera pas filtrée : cela ne sert à rien de vérifier les autres filtres
                }
            }
            else if(liste_filtres.at(j).type_de_filtre == FILTRE_SCORE ||
                    liste_filtres.at(j).type_de_filtre == FILTRE_SCORE_MAXIMUM ||
                    liste_filtres.at(j).type_de_filtre == FILTRE_NOMBRE_DE_FOIS_POSEE ||
                    liste_filtres.at(j).type_de_filtre == FILTRE_NOMBRE_DE_FOIS_REUSSI)
            {
                int *valeur_question_filtree;
                if(liste_filtres.at(j).type_de_filtre == FILTRE_SCORE)
                {
                    valeur_question_filtree = &liste_questions.at(i)->score;
                }
                else if(liste_filtres.at(j).type_de_filtre == FILTRE_SCORE_MAXIMUM)
                {
                    valeur_question_filtree = &liste_questions.at(i)->score_recors;
                }
                else if(liste_filtres.at(j).type_de_filtre == FILTRE_NOMBRE_DE_FOIS_POSEE)
                {
                    valeur_question_filtree = &liste_questions.at(i)->nombre_de_fois_posee;
                }
                else if(liste_filtres.at(j).type_de_filtre == FILTRE_NOMBRE_DE_FOIS_REUSSI)
                {
                    valeur_question_filtree = &liste_questions.at(i)->nombre_de_fois_reussi;
                }

                if((liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_INFERIEUR_A &&
                   *valeur_question_filtree > liste_filtres.at(j).valeur.toInt()) ||
                   (liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_EGAL_A &&
                   *valeur_question_filtree != liste_filtres.at(j).valeur.toInt()) ||
                   (liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_EGAL_A &&
                   *valeur_question_filtree < liste_filtres.at(j).valeur.toInt()))
                {
                    filtrer = false;
                    break;
                }
            }
            else if(liste_filtres.at(j).type_de_filtre == FILTRE_DIFFICULTEE)
            {
                if((liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_PLUS_FACILE_QUE &&
                    liste_filtres.at(j).valeur.toInt() < liste_questions.at(i)->difficulte) ||
                   (liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_DE_DIFFICULTEE &&
                    liste_filtres.at(j).valeur.toInt() != liste_questions.at(i)->difficulte) ||
                   (liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_PLUS_DIFICILE_QUE &&
                    liste_filtres.at(j).valeur.toInt() > liste_questions.at(i)->difficulte))
                {
                    filtrer = false;
                    break;
                }
            }
            else if(liste_filtres.at(j).type_de_filtre == FILTRE_TAGS)
            {
                for(int k = 0 ; k < liste_questions.at(i)->tags.size() ; ++k)
                {
                    if((liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_CONTENANT &&
                        !liste_filtres.at(j).valeur.toStringList().contains(liste_questions.at(i)->tags.at(k))) ||
                       (liste_filtres.at(j).mode_de_filtrage == MODE_FILTRE_NE_CONTENANT_PAS &&
                        liste_filtres.at(j).valeur.toStringList().contains(liste_questions.at(i)->tags.at(k))))
                    {
                        filtrer = false;
                        break;
                    }
                }
            }
        }
        if(filtrer && !liste_filtres.isEmpty())//si la question doit etre filtrée
        {
            for(int j = liste_questions_tableau.indexOf(liste_questions.at(i)) ; j != -1 ; j = liste_questions_tableau.indexOf(liste_questions.at(i), j + 1))
            {
                std::cout << j << "\n";
                tableau_principal->item(j, 0)->setBackgroundColor(COULEUR_FILTREE);
                tableau_principal->item(j, 1)->setBackgroundColor(COULEUR_FILTREE);
                tableau_principal->item(j, 2)->setBackgroundColor(COULEUR_FILTREE);
            }

            liste_questions_interdites.prepend(liste_questions.at(i));
            liste_questions.removeAt(i);
            i--;
        }
    }
    edition_autorisee = true;
}

bool MainWindow::question_filtree(Question *question_a_filtrer)
{
    return false;
}

void MainWindow::poser_nouvelle_question()
{
    champ_reponse->clear();//on vide le champ(utile dans les cas, ajouter, supprimer,...)
    if(!liste_questions.isEmpty())
    {
        if(qrand() % 100 >= valeur_inversion)
        {
            texte_question->setText("<strong>" + (liste_questions.first())->question + "</strong>");//on pose la question
            inverse = false;
        }
        else
        {
            texte_question->setText("<strong>" + (liste_questions.first())->reponse + "</strong>");//on pose la question
            inverse = true;//on a inversé la question et la réponse
        }
    }
    else
    {
        texte_question->clear();//si la liste est vide, on efface la question
    }
}

void MainWindow::reposer_question()
{
    if(!liste_questions.isEmpty())
    {
        if(!inverse)
        {
            texte_question->setText("<strong>" + (liste_questions.first())->question + "</strong>");//on pose la question
        }
        else
        {
            texte_question->setText("<strong>" + (liste_questions.first())->reponse + "</strong>");//on pose la question
        }
    }
    else
    {
        texte_question->clear();//si la liste est vide, on efface la question
    }
}

void MainWindow::question_editee(int row, int column)// DONE
{
    if(edition_autorisee)
    {
        if(liste_questions_tableau.at(row) != NULL)// si on à modifié une question qui existait et non une ligne vide
        {
            edition_autorisee = false;//je ne sais pas s'il y a une recursivité avec les signaux et slots
            if(tableau_principal->item(row, column)->text().isEmpty())// Si on a vidé la case : on remet ce qu'il y avait avant
            {
                if(column == 0)
                {
                    if(!liste_questions_tableau_inverse.at(row))
                    {
                        tableau_principal->item(row, column)->setText(liste_questions_tableau.at(row)->question);//on revient à l'état précédent
                    }
                    else
                    {
                        tableau_principal->item(row, column)->setText(liste_questions_tableau.at(row)->reponse);//on revient à l'état précédent
                    }
                }
                else if(column == 1)
                {
                    if(!liste_questions_tableau_inverse.at(row))
                    {
                        tableau_principal->item(row, column)->setText(liste_questions_tableau.at(row)->reponse);//on revient à l'état précédent
                    }
                    else
                    {
                        tableau_principal->item(row, column)->setText(liste_questions_tableau.at(row)->question);//on revient à l'état précédent
                    }
                }
                else//sinon, on à modifié une colone qui ne nous interesse pas
                {
                }
            }
            else
            {
                if(column == 0)
                {
                    if(!liste_questions_tableau_inverse.at(row))
                    {
                        liste_questions_tableau.at(row)->question = tableau_principal->item(row, column)->text();//en modifiant le tableau, on modifie la liste principale(grace aux pointeur)
                        liste_questions_tableau.at(row)->question_regexp.setPattern(Dialog_ajouter_question::calculer_regexp(liste_questions_tableau.at(row)->question));
                    }
                    else
                    {
                        liste_questions_tableau.at(row)->reponse = tableau_principal->item(row, column)->text();//en modifiant le tableau, on modifie la liste principale(grace aux pointeur)
                        liste_questions_tableau.at(row)->reponse_regexp.setPattern(Dialog_ajouter_question::calculer_regexp(liste_questions_tableau.at(row)->reponse));
                    }
                }
                else if(column == 1)
                {
                    if(!liste_questions_tableau_inverse.at(row))
                    {
                        liste_questions_tableau.at(row)->reponse = tableau_principal->item(row, column)->text();//en modifiant le tableau, on modifie la liste principale(grace aux pointeur)
                        liste_questions_tableau.at(row)->reponse_regexp.setPattern(Dialog_ajouter_question::calculer_regexp(liste_questions_tableau.at(row)->reponse));
                    }
                    else
                    {
                        liste_questions_tableau.at(row)->question = tableau_principal->item(row, column)->text();//en modifiant le tableau, on modifie la liste principale(grace aux pointeur)
                        liste_questions_tableau.at(row)->question_regexp.setPattern(Dialog_ajouter_question::calculer_regexp(liste_questions_tableau.at(row)->question));
                    }
                }
                else//sinon, on à modifié une colone qui ne nous interesse pas
                {
                }
            }

            // Actualisation du tableau
            for(int i = 0 ; i < liste_questions_tableau.size() ; ++i)
            {
                if(liste_questions_tableau.at(i) == liste_questions_tableau.at(row))
                {
                    if(!liste_questions_tableau_inverse.at(i))// si la question a été posée normalement
                    {
                        tableau_principal->item(i, 0)->setText(liste_questions_tableau.at(row)->question);
                        tableau_principal->item(i, 1)->setText(liste_questions_tableau.at(row)->reponse);
                    }
                    else
                    {
                        tableau_principal->item(i, 0)->setText(liste_questions_tableau.at(row)->reponse);
                        tableau_principal->item(i, 1)->setText(liste_questions_tableau.at(row)->question);
                    }
                }
            }

            reposer_question();// si les questions sont posé aléatoirement Q->R et R->Q, ca peut changer
            edition_autorisee = true;
        }
        else
        {
            QMessageBox::critical(this, trUtf8("Erreur"), trUtf8("Erreur : impossible de modifier la question.<br />Raison : la question n'existe pas ou a été supprimée."));
        }
    }
}

void MainWindow::valider()
{
    if(champ_reponse->text().startsWith(":save"))
    {
        if(champ_reponse->text() == ":save")
        {
            enregistrer_liste();
        }
    }
    else if(champ_reponse->text().startsWith(":open"))
    {
    }
    else if(champ_reponse->text().startsWith(":add"))
    {
        QRegExp regexp_nouvelle_question(":add #(.*)# #(.*)#");
        if(regexp_nouvelle_question.indexIn(champ_reponse->text()) == -1)
        {
            ajouter_question();
        }
        else
        {
            // TODO ? ? (détection des doublons et cie)
        }
    }
    else if(champ_reponse->text() == ":clear")
    {
        vider_liste_completement();
    }
    else if(champ_reponse->text().startsWith(":sort"))
    {
        if(champ_reponse->text() == ":sort")
        {
            melanger_liste();
        }
    }
    else if(champ_reponse->text() == ":remove" || champ_reponse->text() == ":delete")
    {
        supprimer_question_actuelle();
    }
    else if(champ_reponse->text().startsWith(":reset"))
    {
        reinitialiser_score();
    }
    else if(champ_reponse->text() == ":)" || champ_reponse->text() == ":-)")
    {
        je_sais();
    }
    // Pour "je ne sais pas", il suffit d'appuyé sur entrée sans rien marquer
    else if(champ_reponse->text() == ":(" || champ_reponse->text() == ":-(")
    {
        je_me_suis_trompe();
    }
    else if(champ_reponse->text() == ":>>" || champ_reponse->text() == ":skip")
    {
    }
    else if(champ_reponse->text() == ":about")
    {
        a_propos();
    }
    else if(champ_reponse->text() == ":help")
    {
    }
    else
    {
        if(!liste_questions.isEmpty())
        {
            QRegExp sauvegarde_reponse_regexp;
            QRegExp sauvegarde_question_regexp;
            QString sauvegarde_champs = champ_reponse->text();
            if(tolerant)
            {
                sauvegarde_reponse_regexp = liste_questions.first()->reponse_regexp;
                sauvegarde_question_regexp = liste_questions.first()->question_regexp;
            }
            else
            {
                sauvegarde_reponse_regexp.setPattern("^" + liste_questions.first()->reponse + "$");
                sauvegarde_question_regexp.setPattern("^" + liste_questions.first()->question + "$");
            }
            if(majuscules_facultatives)
            {
                sauvegarde_reponse_regexp.setCaseSensitivity(Qt::CaseInsensitive);
                sauvegarde_question_regexp.setCaseSensitivity(Qt::CaseInsensitive);
            }
            else
            {
                sauvegarde_reponse_regexp.setCaseSensitivity(Qt::CaseSensitive);
                sauvegarde_question_regexp.setCaseSensitivity(Qt::CaseSensitive);
            }
            if(accents_facultatifs)
            {
                QStringList liste_accents;
                liste_accents << "à" << "a"
                              << "À" << "A"
                              << "â" << "a"
                              << "Â" << "A"
                              << "ä" << "a"
                              << "Ä" << "A"
                              << "é" << "e"
                              << "É" << "E"
                              << "ê" << "e"
                              << "Ê" << "E"
                              << "è" << "e"
                              << "È" << "E"
                              << "ë" << "e"
                              << "Ë" << "E"
                              << "î" << "i"
                              << "Î" << "I"
                              << "ï" << "i"
                              << "Ï" << "I"
                              << "ô" << "o"
                              << "Ô" << "O"
                              << "ö" << "o"
                              << "Ö" << "O"
                              << "ü" << "u"
                              << "Ü" << "U"
                              << "û" << "u"
                              << "Û" << "U"
                              << "æ" << "ae"
                              << "Æ" << "AE"
                              << "œ" << "oe"
                              << "Œ" << "OE"
                              << "ç" << "c"
                              << "Ç" << "C";

                QString patern_sauvegarde_reponse_regexp = sauvegarde_reponse_regexp.pattern();
                QString patern_sauvegarde_question_regexp = sauvegarde_question_regexp.pattern();
                for(int i = 0 ; i < liste_accents.size() ; i+=2)
                {
                    patern_sauvegarde_reponse_regexp.replace(QRegExp(liste_accents.at(i)), "(?:" + liste_accents.at(i) + "|" + liste_accents.at(i + 1) + ")");
                    patern_sauvegarde_question_regexp.replace(QRegExp(liste_accents.at(i)), "(?:" + liste_accents.at(i) + "|" + liste_accents.at(i + 1) + ")");
                }
                sauvegarde_reponse_regexp.setPattern(patern_sauvegarde_reponse_regexp);
                sauvegarde_question_regexp.setPattern(patern_sauvegarde_question_regexp);
            }

            if((sauvegarde_reponse_regexp.indexIn(sauvegarde_champs) != -1 && !inverse) ||
               (sauvegarde_question_regexp.indexIn(sauvegarde_champs) != -1 && inverse))//si on a juste...
            {
                actualiser(true);
            }
            else// sinon, on a faux
            {
                actualiser(false);
            }
        }
    }
    champ_reponse->clear();
}

void MainWindow::ajouter_carractere_special(QString carractere)
{
    champ_reponse->setText(champ_reponse->text() + carractere);
}

void MainWindow::ajouter_supprimer_tags_questions_selectionnees()
{
    ajouter_supprimer_tags(true);
}

void MainWindow::ajouter_clavier_virtuel(QString nom, QString touches_clavier)
{
    claviers_virtuels.append(new QDockWidget(nom, this));

    Clavier_virtuel *widget_clavier = new Clavier_virtuel(nom, touches_clavier);

    claviers_virtuels.last()->setWidget(widget_clavier);
    claviers_virtuels.last()->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea | Qt::BottomDockWidgetArea);
    addDockWidget(Qt::BottomDockWidgetArea, claviers_virtuels.last());
    claviers_virtuels.last()->setMaximumHeight(claviers_virtuels.last()->sizeHint().height());
    claviers_virtuels.last()->hide();
    connect(widget_clavier, SIGNAL(touche_appuyee(QString)), this, SLOT(ajouter_carractere_special(QString)));
    
    //insert l'action juste avant le sépatateur
    QAction *action_separateur = NULL;
    for(int i = 0 ; i < menu_claviers_virtuels->actions().size() ; ++i)
    {
        if(menu_claviers_virtuels->actions().at(i)->isSeparator())
        {
            action_separateur = menu_claviers_virtuels->actions().at(i);
            break;
        }
    }
    menu_claviers_virtuels->insertAction(action_separateur, claviers_virtuels.last()->toggleViewAction());
}

void MainWindow::modifier_liste(bool nouvelle_valeur)
{
    liste_modifiee = nouvelle_valeur;
    actualiser_titre();
}

void MainWindow::je_sais()
{
    if(!liste_questions.isEmpty())
    {
        champ_reponse->clear();// pour la troisième colone
        actualiser(true);
        bouton_je_me_suis_trompe_autorise = true;
    }
}

void MainWindow::je_ne_sais_pas()
{
    // (!) (!) (!) A SUPPRIMER SI LE TEST EST FAIT DANS "ACTUALISER(BOOL)"
    if(!liste_questions.isEmpty())
    {
        champ_reponse->clear();
        actualiser(false);
    }
}

void MainWindow::je_me_suis_trompe()
{
    if(bouton_je_me_suis_trompe_autorise && tableau_principal->item(liste_questions_tableau.size() - 1, 0)->backgroundColor() != COULEUR_FILTREE)
    {
        edition_autorisee = false;

        liste_questions.removeOne(liste_questions_tableau.last());//on suprime la question
        liste_questions.prepend(liste_questions_tableau.last());//on la rajoute en debut de liste pour la reposer

        // Actualisation des lignes du tableau
        tableau_principal->item(liste_questions_tableau.size()-1, 0)->setBackgroundColor(liste_couleurs[SCORE_COULEUR_RATE]);
        tableau_principal->item(liste_questions_tableau.size()-1, 1)->setBackgroundColor(liste_couleurs[SCORE_COULEUR_RATE]);
        tableau_principal->item(liste_questions_tableau.size()-1, 2)->setBackgroundColor(liste_couleurs[SCORE_COULEUR_RATE]);

        liste_questions_tableau.last()->score = 0;
        liste_questions_tableau.last()->nombre_de_fois_reussi--;
        // Si l'utilisateur à obtenu le meilleur score en appuyant sur "je crois savoir", il faut actualiser le meilleur score :
        liste_questions_tableau.last()->score_recors = derniere_question_score_maximum_avant;// si le score précédent était moin élevé

        bouton_je_me_suis_trompe_autorise = false;// l'utilisateur ne peut pas appuyé 2X de suite sur ce bouton
        modifier_liste(true);// La liste est modifiée(si l'utilisateur à sauvegardé juste avant)

        edition_autorisee = true;

        poser_nouvelle_question();//on repose la question
    }
}

void MainWindow::ne_plus_voir_cette_question()
{
    if(!liste_questions.isEmpty())
    {
        Question *sauvegarde_question = liste_questions.first();
        liste_questions.first()->score = SCORE_MAX;
        actualiser(true);
        liste_questions.removeOne(sauvegarde_question);
        liste_questions.append(sauvegarde_question);
    }
}

void MainWindow::sauter_cette_question()
{
    liste_questions.insert(6, liste_questions.first());
    liste_questions.removeFirst();
    reposer_question();//si la question ne fait qu'un element
}

void MainWindow::nouvelle_liste()//crée une nouvelle liste vide// DONE
{
    if(avertir_si_modifier())// On sauvergarde si necessaire
    {
        modifier_liste(false);
        nom_fichier_actuel = "";
        // On ne modifie pas le répertoire par defaut
        actualiser_titre();
        vider_liste_completement();
    }
}

void MainWindow::ouvrir_liste()//ouvre une nouvelle liste qui s'ajoute ou remplace la liste actuelle// DONE
{
    QStringList noms_fichiers_chargements = QFileDialog::getOpenFileNames(this, trUtf8("Charger une/des liste(s)"), nom_repertoire_actuel, FILTRE_CHARGEMENT_SAUVEGARDE);
    if(!noms_fichiers_chargements.isEmpty())
    {
        bool liste_modifiee_sauvegarde;
        bool fichier_reussi = false;
        switch(QMessageBox::question(this, trUtf8("Question"), trUtf8("Voulez vous ajouter ces questions à la liste en cours ?"), QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel))
        {
            case QMessageBox::Yes:
                liste_modifiee_sauvegarde = liste_modifiee;//en cas d'echec, on revient à l'état precedent
                modifier_liste(true);//on modifie la liste actuelle
            break;
            case QMessageBox::No:
                if(avertir_si_modifier())
                {
                    vider_liste_completement();//on vide la liste
                    if(noms_fichiers_chargements.size() == 1)//si on n'a ouvert qu'un seul fichier
                    {
                        nom_fichier_actuel = noms_fichiers_chargements.first();//le seul fichier que l'on a chargé devient le fichier actuel
                        modifier_liste(false);
                    }
                    else//sinon, on a chargé plusieurs fichiers
                    {
                        nom_fichier_actuel = "";//on crée une nouvelle liste
                        modifier_liste(true);//notre liste correnspond à un ensemble de liste que l'on regroupe ensemble
                    }
                    actualiser_titre();
                    liste_modifiee_sauvegarde = false;//en cas d'echec,la liste est vide(donc non modifiée)
                }
                else
                {
                    return;//on retourne au document en cours
                }
            break;
            default://si on annule
                return;//on ne vas pas plus loin
            break;
        }
        for(int i = 0 ; i < noms_fichiers_chargements.size() ; ++i)//on ajoute chaque fichier à la liste actuelle
        {
            if(charger_liste(noms_fichiers_chargements.at(i)) && !fichier_reussi)//si c'est le premier fichier qui se charge correctement
            {
                fichier_reussi = true;
            }
        }
        if(!fichier_reussi)// si on n'a réussi à ouvrir aucun fichier
        {
            liste_modifiee = liste_modifiee_sauvegarde;
        }
    }
    //sinon l'utilisateur n'a selectioné aucun fichier et on retourne a la liste actuele
}

bool MainWindow::enregistrer_liste()//Retourne true si il a enregistré avec succes sinon, retourne false// DONE
{
    if(QFile::exists(nom_fichier_actuel))//si le fichier existe
    {
        if(sauvegarder_liste(nom_fichier_actuel))//on le sauvegarde(et retourne false si on ne réussi pas à l'enregister(pas les droits d'acces,...))
        {
            modifier_liste(false);
            return true;
        }
        else
        {
            return false;//malgres que le fichier existe,on n'a pas réussit a enregistrer la liste
        }
    }
    else
    {
        return enregistrer_liste_sous();
    }
}

bool MainWindow::enregistrer_liste_sous()//Retourne true si il a enregistré avec succes sinon, retourne false// DONE
{
    QString nom_fichier_sauvegarde = QFileDialog::getSaveFileName(this, trUtf8("Sauvegarder la liste sous..."), nom_repertoire_actuel, FILTRE_CHARGEMENT_SAUVEGARDE);
    if(sauvegarder_liste(nom_fichier_sauvegarde))//retourne true si le fichier s'est sauvegarder avec succes. Si on a cliqué sur annulé dans la boite de dialogue si dessus, la fonction retourne false
    {
        modifier_liste(false);
        nom_fichier_actuel = nom_fichier_sauvegarde;
        actualiser_titre();
        return true;
    }
    return false;
}

void MainWindow::imprimer()
{
}

void MainWindow::ajouter_question()// DONE(sauf choix rajouter debut ou rajouter fin et détection des doublons)
{
    // Variables pour les doublons
    bool accepter = true;
    bool doublon = false;

    // Création de la boite de dialogue
    Dialog_ajouter_question *dialog_ajouter_question = new Dialog_ajouter_question(&liste_tags, &liste_tags_defaut, &claviers_virtuels, this);
    Question *nouvelle_question = new Question();
    while(dialog_ajouter_question->exec(nouvelle_question))
    {
        // Vérifications des doublons
        doublon = false;
        for(int i = 0 ; i < liste_questions.size() ; ++i)
        {
            if(liste_questions.at(i)->question == nouvelle_question->question &&
               liste_questions.at(i)->reponse == nouvelle_question->reponse)// si la question existe déjà
            {
                doublon = true;
                break;// Ca ne sert plus à rien de continuer à parcourir la boucle : on a trouvé une question identique
            }
        }
        if(!doublon)
        {
            for(int i = 0 ; i < liste_questions_interdites.size() ; ++i)
            {
                if(liste_questions_interdites.at(i)->question == nouvelle_question->question &&
                   liste_questions_interdites.at(i)->reponse == nouvelle_question->reponse)// si la question existe déjà
                {
                    doublon = true;
                    break;// Ca ne sert plus à rien de continuer à parcourir la boucle : on a trouvé une question identique
                }
            }
        }
        if(doublon)
        {
            if(QMessageBox::information(this, trUtf8("Doublon"), nouvelle_question->question + trUtf8(" > ") + nouvelle_question->reponse + trUtf8(" ce trouve déjà dans la liste.<br />Voulez vous conserver les deux?"), QMessageBox::Yes | QMessageBox::Cancel) == QMessageBox::Yes)
            {
                accepter = true;
            }
            else
            {
                accepter = false;
            }
        }
        if(!doublon || (doublon && accepter))// si on rajoute réelement la question
        {
            if(!questions_ajoutees_a_la_fin)
            {
                liste_questions.append(nouvelle_question);
            }
            else
            {
                liste_questions.prepend(nouvelle_question);// On la rajoute au début de la liste
            }
            actualiser_filtre();
            if(liste_questions.first() == nouvelle_question)
            {
                poser_nouvelle_question();// Actualisation l'affichage
            }//sinon, la question à été filtrée(dificultée trop dure,...) donc on garde la meme question
            modifier_liste(true);// Maintenant, la liste est modifiée
        }
        // Création d'une nouvelle question
        nouvelle_question= new Question();
    }
    delete nouvelle_question;//supprime la question que l'on n'a pas ajoutée
    delete dialog_ajouter_question;
}

void MainWindow::supprimer_question_actuelle()//DONE
{
    if(!liste_questions.isEmpty() && QMessageBox::warning(this, "Confirmation", "Êtes vous sûr de vouloir supprimer la question actuelle ?", QMessageBox::Yes | QMessageBox::Cancel) == QMessageBox::Yes)
    {
        edition_autorisee = false;

        for(int i = 0 ; i < liste_questions_tableau.size() ; ++i)
        {
            if(liste_questions_tableau.at(i) == liste_questions.first())// (!) Remarque : la question actuelle ce trouve forcement dans la liste active
            {
                liste_questions_tableau[i] = NULL;// supprime toutes les questions que l'on a supprimé
                tableau_principal->item(i, 0)->setBackgroundColor(COULEUR_SUPPRIMEE);
                tableau_principal->item(i, 1)->setBackgroundColor(COULEUR_SUPPRIMEE);
                tableau_principal->item(i, 2)->setBackgroundColor(COULEUR_SUPPRIMEE);
            }
        }

        edition_autorisee = true;

        delete liste_questions.first();
        liste_questions.removeFirst();

        poser_nouvelle_question();

        modifier_liste(true);
    }
}

void MainWindow::supprimer_questions_selectionnees()// DONE
{
    bool selection_valide = false;
    for(int i = 0 ; i < tableau_principal->selectedItems().size() ; ++i)// On parcour déjà une fois la liste pour savoir s'il y a au moins un element à supprimer
    {
        int ligne = tableau_principal->selectedItems().at(i)->row();
        if(liste_questions_tableau.at(ligne) != NULL)
        {
            selection_valide = true;
            break;//on ne va pas plus loin
        }
    }

    // la séléction n'est pas valide si la liste est vide(toutes les cases sont NULL) ou si la séléction est vide
    if(selection_valide && QMessageBox::warning(this, "Confirmation", "Êtes vous sûr de vouloir supprimer les questions séléctionées", QMessageBox::Yes | QMessageBox::Cancel) == QMessageBox::Yes)
    {
        edition_autorisee = false;

        for(int i = 0 ; i < tableau_principal->selectedItems().size() ; ++i)
        {
            int ligne = tableau_principal->selectedItems().at(i)->row();
            if(liste_questions_tableau.at(ligne) != NULL)// Si la question n'existe pas(ou à déjà été supprimée), on ne fait rien. Sinon,...
            {
                if(!liste_questions.removeOne(liste_questions_tableau.at(ligne)))//si la question ne se trouve pas dans la liste
                {
                    liste_questions_interdites.removeOne(liste_questions_tableau.at(ligne));//...elle se trouve forcement dans la liste des questions filtrées
                }

                Question *question_sauvegarde = liste_questions_tableau.at(ligne);//on sauvegarde la question car une fois que l'on vide le tableau, on n'a plus aucun moyen de savoir l'adresse de la question pour pouvoir la supprimer
                for(int j = 0 ; j < liste_questions_tableau.size() ; ++j)
                {
                    if(liste_questions_tableau.at(j) == question_sauvegarde)// supprime toutes les questions que l'on a supprimé
                    {
                        liste_questions_tableau[j] = NULL;// la ligne est désormais vide
                        tableau_principal->item(j, 0)->setBackgroundColor(COULEUR_SUPPRIMEE);
                        tableau_principal->item(j, 1)->setBackgroundColor(COULEUR_SUPPRIMEE);
                        tableau_principal->item(j, 2)->setBackgroundColor(COULEUR_SUPPRIMEE);
                    }
                }
                delete question_sauvegarde;//on libere la memoire pour supprimer definitivement la question
            }
        }
        edition_autorisee = true;

        modifier_liste(true);//la liste est forcement modifiée puisque la liste est valide
        // (!) Limite : ne pose pas une nouvelle question uniquement si on a supprimer la premiere : on ne respecte pas le hasard
        reposer_question();
    }
    tableau_principal->clearSelection();
}

void MainWindow::modifier_question_actuelle()// DONE
{
    if(!liste_questions.isEmpty())
    {
        Dialog_ajouter_question *dialog_ajouter_question = new Dialog_ajouter_question(&liste_tags, &liste_tags_defaut, &claviers_virtuels, this);
        if(dialog_ajouter_question->exec(liste_questions.first()))
        {
            edition_autorisee = false;
            for(int i = 0 ; i < liste_questions_tableau.size() ; ++i)
            {
                if(liste_questions_tableau.at(i) == liste_questions.first())
                {
                    if(!liste_questions_tableau_inverse.at(i))// si la question a été posée normalement
                    {
                        tableau_principal->item(i, 0)->setText(liste_questions.first()->question);
                        tableau_principal->item(i, 1)->setText(liste_questions.first()->reponse);
                    }
                    else
                    {
                        tableau_principal->item(i, 0)->setText(liste_questions.first()->reponse);
                        tableau_principal->item(i, 1)->setText(liste_questions.first()->question);
                    }
                }
            }
            edition_autorisee = true;

            // Maintenant, la liste est modifiée
            modifier_liste(true);
            reposer_question();//on pose la question dans le meme sens
        }
        delete dialog_ajouter_question;
    }
}

void MainWindow::modifier_questions_selectionnees()// DONE
{
    edition_autorisee = false;

    Dialog_ajouter_question *dialog_ajouter_question = new Dialog_ajouter_question(&liste_tags, &liste_tags_defaut, &claviers_virtuels, this);

    QList<Question*> liste_question_deja_passees;
    for(int i = 0 ; i < tableau_principal->selectedItems().size() ; ++i)
    {
        int ligne = tableau_principal->selectedItems().at(i)->row();
        if(liste_questions_tableau.at(ligne) != NULL)// Si la ligne n'est pas vide
        {
            bool deja_pasee = false;
            for(int j = 0 ; j < liste_question_deja_passees.size() ; ++j)// On ne prend qu'une seule ligne chaque fois...
            {
                if(liste_question_deja_passees.at(j) == liste_questions_tableau.at(ligne))//...donc on regarde si on a déjà vue la question
                {
                    deja_pasee = true;
                }
            }
            if(!deja_pasee)
            {
                if(dialog_ajouter_question->exec(liste_questions_tableau.at(ligne)))
                {
                    for(int j = 0 ; j < liste_questions_tableau.size() ; ++j)// On parcour le tableau et modifie toutes les lignes corespondant à la question que l'on à modifié
                    {
                        if(liste_questions_tableau.at(j) == liste_questions_tableau.at(ligne))
                        {
                            if(!liste_questions_tableau_inverse.at(j))// si la question a été posée normalement
                            {
                                tableau_principal->item(j, 0)->setText(liste_questions_tableau.at(ligne)->question);
                                tableau_principal->item(j, 1)->setText(liste_questions_tableau.at(ligne)->reponse);
                            }
                            else
                            {
                                tableau_principal->item(j, 0)->setText(liste_questions_tableau.at(ligne)->reponse);
                                tableau_principal->item(j, 1)->setText(liste_questions_tableau.at(ligne)->question);
                            }
                        }
                    }
                    modifier_liste(true);
                }
                liste_question_deja_passees.append(liste_questions_tableau.at(ligne));//maintenant, la question est déjà passée
            }
        }
    }
    edition_autorisee = true;

    reposer_question();// on repose la question meme si on n'a pas pas modifié la liste(dans ce cas, repose la meme question dans le bon ordre)
    tableau_principal->clearSelection();

    delete dialog_ajouter_question;
}

void MainWindow::ne_plus_voir_question_actuelle()
{
    ne_plus_voir_cette_question();
}

void MainWindow::ne_plus_voir_questions_selectionnees()
{
    QList<Question*> liste_question_deja_pasee;
    for(int i = 0 ; i < tableau_principal->selectedItems().size() ; ++i)
    {
        int ligne = tableau_principal->selectedItems().at(i)->row();
        // (!) Remarque : si on a séléctionné plusieurs fois la meme question(et c'est le cas car on séléctionne plusieurs cases) elle N'est PAS supprimée et rajoutée plusieurs fois
        if(liste_questions_tableau.at(ligne) != NULL)
        {
            bool deja_passee = false;
            for(int j = 0 ; j < liste_question_deja_pasee.size() ; ++j)
            {
                if(liste_question_deja_pasee.at(j) == liste_questions_tableau.at(ligne))
                {
                    deja_passee = true;
                    break;
                }
            }
            if(!deja_passee)
            {
                liste_questions_tableau.at(ligne)->score = SCORE_MAX;
                if(liste_questions.removeOne(liste_questions_tableau.at(ligne)))//si la question est bien dans la liste(et pas dans la liste des questions supprimées)
                {
                    liste_questions.append(liste_questions_tableau.at(ligne));//on replace la question à la fin de la liste
                }
                liste_question_deja_pasee.prepend(liste_questions_tableau.at(ligne));//la liste est maintenant passée
            }
        }
    }

    tableau_principal->clearSelection();
    reposer_question();// au cas ou la question posée fait partie des questions séléctionnée
}

void MainWindow::inverser_reponse_derniere_question()
{
    if(liste_questions_tableau.last() != NULL && tableau_principal->item(liste_questions_tableau.size() - 1, 0)->backgroundColor() != COULEUR_FILTREE)
    {
        if(tableau_principal->item(liste_questions_tableau.size()-1, 0)->backgroundColor() == liste_couleurs[SCORE_COULEUR_RATE])// Si on avait faux
        {
            // Actualisation du score
            liste_questions_tableau.last()->score = derniere_question_score_avant;
            liste_questions_tableau.last()->nombre_de_fois_posee--;
            // On replace la question en première position pour pouvoir y répondre de nouveau
            liste_questions.removeOne(liste_questions_tableau.last());
            liste_questions.prepend(liste_questions_tableau.last());//normalement, ces deux lignes ne servent à rien(la question est déja au début car on a eu faux) mais ca peut servir si je modifie l'alogorythme
            // Décalage des lignes du tableau
            liste_questions_tableau.pop_back();
            liste_questions_tableau.push_front(NULL);
            edition_autorisee = false;
            tableau_principal->insertRow(0);
            delete tableau_principal->item(liste_questions_tableau.size(), 0);
            delete tableau_principal->item(liste_questions_tableau.size(), 1);
            delete tableau_principal->item(liste_questions_tableau.size(), 2);
            tableau_principal->removeRow(liste_questions_tableau.size());// pas de "-1" car le tableau fait une ligne de plus que liste_questions_tableau
            edition_autorisee = true;
            // On réactualise comme si on avais eu bon
            actualiser(true);
        }
        else
        {
            bouton_je_me_suis_trompe_autorise = true;
            je_me_suis_trompe();
        }
        modifier_liste(true);
    }
}

void MainWindow::sauter_question_actuelle()
{
    sauter_cette_question();
}

void MainWindow::vider_liste()// DONE
{
    if(!liste_questions.isEmpty() || !liste_questions_interdites.isEmpty())
    {
        if(QMessageBox::warning(this,
                                trUtf8("Confirmation"),
                                trUtf8("Êtes vous sûr de vouloir effacer <strong>toute</strong> la liste ?"),
                                QMessageBox::Yes | QMessageBox::Cancel) == QMessageBox::Yes)
        {
            vider_liste_completement();
        }
    }
    else
    {
        vider_liste_completement();// Si la liste est vide, on efface quand meme le tableau
    }
}

void MainWindow::melanger_liste()
{
    Dialog_trier *dialog_trier = new Dialog_trier(&liste_questions, this);
    if(dialog_trier->exec())
    {
        reposer_question();// la question n'est plus la meme(normalement) donc peut etre faut il mieux utiliser poser_nouvelle_question
    }
    delete dialog_trier;
}

void MainWindow::melanger_liste_defaut()
{
}

void MainWindow::filtrer_liste()
{
    Dialog_filtrer *dialog_filtrer = new Dialog_filtrer(&liste_filtres, &liste_tags, this);
    if(dialog_filtrer->exec())
    {
        // TODO : récupération du choix de l'utilisateur
        actualiser_filtre();
        reposer_question();
    }
    delete dialog_filtrer;
}

void MainWindow::inverser_filtre()
{
    // TODO
}

void MainWindow::reinitialiser_score()
{
    if(!liste_questions.isEmpty() || !liste_questions_interdites.isEmpty())
    {
        Dialog_reinitialiser *dialog_reinitialiser = new Dialog_reinitialiser(&liste_questions, &liste_questions_interdites, this);
        if(dialog_reinitialiser->exec())
        {
            modifier_liste(true);
            actualiser_filtre();
        }
        delete dialog_reinitialiser;
    }
}

void MainWindow::statistiques_bonnes_reponses()// TODO : Completer le contenu
{
    if(!liste_questions.isEmpty())
    {
        // Calcul des statistiques
        float nombre_question_sues_parfaitement = 0.0;
        float nombre_question_sues_tres_bien = 0.0;
        float nombre_question_sues_bien = 0.0;
        float nombre_question_sues_assez_bien = 0.0;
        float nombre_question_sues_moyenement = 0.0;
        float nombre_question_sues = 0.0;
        for(int i = 0 ; i < liste_questions.size() ; ++i)
        {
            if(liste_questions.at(i)->score > 0)
            {
                nombre_question_sues++;
                if(liste_questions.at(i)->score > 1)
                {
                    nombre_question_sues_moyenement++;
                    if(liste_questions.at(i)->score > 2)
                    {
                        nombre_question_sues_assez_bien++;
                        if(liste_questions.at(i)->score > 3)
                        {
                            nombre_question_sues_bien++;
                            if(liste_questions.at(i)->score > 4)
                            {
                                nombre_question_sues_tres_bien++;
                                if(liste_questions.at(i)->score > 5)
                                {
                                    nombre_question_sues_parfaitement++;
                                }
                            }
                        }
                    }
                }
            }
        }
        float pour_centage_question_sues_parfaitement = (nombre_question_sues_parfaitement / liste_questions.size()) * 100;
        float pour_centage_question_sues_tres_bien = (nombre_question_sues_tres_bien / liste_questions.size()) * 100;
        float pour_centage_question_sues_bien = (nombre_question_sues_bien / liste_questions.size()) * 100;
        float pour_centage_question_sues_assez_bien = (nombre_question_sues_assez_bien / liste_questions.size()) * 100;
        float pour_centage_question_sues_moyenement = (nombre_question_sues_moyenement / liste_questions.size()) * 100;
        float pour_centage_question_sues = (nombre_question_sues / liste_questions.size()) * 100;

        QDialog *dialog_statistiques_bonnes_reponses = new QDialog(this);

        //propriétés général de la fenetre
        dialog_statistiques_bonnes_reponses->setWindowTitle(trUtf8("Statistiques des bonnes réponses"));

        //on initialise les éléments de la boite de dialogue
        QLabel *texte_question_remarques = new QLabel(trUtf8("<strong>Attention</strong> : Ces statistiques ne prennent pas en compte les questions filtrées"));
        QLabel *texte_question_sues_parfaitement_pour_cent = new QLabel(trUtf8("Pourcentage de question parfaitement sues(score > 5) : ") + QString::number(pour_centage_question_sues_parfaitement) + "%");
        QLabel *texte_question_sues_sues_tres_bien_pour_cent = new QLabel(trUtf8("Pourcentage de question parfaitement très bien sues(score > 4) : ") + QString::number(pour_centage_question_sues_tres_bien) + "%");
        QLabel *texte_question_sues_bien_pour_cent = new QLabel(trUtf8("Pourcentage de question parfaitement bien sues(score > 3) : ") + QString::number(pour_centage_question_sues_bien) + "%");
        QLabel *texte_question_sues_assez_bien_pour_cent = new QLabel(trUtf8("Pourcentage de question assez bien sues(score > 2) : ") + QString::number(pour_centage_question_sues_assez_bien) + "%");
        QLabel *texte_question_sues_moyenement_pour_cent = new QLabel(trUtf8("Pourcentage de question moyenement sues(score > 1) : ") + QString::number(pour_centage_question_sues_moyenement) + "%");
        QLabel *texte_question_sues_pour_cent = new QLabel(trUtf8("Pourcentage de question sues(score > 0) : ") + QString::number(pour_centage_question_sues) + "%");

        QPushButton *bouton_fermer = new QPushButton(trUtf8("Fermer"));
        connect(bouton_fermer, SIGNAL(clicked()), dialog_statistiques_bonnes_reponses, SLOT(close()));

        //on place les elements dans la fenetre
        QVBoxLayout *layout_contenu = new QVBoxLayout;
        layout_contenu->addWidget(texte_question_sues_parfaitement_pour_cent);
        layout_contenu->addWidget(texte_question_sues_sues_tres_bien_pour_cent);
        layout_contenu->addWidget(texte_question_sues_bien_pour_cent);
        layout_contenu->addWidget(texte_question_sues_assez_bien_pour_cent);
        layout_contenu->addWidget(texte_question_sues_moyenement_pour_cent);
        layout_contenu->addWidget(texte_question_sues_pour_cent);

        QFrame *cadre_question_sues_pour_cent = new QFrame;
        cadre_question_sues_pour_cent->setFrameShape(QFrame::StyledPanel);
        cadre_question_sues_pour_cent->setFrameShadow(QFrame::Raised);
        cadre_question_sues_pour_cent->setLayout(layout_contenu);

        QHBoxLayout *layout_boutons = new QHBoxLayout;
        layout_boutons->addStretch();
        layout_boutons->addWidget(bouton_fermer);
        layout_boutons->addStretch();//(!) : rajouter 2 espaces de cette facon n'est peut etre pas la meilleur solution

        QVBoxLayout *layout_principal = new QVBoxLayout;
        layout_principal->addWidget(texte_question_remarques);
        layout_principal->addWidget(cadre_question_sues_pour_cent);
        layout_principal->addLayout(layout_boutons);

        dialog_statistiques_bonnes_reponses->setLayout(layout_principal);

        dialog_statistiques_bonnes_reponses->setFixedSize(dialog_statistiques_bonnes_reponses->sizeHint());

        dialog_statistiques_bonnes_reponses->exec();
        delete dialog_statistiques_bonnes_reponses;
    }
    else// Si la liste est vide
    {
        QMessageBox::critical(this, trUtf8("Erreur"), trUtf8("Impossible de calculer les statistiques des questions : la liste est vide."));
    }
}

void MainWindow::liste_des_questions()
{
}

void MainWindow::ordre(QAction *action_activee)// DONE ?
{
    if(action_activee == action_ordre_question_reponse)
    {
        valeur_inversion = 0;
    }
    else if(action_activee == action_ordre_question_reponse_75_pour_cent)
    {
        valeur_inversion = 25;
    }
    else if(action_activee == action_ordre_aleatoire)
    {
        valeur_inversion = 50;
    }
    else if(action_activee == action_ordre_reponse_question_75_pour_cent)
    {
        valeur_inversion = 75;
    }
    else if(action_activee == action_ordre_reponse_question)
    {
        valeur_inversion = 100;
    }
    poser_nouvelle_question();
}

void MainWindow::affichage_duree(QAction *action_activee)
{
    if(action_activee == action_affichage_duree_jours)
    {
        valeur_duree = QDateTime::currentDateTime().addDays(-1);
    }
    else if(action_activee == action_affichage_duree_semaine)
    {
        valeur_duree = QDateTime::currentDateTime().addDays(-7);
    }
    else if(action_activee == action_affichage_duree_mois)
    {
        valeur_duree = QDateTime::currentDateTime().addMonths(-1);
    }
    else if(action_activee == action_affichage_duree_annee)
    {
        valeur_duree = QDateTime::currentDateTime().addYears(-1);
    }
    else if(action_activee == action_affichage_duree_tous)
    {
        valeur_duree = QDateTime();
    }
    actualiser_filtre();
    reposer_question();//actualisation de l'affichage
}

void MainWindow::affichage_duree_personnalise()
{
}

void MainWindow::definir_tags_defaut()
{
    QDialog *dialog_tags_defaut = new QDialog(this);
    dialog_tags_defaut->setWindowTitle(trUtf8("Tags par defaut"));

    QLabel *text = new QLabel(trUtf8("Tags par defaut à l'ajout d'une nouvelle question : "));
    Widget_tags *widget_tags = new Widget_tags(&liste_tags);
    widget_tags->champ_tags->setText(liste_tags_defaut.join(", "));
    QPushButton *bouton_valider = new QPushButton(trUtf8("Valider"));
    QPushButton *bouton_annuler = new QPushButton(trUtf8("Annuler"));
    connect(bouton_valider, SIGNAL(clicked()), dialog_tags_defaut, SLOT(accept()));
    connect(bouton_annuler, SIGNAL(clicked()), dialog_tags_defaut, SLOT(reject()));

    QHBoxLayout *layout_boutons = new QHBoxLayout;
    layout_boutons->addStretch();
    layout_boutons->addWidget(bouton_valider);
    layout_boutons->addWidget(bouton_annuler);

    QVBoxLayout *layout_principal = new QVBoxLayout;
    layout_principal->addWidget(text);
    layout_principal->addWidget(widget_tags);
    layout_principal->addLayout(layout_boutons);

    dialog_tags_defaut->setLayout(layout_principal);
    widget_tags->afficher_liste_tags(false);

    if(dialog_tags_defaut->exec())
    {
        liste_tags_defaut = widget_tags->to_string_list();
        for(int i = 0 ; i < liste_tags_defaut.size() ; ++i)
        {
            bool existe_deja = false;
            for(int j = 0 ; j < liste_tags.size() ; ++j)
            {
                if(liste_tags.at(j) == liste_tags_defaut.at(i))
                {
                    existe_deja = true;
                    break;
                }
            }
            if(!existe_deja)
            {
                liste_tags.append(liste_tags_defaut.at(i));
            }
        }
    }
}

void MainWindow::ajouter_supprimer_tags(bool supprimer)
{
    QDialog *dialog_ajouter_supprimer_tags = new QDialog(this);

    QComboBox *liste_deroulante_ajouter_supprimer = new QComboBox;
    liste_deroulante_ajouter_supprimer->addItems(QStringList() << trUtf8("Ajouter les tags des/de la question(s)...") << trUtf8("Supprimer les tags des/de la question(s)..."));
    QCheckBox *checkbox_liste_filtree = new QCheckBox(trUtf8("...de la liste filtree"));
    QCheckBox *checkbox_liste_non_filtree = new QCheckBox(trUtf8("...de la liste non filtree"));
    QCheckBox *checkbox_selectionnees = new QCheckBox(trUtf8("...séléctionnée(s)"));
    checkbox_selectionnees->setChecked(supprimer);
    Widget_tags *widget_tags = new Widget_tags(&liste_tags);

    QPushButton *bouton_valider = new QPushButton(trUtf8("Valider"));
    QPushButton *bouton_annuler = new QPushButton(trUtf8("Annuler"));
    connect(bouton_valider, SIGNAL(clicked()), dialog_ajouter_supprimer_tags, SLOT(accept()));
    connect(bouton_annuler, SIGNAL(clicked()), dialog_ajouter_supprimer_tags, SLOT(reject()));

    QHBoxLayout *layout_boutons = new QHBoxLayout;
    layout_boutons->addStretch();
    layout_boutons->addWidget(bouton_valider);
    layout_boutons->addWidget(bouton_annuler);

    QVBoxLayout *layout_principal = new QVBoxLayout;
    layout_principal->addWidget(liste_deroulante_ajouter_supprimer);
    layout_principal->addWidget(checkbox_liste_filtree);
    layout_principal->addWidget(checkbox_liste_non_filtree);
    layout_principal->addWidget(checkbox_selectionnees);
    layout_principal->addWidget(widget_tags);
    layout_principal->addLayout(layout_boutons);

    dialog_ajouter_supprimer_tags->setLayout(layout_principal);

    widget_tags->afficher_liste_tags(false);

    if(dialog_ajouter_supprimer_tags->exec())
    {
        QStringList liste_tags_a_modifier = widget_tags->to_string_list();
        if(checkbox_liste_filtree->isChecked())
        {
            for(int i = 0 ; i < liste_questions_interdites.size() ; ++i)
            {
                for(int j = 0 ; j < liste_tags_a_modifier.size() ; ++j)
                {
                    if(liste_deroulante_ajouter_supprimer->currentIndex() == 0)
                    {
                        if(!liste_questions_interdites.at(i)->tags.contains(liste_tags_a_modifier.at(j)))
                        {
                            liste_questions_interdites.at(i)->tags.append(liste_tags_a_modifier.at(j));
                        }
                    }
                    else if(liste_deroulante_ajouter_supprimer->currentIndex() == 1)
                    {
                        liste_questions_interdites.at(i)->tags.removeOne(liste_tags_a_modifier.at(j));
                    }
                }
            }
        }
        if(checkbox_liste_non_filtree->isChecked())
        {
            for(int i = 0 ; i < liste_questions.size() ; ++i)
            {
                for(int j = 0 ; j < liste_tags_a_modifier.size() ; ++j)
                {
                    if(liste_deroulante_ajouter_supprimer->currentIndex() == 0)
                    {
                        if(!liste_questions.at(i)->tags.contains(liste_tags_a_modifier.at(j)))
                        {
                            liste_questions.at(i)->tags.append(liste_tags_a_modifier.at(j));
                        }
                    }
                    else if(liste_deroulante_ajouter_supprimer->currentIndex() == 1)
                    {
                        liste_questions.at(i)->tags.removeOne(liste_tags_a_modifier.at(j));
                    }
                }
            }
        }
        if(checkbox_selectionnees->isChecked())
        {
            QList<Question*> liste_question_deja_pasee;
            for(int i = 0 ; i < tableau_principal->selectedItems().size() ; ++i)
            {
                int ligne = tableau_principal->selectedItems().at(i)->row();
                if(liste_questions_tableau.at(ligne) != NULL)
                {
                    bool deja_passee = false;
                    for(int j = 0 ; j < liste_question_deja_pasee.size() ; ++j)
                    {
                        if(liste_question_deja_pasee.at(j) == liste_questions_tableau.at(ligne))
                        {
                            deja_passee = true;
                            break;
                        }
                    }
                    if(!deja_passee)
                    {
                        for(int j = 0 ; j < liste_tags_a_modifier.size() ; ++j)
                        {
                            if(liste_deroulante_ajouter_supprimer->currentIndex() == 0)
                            {
                                if(!liste_questions_tableau.at(ligne)->tags.contains(liste_tags_a_modifier.at(j)))
                                {
                                    liste_questions_tableau.at(ligne)->tags.append(liste_tags_a_modifier.at(j));
                                }
                            }
                            else if(liste_deroulante_ajouter_supprimer->currentIndex() == 1)
                            {
                                liste_questions_tableau.at(ligne)->tags.removeOne(liste_tags_a_modifier.at(j));
                            }
                        }
                    }
                }
            }
        }
        if(liste_deroulante_ajouter_supprimer->currentIndex() == 0)
        {
            for(int i = 0 ; i < liste_tags_a_modifier.size() ; ++i)
            {
                if(!liste_tags.contains(liste_tags_a_modifier.at(i)))
                {
                    liste_tags.append(liste_tags_a_modifier.at(i));
                }
            }
        }
        if(!liste_tags_a_modifier.isEmpty() &&
           (checkbox_liste_filtree->isChecked() ||
            checkbox_liste_non_filtree->isChecked() ||
            (checkbox_selectionnees->isChecked() && tableau_principal->selectedItems().isEmpty())))
        {
            modifier_liste(true);
        }
    }
    delete dialog_ajouter_supprimer_tags;
}

void MainWindow::supprimer_question_avec_tags()
{
    QDialog *dialog_supprimer_question_avec_tags = new QDialog(this);
    dialog_supprimer_question_avec_tags->setWindowTitle(trUtf8("Supprimer questions avec tags"));

    QLabel *text = new QLabel(trUtf8("Supprimer toutes les questions avec les tags suivants : "));
    Widget_tags *widget_tags = new Widget_tags(&liste_tags);
    QCheckBox *check_box_liste_filtree = new QCheckBox(trUtf8("Supprimer aussi les questions de la liste filtrée"));
    check_box_liste_filtree->setChecked(true);
    QPushButton *bouton_valider = new QPushButton(trUtf8("Valider"));
    QPushButton *bouton_annuler = new QPushButton(trUtf8("Annuler"));
    connect(bouton_valider, SIGNAL(clicked()), dialog_supprimer_question_avec_tags, SLOT(accept()));
    connect(bouton_annuler, SIGNAL(clicked()), dialog_supprimer_question_avec_tags, SLOT(reject()));

    QHBoxLayout *layout_boutons = new QHBoxLayout;
    layout_boutons->addStretch();
    layout_boutons->addWidget(bouton_valider);
    layout_boutons->addWidget(bouton_annuler);

    QVBoxLayout *layout_principal = new QVBoxLayout;
    layout_principal->addWidget(text);
    layout_principal->addWidget(widget_tags);
    layout_principal->addWidget(check_box_liste_filtree);
    layout_principal->addLayout(layout_boutons);

    dialog_supprimer_question_avec_tags->setLayout(layout_principal);
    widget_tags->afficher_liste_tags(false);

    if(dialog_supprimer_question_avec_tags->exec())
    {
        QStringList liste_tags_a_supprimer = widget_tags->to_string_list();
        for(int i = 0 ; i < liste_tags_a_supprimer.size() ; ++i)
        {
            if(!liste_tags_a_supprimer.isEmpty() &&
               ((liste_questions.isEmpty() && check_box_liste_filtree->isChecked() && !liste_questions_interdites.isEmpty()) ||
               (!liste_questions.isEmpty())) &&
               QMessageBox::warning(this, "Confirmation", "Êtes vous sûr de vouloir supprimer les questions avec tags ?", QMessageBox::Yes | QMessageBox::Cancel) == QMessageBox::Yes)
            {
                for(int j = 0 ; j < liste_questions.size() ; ++j)
                {
                    for(int k = 0 ; k < liste_questions.at(j)->tags.size() ; ++k)
                    {
                        if(liste_questions.at(j)->tags.at(k) == liste_tags_a_supprimer.at(i))
                        {
                            edition_autorisee = false;
                            for(int l = 0 ; l < liste_questions_tableau.size() ; ++l)
                            {
                                if(liste_questions_tableau.at(l) == liste_questions.at(j))
                                {
                                    liste_questions_tableau[l] = NULL;// supprime toutes les questions que l'on a supprimé
                                    tableau_principal->item(l, 0)->setBackgroundColor(COULEUR_SUPPRIMEE);
                                    tableau_principal->item(l, 1)->setBackgroundColor(COULEUR_SUPPRIMEE);
                                    tableau_principal->item(l, 2)->setBackgroundColor(COULEUR_SUPPRIMEE);
                                }
                            }
                            edition_autorisee = true;

                            delete liste_questions.at(j);
                            liste_questions.removeAt(j);
                            j--;
                            break;
                        }
                    }
                }
                if(check_box_liste_filtree->isChecked())
                {
                    for(int j = 0 ; j < liste_questions_interdites.size() ; ++j)
                    {
                        for(int k = 0 ; k < liste_questions_interdites.at(j)->tags.size() ; ++k)
                        {
                            if(liste_questions_interdites.at(j)->tags.at(k) == liste_tags_a_supprimer.at(i))
                            {

                            edition_autorisee = false;
                            for(int l = 0 ; l < liste_questions_tableau.size() ; ++l)
                            {
                                if(liste_questions_tableau.at(l) == liste_questions_interdites.at(j))
                                {
                                    liste_questions_tableau[l] = NULL;// supprime toutes les questions que l'on a supprimé
                                    tableau_principal->item(l, 0)->setBackgroundColor(COULEUR_SUPPRIMEE);
                                    tableau_principal->item(l, 1)->setBackgroundColor(COULEUR_SUPPRIMEE);
                                    tableau_principal->item(l, 2)->setBackgroundColor(COULEUR_SUPPRIMEE);
                                }
                            }
                            edition_autorisee = true;

                                delete liste_questions_interdites.at(j);
                                liste_questions_interdites.removeAt(j);
                                j--;
                                break;
                            }
                        }
                    }
                }
            }
        }
        modifier_liste(true);
        reposer_question();
    }
}

void MainWindow::gerer_tags()
{
    QList<Tag_modifie> modifications_tag;
    Dialog_gerer_tags *dialog_gerer_tags = new Dialog_gerer_tags(&liste_tags, &modifications_tag, this);
    if(dialog_gerer_tags->exec())
    {
        for(int i = 0 ; i < modifications_tag.size() ; ++i)
        {
            if(modifications_tag.at(i).nouveau_tag == "")// si on a supprimé le tag
            {
                for(int j = 0 ; j < liste_questions.size() ; ++j)
                {
                    liste_questions.at(j)->tags.removeOne(modifications_tag.at(i).ancien_tag);
                }
                for(int j = 0 ; j < liste_questions_interdites.size() ; ++j)
                {
                    liste_questions_interdites.at(j)->tags.removeOne(modifications_tag.at(i).ancien_tag);
                }
                liste_tags.removeOne(modifications_tag.at(i).ancien_tag);
            }
            else if(modifications_tag.at(i).ancien_tag == "")// si on a ajouter le tag
            {
                liste_tags.append(modifications_tag.at(i).nouveau_tag);
            }
            else// si on a modifier le tag
            {
                for(int j = 0 ; j < liste_questions.size() ; ++j)
                {
                    if(liste_questions.at(j)->tags.contains(modifications_tag.at(i).ancien_tag))
                    {
                        liste_questions.at(j)->tags.replace(liste_questions.at(j)->tags.indexOf(modifications_tag.at(i).ancien_tag), modifications_tag.at(i).nouveau_tag);
                    }
                }
                for(int j = 0 ; j < liste_questions_interdites.size() ; ++j)
                {
                    if(liste_questions_interdites.at(j)->tags.contains(modifications_tag.at(i).ancien_tag))
                    {
                        liste_questions_interdites.at(j)->tags.replace(liste_questions_interdites.at(j)->tags.indexOf(modifications_tag.at(i).ancien_tag), modifications_tag.at(i).nouveau_tag);
                    }
                }
                liste_tags.replace(liste_tags.indexOf(modifications_tag.at(i).ancien_tag), modifications_tag.at(i).nouveau_tag);
            }
        }
        if(!modifications_tag.isEmpty())
        {
            modifier_liste(true);
        }
    }
    delete dialog_gerer_tags;
}

void MainWindow::gerer_claviers_virtuels()
{
    Dialog_gerer_claviers_virtuels *dialog_gerer_claviers_virtuels = new Dialog_gerer_claviers_virtuels(&claviers_virtuels, this);
    dialog_gerer_claviers_virtuels->exec();
    delete dialog_gerer_claviers_virtuels;
}

void MainWindow::calcul_automatique_difficultee(bool nouvelle_valeur)
{
    difficultee_calculee_automatiquement = nouvelle_valeur;
}

void MainWindow::actualiser_filtre_en_permanace(bool nouvelle_valeur)
{
    filtre_actualise_en_permanace = nouvelle_valeur;
}

void MainWindow::ajouter_a_la_fin(bool nouvelle_valeur)
{
    questions_ajoutees_a_la_fin = nouvelle_valeur;
}

void MainWindow::tolerance(bool nouvelle_tolerance)// DONE
{
    tolerant = nouvelle_tolerance;
}

void MainWindow::tolerance_accents(bool nouvelle_tolerance_accents)// DONE
{
    accents_facultatifs = nouvelle_tolerance_accents;
}

void MainWindow::tolerance_majuscules(bool nouvelle_tolerance_majuscules)
{
    majuscules_facultatives = nouvelle_tolerance_majuscules;
}

void MainWindow::preferences()
{
}

void MainWindow::manuel()
{
}

void MainWindow::aide_en_ligne()
{
    QDesktopServices::openUrl(QUrl("http://vocabpp.free.fr/home.php?page=aide"));
}

void MainWindow::rechercher_mise_a_jour()
{
}

void MainWindow::a_propos()// DONE(mais il faudrais un peu enrichir le texte)
{
    QDialog *dialog_a_propos = new QDialog(this);

    //propriétés général de la fenetre
    dialog_a_propos->setWindowTitle(trUtf8("A propos"));
    dialog_a_propos->setFixedSize(dialog_a_propos->sizeHint());

    //on initialise les éléments de la boite de dialogue
    QLabel *contenu_a_propos = new QLabel(trUtf8("<h1>A Propos</h1> <p>Vous utilisez Vocab++ version 0.3.4<br />Cette version est sous licence GPL V3<br />Pour plus d'information, consultez le site <a href=\"http://vocabpp.free.fr\">vocabpp.free.fr</a></p>"));
    contenu_a_propos->setOpenExternalLinks(true);

    QTextEdit *copying = new QTextEdit(
    "Vocab++ is a programm to learn vocabulary<br />"
    "Copyright (C) 2009  Pot Etienne<br /><br />"

    "This program is free software: you can redistribute it and/or modify<br />"
    "it under the terms of the GNU General Public License as published by<br />"
    "the Free Software Foundation, either version 3 of the License, or<br />"
    "(at your option) any later version.<br /><br />"

    "This program is distributed in the hope that it will be useful,<br />"
    "but WITHOUT ANY WARRANTY; without even the implied warranty of<br />"
    "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the<br />"
    "GNU General Public License for more details.<br /><br />"

    "You should have received a copy of the GNU General Public License<br />"
    "along with this program.  If not, see <a href=\"http://www.gnu.org/licenses/\">http://www.gnu.org/licenses/</a>.");
    copying->setReadOnly(true);
    copying->setLineWrapMode(QTextEdit::NoWrap);
    copying->setFixedWidth(500);
    copying->setFixedHeight(170);

    QPushButton *bouton_fermer = new QPushButton(trUtf8("Fermer"));
    connect(bouton_fermer, SIGNAL(clicked()), dialog_a_propos, SLOT(close()));

    //on place les elements dans la fenetre
    QHBoxLayout *layout_boutons = new QHBoxLayout;
    layout_boutons->addStretch();
    layout_boutons->addWidget(bouton_fermer);
    layout_boutons->addStretch();//(!) (!) (!) : rajouter 2 espaces de cette facon n'est peut etre pas la meilleur solution

    QVBoxLayout *layout_principal = new QVBoxLayout;
    layout_principal->addWidget(contenu_a_propos);
    layout_principal->addWidget(copying);
    layout_principal->addLayout(layout_boutons);
    dialog_a_propos->setLayout(layout_principal);

    dialog_a_propos->exec();
    delete dialog_a_propos;
}

void MainWindow::creer_widget_principal()
{
    liste_questions_tableau.resize(settings.value("Affichage/nombre_de_ligne_tableau", DEFAUT_NOMBRE_DE_LIGNE_TABLEAU).toInt());
    for(int i = 0 ; i < liste_questions_tableau.size() ; ++i)
    {
        liste_questions_tableau[i] = NULL;
    }
    liste_questions_tableau_inverse.resize(liste_questions_tableau.size());
    liste_questions_tableau_score.resize(liste_questions_tableau.size());

    // Création des widget de la fenetre
    tableau_principal = new QTableWidget(liste_questions_tableau.size(), 3);
    tableau_principal->setHorizontalHeaderLabels(QStringList() << trUtf8("Question") << trUtf8("Solution") << trUtf8("Ma réponse"));// Entete du tableau
    tableau_principal->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
    tableau_principal->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);//mode de defiliment doux(pixel par pixel)
    tableau_principal->setSelectionBehavior(QAbstractItemView::SelectRows);
    tableau_principal->verticalHeader()->setResizeMode(QHeaderView::Fixed);//on ne peut pas redimentionner le tableau en hauteur
    tableau_principal->horizontalHeader()->setResizeMode(QHeaderView::Fixed);//ni en largeur
    actualiser_header_verticals();
    connect(tableau_principal, SIGNAL(cellChanged(int, int)), this, SLOT(question_editee(int, int)));

    texte_question_permanent = new QLabel(trUtf8("Question : "));
    texte_question = new QLabel;
    texte_reponse = new QLabel(trUtf8("Réponse"));
    champ_reponse = new QLineEdit;
    bouton_changer = new QPushButton(trUtf8(">>"));
    bouton_supprimer = new QPushButton(trUtf8("Supprimer"));
    bouton_modifier = new QPushButton(trUtf8("Modifier"));
    bouton_ne_plus_voir = new QPushButton(trUtf8("Ne plus voir"));
    bouton_valider = new QPushButton(trUtf8("Valider"));
    bouton_effacer = new QPushButton(trUtf8("Effacer"));

    if(!settings.value("Affichage/bouton_changer", DEFAUT_BOUTON_CHANGER).toBool())
    {
        bouton_changer->hide();
    }
    if(!settings.value("Affichage/bouton_supprimer", DEFAUT_BOUTON_SUPPRIMER).toBool())
    {
        bouton_supprimer->hide();
    }
    if(!settings.value("Affichage/bouton_modifier", DEFAUT_BOUTON_MODIFIER).toBool())
    {
        bouton_modifier->hide();
    }
    if(!settings.value("Affichage/bouton_ne_plus_voir", DEFAUT_BOUTON_NE_PLUS_VOIR).toBool())
    {
        bouton_ne_plus_voir->hide();
    }
    if(!settings.value("Affichage/bouton_effacer", DEFAUT_BOUTON_EFFACER).toBool())
    {
        bouton_effacer->hide();
    }

    connect(champ_reponse,           SIGNAL(returnPressed()), this,          SLOT(valider()));
    connect(bouton_changer,          SIGNAL(clicked()),       this,          SLOT(sauter_cette_question()));
    connect(bouton_supprimer,        SIGNAL(clicked()),       this,          SLOT(supprimer_question_actuelle()));
    connect(bouton_modifier,         SIGNAL(clicked()),       this,          SLOT(modifier_question_actuelle()));
    connect(bouton_ne_plus_voir,     SIGNAL(clicked()),       this,          SLOT(ne_plus_voir_question_actuelle()));
    connect(bouton_valider,          SIGNAL(clicked()),       this,          SLOT(valider()));
    connect(bouton_effacer,          SIGNAL(clicked()),       champ_reponse, SLOT(clear()));

    bouton_je_sais = new QPushButton(trUtf8("Je pense savoir"));
    bouton_je_ne_sais_pas = new QPushButton(trUtf8("Je ne sais pas"));
    bouton_je_me_suis_trompe = new QPushButton(trUtf8("Je me suis trompé"));
    bouton_ne_plus_voir_cette_question = new QPushButton(trUtf8("Ne plus voir cette question"));
    bouton_sauter_cette_question = new QPushButton(trUtf8("Aller à la question suivante"));
    connect(bouton_je_sais,                        SIGNAL(clicked()), this, SLOT(je_sais()));
    connect(bouton_je_ne_sais_pas,                 SIGNAL(clicked()), this, SLOT(je_ne_sais_pas()));
    connect(bouton_je_me_suis_trompe,              SIGNAL(clicked()), this, SLOT(je_me_suis_trompe()));
    connect(bouton_ne_plus_voir_cette_question,    SIGNAL(clicked()), this, SLOT(ne_plus_voir_cette_question()));
    connect(bouton_sauter_cette_question,          SIGNAL(clicked()), this, SLOT(sauter_cette_question()));

    if(!settings.value("Affichage/bouton_sauter_cette_question", DEFAUT_SAUTER_CETTE_QUESTION).toBool())
    {
        bouton_sauter_cette_question->hide();
    }

    bouton_ajouter_question = new QPushButton(trUtf8("Ajouter une/des question(s)..."));
    bouton_supprimer_question_actuelle = new QPushButton(trUtf8("Supprimer la question actuelle"));
    bouton_modifier_question_actuelle = new QPushButton(trUtf8("Modifier la question actuelle"));
    bouton_vider_liste = new QPushButton(trUtf8("Vider la liste"));
    bouton_melanger_liste = new QPushButton(trUtf8(">>"));// TODO !!!!!!!!
    bouton_melanger_liste_defaut = new QPushButton(trUtf8(">>"));// TODO !!!!!!!!
    bouton_reinitialiser_score = new QPushButton(trUtf8(">>"));
    bouton_statistiques_bonnes_reponses = new QPushButton(trUtf8(">>"));
    bouton_liste_des_questions = new QPushButton(trUtf8("Liste des questions..."));

    if(!settings.value("Affichage/bouton_ajouter_question", DEFAUT_AJOUTER_QUESTIONS).toBool())
    {
        bouton_ajouter_question->hide();
    }
    if(!settings.value("Affichage/bouton_supprimer_question_actuelle", DEFAUT_BOUTON_SUPPRIMER_QUESTION_ACTUELLE).toBool())
    {
        bouton_supprimer_question_actuelle->hide();
    }
    if(!settings.value("Affichage/bouton_modifier_question_actuelle", DEFAUT_BOUTON_MODIFIER_QUESTION_ACTUELLE).toBool())
    {
        bouton_modifier_question_actuelle->hide();
    }
    if(!settings.value("Affichage/bouton_vider_liste", DEFAUT_BOUTON_VIDER_LISTE).toBool())
    {
        bouton_vider_liste->hide();
    }
    if(!settings.value("Affichage/bouton_liste_des_questions", DEFAUT_BOUTON_LISTE_DES_QUESTION).toBool())
    {
        bouton_liste_des_questions->hide();
    }

    connect(bouton_ajouter_question,                     SIGNAL(clicked()), this, SLOT(ajouter_question()));
    connect(bouton_supprimer_question_actuelle,          SIGNAL(clicked()), this, SLOT(supprimer_question_actuelle()));
    connect(bouton_modifier_question_actuelle,           SIGNAL(clicked()), this, SLOT(modifier_question_actuelle()));
    connect(bouton_vider_liste,                          SIGNAL(clicked()), this, SLOT(vider_liste()));
    connect(bouton_liste_des_questions,                  SIGNAL(clicked()), this, SLOT(liste_des_questions()));

    // Création du widget central
    cadre_principal = new QWidget;

    // On place les widget dans la fenetre
    QVBoxLayout *layout_boutons_raccourcis = new QVBoxLayout;
    layout_boutons_raccourcis->addWidget(bouton_je_sais);
    layout_boutons_raccourcis->addWidget(bouton_je_ne_sais_pas);
    layout_boutons_raccourcis->addWidget(bouton_je_me_suis_trompe);
    layout_boutons_raccourcis->addWidget(bouton_ne_plus_voir_cette_question);
    layout_boutons_raccourcis->addWidget(bouton_sauter_cette_question);

    QVBoxLayout *layout_boutons_operations = new QVBoxLayout;
    layout_boutons_operations->addWidget(bouton_ajouter_question);
    layout_boutons_operations->addWidget(bouton_supprimer_question_actuelle);
    layout_boutons_operations->addWidget(bouton_modifier_question_actuelle);
    layout_boutons_operations->addWidget(bouton_vider_liste);
    layout_boutons_operations->addWidget(bouton_liste_des_questions);

    QGroupBox *group_boutons_raccourcis = new QGroupBox(trUtf8("Raccourcis"));
    group_boutons_raccourcis->setLayout(layout_boutons_raccourcis);

    QGroupBox *group_boutons_operations = new QGroupBox(trUtf8("Opérations"));
    group_boutons_operations->setLayout(layout_boutons_operations);
    //TODO : CHARGEMENT DES BOUTONS OPERATIONS !!!!!!!!!!!!!!
    if(bouton_ajouter_question->isHidden() &&
       bouton_supprimer_question_actuelle->isHidden() &&
       bouton_modifier_question_actuelle->isHidden() &&
       bouton_vider_liste->isHidden() &&
       bouton_liste_des_questions->isHidden())
    {
        group_boutons_operations->hide();
    }

    QVBoxLayout *layout_boutons_options = new QVBoxLayout;
    layout_boutons_options->addWidget(group_boutons_raccourcis);
    layout_boutons_options->addWidget(group_boutons_operations);
    layout_boutons_options->addStretch();

    QVBoxLayout *layout_tableau = new QVBoxLayout;
    layout_tableau->addWidget(tableau_principal);
    //layout_tableau->addStretch();

    QHBoxLayout *layout_zone_principale = new QHBoxLayout;
    layout_zone_principale->addStretch();
    layout_zone_principale->addLayout(layout_tableau);
    layout_zone_principale->addLayout(layout_boutons_options);
    layout_zone_principale->addStretch();

    QHBoxLayout *layout_question = new QHBoxLayout;
    layout_question->addWidget(texte_question_permanent);
    layout_question->addWidget(texte_question);
    layout_question->addStretch();
    layout_question->addWidget(bouton_changer);
    layout_question->addWidget(bouton_supprimer);
    layout_question->addWidget(bouton_modifier);
    layout_question->addWidget(bouton_ne_plus_voir);

    QHBoxLayout *layout_reponse = new QHBoxLayout;
    layout_reponse->addWidget(texte_reponse);
    layout_reponse->addWidget(champ_reponse);
    layout_reponse->addWidget(bouton_valider);
    layout_reponse->addWidget(bouton_effacer);

    QVBoxLayout *layout_actuel = new QVBoxLayout;
    layout_actuel->addLayout(layout_question);
    layout_actuel->addLayout(layout_reponse);

    QFrame *cadre_actuel = new QFrame;
    cadre_actuel->setFrameShape(QFrame::StyledPanel);
    cadre_actuel->setLayout(layout_actuel);

    QVBoxLayout *layout_principal = new QVBoxLayout;
    layout_principal->addLayout(layout_zone_principale);
    layout_principal->addWidget(cadre_actuel);

    // On indique à la fenetre quelle est le widget central
    cadre_principal->setLayout(layout_principal);

    setCentralWidget(cadre_principal);
}

void MainWindow::creer_actions()
{
    //menu fichier
    action_nouveau                  = new QAction(QIcon("data/images/img_new.png"),       trUtf8("Nouveau"), this);
    action_ouvrir                   = new QAction(QIcon("data/images/img_open.png"),      trUtf8("Ouvrir..."), this);
    action_sauvegarder              = new QAction(QIcon("data/images/img_save.png"),      trUtf8("Sauvegarder"), this);
    action_sauvegarder_sous         = new QAction(                                        trUtf8("Sauvegarder sous..."), this);
    action_imprimer                 = new QAction(                                        trUtf8("Imprimer..."), this);
    action_quitter                  = new QAction(                                        trUtf8("Quitter"), this);
    connect(action_nouveau,          SIGNAL(triggered()), this, SLOT(nouvelle_liste()));
    connect(action_ouvrir,           SIGNAL(triggered()), this, SLOT(ouvrir_liste()));
    connect(action_sauvegarder,      SIGNAL(triggered()), this, SLOT(enregistrer_liste()));
    connect(action_sauvegarder_sous, SIGNAL(triggered()), this, SLOT(enregistrer_liste_sous()));
    connect(action_imprimer,         SIGNAL(triggered()), this, SLOT(imprimer()));
    connect(action_quitter,          SIGNAL(triggered()), this, SLOT(close()));

    //menu edition
    action_ajouter_question                     = new QAction(trUtf8("Ajouter une/des question(s)..."), this);
    action_supprimer_question_actuelle          = new QAction(trUtf8("Supprimer la question actuelle"), this);
    action_supprimer_questions_selectionnees    = new QAction(trUtf8("Supprimer toutes les questions séléctionnées"), this);
    action_modifier_question_actuelle           = new QAction(trUtf8("Modifier la question actuelle"), this);
    action_modifier_questions_selectionnees     = new QAction(trUtf8("Modifier toutes les questions séléctionnées"), this);
    action_ne_plus_voir_question_actuelle       = new QAction(trUtf8("Ne plus voir la question actuelle"), this);
    action_ne_plus_voir_questions_selectionnees = new QAction(trUtf8("Ne plus voir toutes les questions séléctionnées"), this);
    action_inverser_reponse_derniere_question   = new QAction(trUtf8("Inverser la reponse de la derniere question"), this);
    action_sauter_question_actuelle             = new QAction(trUtf8("Sauter la question actuelle"), this);
    action_vider_liste                          = new QAction(trUtf8("Vider la liste"), this);
    action_melanger_liste                       = new QAction(trUtf8("Trier la liste..."), this);
    action_melanger_liste_defaut                = new QAction(trUtf8("Mélanger la liste"), this);
    action_filtrer_liste                        = new QAction(trUtf8("Filtrer la liste"), this);
    action_inverser_filtre                      = new QAction(trUtf8("Inverser le filtre"), this);
    action_reinitialiser_score                  = new QAction(trUtf8("Réinitialiser..."), this);
    action_statistiques_bonnes_reponses         = new QAction(trUtf8("Statistiques des bonnes réponses"), this);
    action_liste_des_questions                  = new QAction(trUtf8("Liste des questions..."), this);
    connect(action_ajouter_question,                     SIGNAL(triggered()), this, SLOT(ajouter_question()));
    connect(action_supprimer_question_actuelle,          SIGNAL(triggered()), this, SLOT(supprimer_question_actuelle()));
    connect(action_supprimer_questions_selectionnees,    SIGNAL(triggered()), this, SLOT(supprimer_questions_selectionnees()));
    connect(action_modifier_question_actuelle,           SIGNAL(triggered()), this, SLOT(modifier_question_actuelle()));
    connect(action_modifier_questions_selectionnees,     SIGNAL(triggered()), this, SLOT(modifier_questions_selectionnees()));
    connect(action_ne_plus_voir_question_actuelle,       SIGNAL(triggered()), this, SLOT(ne_plus_voir_question_actuelle()));
    connect(action_ne_plus_voir_questions_selectionnees, SIGNAL(triggered()), this, SLOT(ne_plus_voir_questions_selectionnees()));
    connect(action_inverser_reponse_derniere_question,   SIGNAL(triggered()), this, SLOT(inverser_reponse_derniere_question()));
    connect(action_sauter_question_actuelle,             SIGNAL(triggered()), this, SLOT(sauter_question_actuelle()));
    connect(action_vider_liste,                          SIGNAL(triggered()), this, SLOT(vider_liste()));
    connect(action_melanger_liste,                       SIGNAL(triggered()), this, SLOT(melanger_liste()));
    connect(action_melanger_liste_defaut,                SIGNAL(triggered()), this, SLOT(melanger_liste_defaut()));
    connect(action_filtrer_liste,                        SIGNAL(triggered()), this, SLOT(filtrer_liste()));
    connect(action_inverser_filtre,                      SIGNAL(triggered()), this, SLOT(inverser_filtre()));
    connect(action_reinitialiser_score,                  SIGNAL(triggered()), this, SLOT(reinitialiser_score()));
    connect(action_statistiques_bonnes_reponses,         SIGNAL(triggered()), this, SLOT(statistiques_bonnes_reponses()));
    connect(action_liste_des_questions,                  SIGNAL(triggered()), this, SLOT(liste_des_questions()));

    //menu preference
    action_ordre_question_reponse                = new QAction(trUtf8("Question -> Réponse(0%)"), this);
    action_ordre_question_reponse_75_pour_cent   = new QAction(trUtf8("Question -> Réponse(25%)"), this);
    action_ordre_reponse_question                = new QAction(trUtf8("Réponse -> Question(100%)"), this);
    action_ordre_reponse_question_75_pour_cent   = new QAction(trUtf8("Réponse -> Question(75%)"), this);
    action_ordre_aleatoire                       = new QAction(trUtf8("Aléatoire(50%)"), this);
    action_ordre_question_reponse->setCheckable(true);
    action_ordre_question_reponse_75_pour_cent->setCheckable(true);
    action_ordre_reponse_question->setCheckable(true);
    action_ordre_reponse_question_75_pour_cent->setCheckable(true);
    action_ordre_aleatoire->setCheckable(true);
    group_ordre                                  = new QActionGroup(this);
    group_ordre->addAction(action_ordre_question_reponse);
    group_ordre->addAction(action_ordre_question_reponse_75_pour_cent);
    group_ordre->addAction(action_ordre_aleatoire);
    group_ordre->addAction(action_ordre_reponse_question_75_pour_cent);
    group_ordre->addAction(action_ordre_reponse_question);
    group_ordre->actions().at(settings.value("Options/groupe_ordre_action", DEFAUT_GROUPE_ORDRE_ACTION).toInt())->setChecked(true);

    action_affichage_duree_jours                 = new QAction(trUtf8("Du jour"), this);
    action_affichage_duree_semaine               = new QAction(trUtf8("De la semaine"), this);
    action_affichage_duree_mois                  = new QAction(trUtf8("Du mois"), this);
    action_affichage_duree_annee                 = new QAction(trUtf8("De l'année"), this);
    action_affichage_duree_tous                  = new QAction(trUtf8("Tous"), this);
    action_affichage_duree_depuis_le             = new QAction(trUtf8("Personnalisé : depuis le"), this);
    action_affichage_duree_jours->setCheckable(true);
    action_affichage_duree_semaine->setCheckable(true);
    action_affichage_duree_mois->setCheckable(true);
    action_affichage_duree_annee->setCheckable(true);
    action_affichage_duree_tous->setCheckable(true);
    action_affichage_duree_depuis_le->setCheckable(true);
    group_affichage_duree                        = new QActionGroup(this);
    group_affichage_duree->addAction(action_affichage_duree_jours);
    group_affichage_duree->addAction(action_affichage_duree_semaine);
    group_affichage_duree->addAction(action_affichage_duree_mois);
    group_affichage_duree->addAction(action_affichage_duree_annee);
    group_affichage_duree->addAction(action_affichage_duree_tous);
    group_affichage_duree->addAction(action_affichage_duree_depuis_le);
    group_affichage_duree->actions().at(settings.value("Options/affichage_duree", DEFAUT_AFFICHAGE_DUREE).toInt())->setChecked(true);

    action_affichage_duree_personnalise          = new QAction(trUtf8("Personnaliser..."), this);
    action_definir_tags_defaut                   = new QAction(trUtf8("Définir les tags par defaut..."), this);
    action_ajouter_supprimer_tags                = new QAction(trUtf8("Ajouter ou supprimer des tags..."), this);
    action_supprimer_question_avec_tags          = new QAction(trUtf8("Supprimer questions avec tags..."), this);
    action_gerer_tags                            = new QAction(trUtf8("Gerer les tags..."), this);
    action_gerer_claviers_virtuels               = new QAction(trUtf8("Gerer les claviers virtuels..."), this);
    action_calcul_automatique_difficultee        = new QAction(trUtf8("Calcul automatique difficultee"), this);
    action_calcul_automatique_difficultee->setCheckable(true);
    action_calcul_automatique_difficultee->setChecked(settings.value("Options/calcul_automatique_difficultee", DEFAUT_CALCUL_AUTOMATIQUE_DIFICULTEE).toBool());
    action_actualiser_filtre_en_permanace        = new QAction(trUtf8("Actualiser les filtres en permanance"), this);
    action_actualiser_filtre_en_permanace->setCheckable(true);
    action_actualiser_filtre_en_permanace->setChecked(settings.value("Options/actualiser_filtre_en_permanace", DEFAUT_ACTUALISER_FILTRE_EN_PERMANANCE).toBool());
    action_ajouter_a_la_fin                      = new QAction(trUtf8("Ajouter les questions à la fin de la liste"), this);
    action_ajouter_a_la_fin->setCheckable(true);
    action_ajouter_a_la_fin->setChecked(settings.value("Options/ajouter_a_la_fin", DEFAUT_AJOUTER_A_LA_FIN).toBool());
    action_tolerance                             = new QAction(trUtf8("Tolérance"), this);
    action_tolerance->setCheckable(true);
    action_tolerance->setChecked(settings.value("Options/tolerance", DEFAUT_TOLERANCE).toBool());
    action_tolerance_accents                     = new QAction(trUtf8("Accents facultatifs"), this);//prend en compte les accents
    action_tolerance_accents->setCheckable(true);
    action_tolerance_accents->setChecked(settings.value("Options/tolerance_accents", DEFAUT_TOLERANCE_ACCENTS).toBool());
    action_tolerance_majuscules                  = new QAction(trUtf8("Majuscules facultatives"), this);//prend en compte les majuscules
    action_tolerance_majuscules->setCheckable(true);
    action_tolerance_majuscules->setChecked(settings.value("Options/tolerance_majuscules", DEFAUT_TOLERANCE_MAJUSCULES).toBool());
    action_preferences                           = new QAction(trUtf8("Préférences..."), this);

    connect(group_ordre,                           SIGNAL(triggered(QAction*)),  this, SLOT(ordre(QAction*)));
    connect(group_affichage_duree,                 SIGNAL(triggered(QAction*)),  this, SLOT(affichage_duree(QAction*)));
    connect(action_affichage_duree_personnalise,   SIGNAL(triggered()),          this, SLOT(affichage_duree_personnalise()));
    connect(action_definir_tags_defaut,            SIGNAL(triggered()),          this, SLOT(definir_tags_defaut()));
    connect(action_ajouter_supprimer_tags,         SIGNAL(triggered()),          this, SLOT(ajouter_supprimer_tags()));
    connect(action_supprimer_question_avec_tags,   SIGNAL(triggered()),          this, SLOT(supprimer_question_avec_tags()));
    connect(action_gerer_tags,                     SIGNAL(triggered()),          this, SLOT(gerer_tags()));
    connect(action_gerer_claviers_virtuels,        SIGNAL(triggered()),          this, SLOT(gerer_claviers_virtuels()));
    connect(action_calcul_automatique_difficultee, SIGNAL(triggered(bool)),      this, SLOT(calcul_automatique_difficultee(bool)));
    connect(action_actualiser_filtre_en_permanace, SIGNAL(triggered(bool)),      this, SLOT(actualiser_filtre_en_permanace(bool)));
    connect(action_ajouter_a_la_fin,               SIGNAL(triggered(bool)),      this, SLOT(ajouter_a_la_fin(bool)));
    connect(action_tolerance,                      SIGNAL(triggered(bool)),      this, SLOT(tolerance(bool)));
    connect(action_tolerance_accents,              SIGNAL(triggered(bool)),      this, SLOT(tolerance_accents(bool)));
    connect(action_tolerance_majuscules,           SIGNAL(triggered(bool)),      this, SLOT(tolerance_majuscules(bool)));
    connect(action_preferences,                    SIGNAL(triggered()),          this, SLOT(preferences()));
    ordre(group_ordre->checkedAction());
    affichage_duree(group_affichage_duree->checkedAction());
    calcul_automatique_difficultee(action_calcul_automatique_difficultee->isChecked());
    actualiser_filtre_en_permanace(action_actualiser_filtre_en_permanace->isChecked());
    tolerance(action_tolerance->isChecked());
    tolerance_accents(action_tolerance_accents->isChecked());
    tolerance_majuscules(action_tolerance_majuscules->isChecked());

    //menu aide
    action_manuel                 = new QAction(QIcon("data/images/img_manuel.png"),   trUtf8("Manuel"), this);
    action_aide_en_ligne          = new QAction(                                       trUtf8("Aide en ligne"), this);
    action_rechercher_mise_a_jour = new QAction(                                       trUtf8("Rechercher des mises à jours"), this);
    action_a_propos               = new QAction(                                       trUtf8("A propos..."), this);
    action_a_propos_qt            = new QAction(QIcon("data/images/img_about_qt.png"), trUtf8("A propos de Qt..."), this);
    connect(action_manuel,                 SIGNAL(triggered()), this, SLOT(manuel()));
    connect(action_aide_en_ligne,          SIGNAL(triggered()), this, SLOT(aide_en_ligne()));
    connect(action_rechercher_mise_a_jour, SIGNAL(triggered()), this, SLOT(rechercher_mise_a_jour()));
    connect(action_a_propos,               SIGNAL(triggered()), this, SLOT(a_propos()));
    connect(action_a_propos_qt,            SIGNAL(triggered()), qApp, SLOT(aboutQt()));
}

void MainWindow::creer_menus()
{
    QMenu *menu_fichier = menuBar()->addMenu(trUtf8("Fichier"));
    menu_fichier->addAction(action_nouveau);
    menu_fichier->addAction(action_ouvrir);
    menu_fichier->addSeparator();
    menu_fichier->addAction(action_sauvegarder);
    menu_fichier->addAction(action_sauvegarder_sous);
    menu_fichier->addSeparator();
    menu_fichier->addAction(action_imprimer);
    menu_fichier->addSeparator();
    menu_fichier->addAction(action_quitter);

    QMenu *menu_edition = menuBar()->addMenu(trUtf8("Edition"));
    menu_edition->addAction(action_ajouter_question);
    menu_edition->addAction(action_supprimer_question_actuelle);
    menu_edition->addAction(action_supprimer_questions_selectionnees);
    menu_edition->addAction(action_modifier_question_actuelle);
    menu_edition->addAction(action_modifier_questions_selectionnees);
    menu_edition->addAction(action_ne_plus_voir_question_actuelle);
    menu_edition->addAction(action_ne_plus_voir_questions_selectionnees);
    menu_edition->addAction(action_inverser_reponse_derniere_question);
    menu_edition->addAction(action_sauter_question_actuelle);
    menu_edition->addSeparator();
    menu_edition->addAction(action_vider_liste);
    menu_edition->addSeparator();
    menu_edition->addAction(action_melanger_liste);
    menu_edition->addAction(action_melanger_liste_defaut);
    menu_edition->addAction(action_filtrer_liste);
    menu_edition->addAction(action_inverser_filtre);
    menu_edition->addSeparator();
    menu_edition->addAction(action_reinitialiser_score);
    menu_edition->addAction(action_statistiques_bonnes_reponses);
    menu_edition->addSeparator();
    menu_edition->addAction(action_liste_des_questions);

    QMenu *menu_outils = menuBar()->addMenu(trUtf8("Outils"));
        QMenu *menu_ordre = menu_outils->addMenu(trUtf8("Ordre de questionnage"));
        menu_ordre->addActions(group_ordre->actions());
        QMenu *menu_affichage_duree = menu_outils->addMenu(trUtf8("Afficher uniquement depuis"));
        menu_affichage_duree->addActions(group_affichage_duree->actions());
        menu_affichage_duree->addSeparator();
        menu_affichage_duree->addAction(action_affichage_duree_personnalise);
        QMenu *menu_tags = menu_outils->addMenu(trUtf8("Tags"));
        menu_tags->addAction(action_definir_tags_defaut);
        menu_tags->addAction(action_ajouter_supprimer_tags);
        menu_tags->addAction(action_supprimer_question_avec_tags);
        menu_tags->addAction(action_gerer_tags);
        menu_claviers_virtuels = menu_outils->addMenu(trUtf8("Claviers virtuels"));
        menu_claviers_virtuels->addSeparator();
        menu_claviers_virtuels->addAction(action_gerer_claviers_virtuels);
    menu_outils->addSeparator();
    menu_outils->addAction(action_calcul_automatique_difficultee);
    menu_outils->addAction(action_actualiser_filtre_en_permanace);
    menu_outils->addAction(action_ajouter_a_la_fin);
    menu_outils->addSeparator();
    menu_outils->addAction(action_tolerance);
    menu_outils->addAction(action_tolerance_accents);
    menu_outils->addAction(action_tolerance_majuscules);
    menu_outils->addSeparator();
    menu_outils->addAction(action_preferences);

    QMenu *menu_aide = menuBar()->addMenu(trUtf8("Aide"));
    menu_aide->addAction(action_manuel);
    menu_aide->addAction(action_aide_en_ligne);
    menu_aide->addSeparator();
    menu_aide->addAction(action_rechercher_mise_a_jour);
    menu_aide->addSeparator();
    menu_aide->addAction(action_a_propos);
    menu_aide->addAction(action_a_propos_qt);
}

void MainWindow::creer_menu_contextuel()
{
    QAction *action_ne_plus_voir = new QAction(trUtf8("Ne plus voir"), this);
    QAction *action_ajouter_supprimer_tags_questions_selectionnees = new QAction(trUtf8("Ajouter ou supprimer des tags..."), this);
    QAction *action_supprimer = new QAction(trUtf8("Supprimer"), this);
    QAction *action_action_modifier = new QAction(trUtf8("Modifier..."), this);
    connect(action_supprimer,                                      SIGNAL(triggered()), this, SLOT(supprimer_questions_selectionnees()));
    connect(action_ajouter_supprimer_tags_questions_selectionnees, SIGNAL(triggered()), this, SLOT(ajouter_supprimer_tags_questions_selectionnees()));
    connect(action_action_modifier,                                SIGNAL(triggered()), this, SLOT(modifier_questions_selectionnees()));
    connect(action_ne_plus_voir,                                   SIGNAL(triggered()), this, SLOT(ne_plus_voir_questions_selectionnees()));

    tableau_principal->addAction(action_ne_plus_voir);
    tableau_principal->addAction(action_ajouter_supprimer_tags_questions_selectionnees);
    tableau_principal->addAction(action_supprimer);
    tableau_principal->addAction(action_action_modifier);
    tableau_principal->setContextMenuPolicy(Qt::ActionsContextMenu);
}

void MainWindow::creer_docks()
{
    setDockNestingEnabled(true);
    if(!settings.contains("Claviers/clavier_0_nom"))// si c'est la première fois que l'on lance le programme
    {
        settings.setValue("Claviers/clavier_0_nom", "Clavier standard");
        settings.setValue("Claviers/clavier_0", "1234567890\n\nABCDEFG\nHIJKLMNO\nPQRSTUV\nWXYZ");
        settings.setValue("Claviers/clavier_1_nom", "Clavier de carractères speciaux");
        settings.setValue("Claviers/clavier_1", "æâ€êþÿûîœô\näßë‘’ðüïŀö\n«»©↓¬");
        settings.setValue("Claviers/clavier_2_nom", "Pavé numérique");
        settings.setValue("Claviers/clavier_2", "789\n456\n123\n0");
    }
    long nombre_de_clavier = 0;
    while(settings.contains("Claviers/clavier_" + QString::number(nombre_de_clavier) + "_nom"))
    {
        ajouter_clavier_virtuel(settings.value("Claviers/clavier_" + QString::number(nombre_de_clavier) + "_nom").toString(), settings.value("Claviers/clavier_" + QString::number(nombre_de_clavier)).toString());
        nombre_de_clavier++;
    }
    connect(&mapper_clavier_virtuels, SIGNAL(mapped(const QString &)), this, SLOT(ajouter_carractere_special(QString)));
}

void MainWindow::creer_barres_outils()
{
    //Vide ?
}

void MainWindow::creer_barre_etat()
{
}

MainWindow::~MainWindow()
{
}
