/*
  Copyright (C) 2009 Pot Etienne

  This file is part of Vocab++.

  Vocab++ is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Vocab++ is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Vocab++.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "widget_tags.hpp"

#define TAILLE_MAXIMUM_SCROLL_AREA 150

Widget_tags::Widget_tags(QStringList *c_liste_tags, QWidget *parent) : QWidget(parent)
{
    liste_tags = c_liste_tags;
    vue_tag_clique_premier = false;
    model_tag_change_premier = false;
    champ_tags_modification_autorise = true;
    model_modification_autorise = true;

    champ_tags = new QLineEdit;
    connect(champ_tags, SIGNAL(textChanged(QString)), this, SLOT(champ_tags_modifie()));

    bouton_liste_tags                = new QPushButton(trUtf8("Liste..."));
    bouton_liste_tags->setCheckable(true);
    connect(bouton_liste_tags, SIGNAL(clicked(bool)), this, SLOT(afficher_liste_tags(bool)));

    model_liste_tags = new QStandardItemModel;
    for(int i = 0 ; i < liste_tags->size() ; ++i)
    {
        QStandardItem *item_liste_tags = new QStandardItem;
        item_liste_tags->setFlags(Qt::ItemIsEnabled | Qt::ItemIsUserCheckable);
        item_liste_tags->setCheckState(Qt::Unchecked);
        item_liste_tags->setText(liste_tags->at(i));
        model_liste_tags->appendRow(item_liste_tags);
    }
    model_liste_tags->sort(0, Qt::AscendingOrder);
    if(liste_tags->isEmpty())
    {
        bouton_liste_tags->hide();// on ne grise pas le bouton car ca ne sert à rien(les tags que l'on ajoutent sont tous utiles)
    }
    vue_liste_tags = new QListView;
    vue_liste_tags->setModel(model_liste_tags);
    vue_liste_tags->setSelectionMode(QAbstractItemView::NoSelection);
    vue_liste_tags->setMaximumHeight(TAILLE_MAXIMUM_SCROLL_AREA);
    connect(model_liste_tags,       SIGNAL(itemChanged(QStandardItem*)), this, SLOT(model_tag_change(QStandardItem*)));
    connect(vue_liste_tags,         SIGNAL(clicked(QModelIndex)),        this, SLOT(vue_tag_clique(QModelIndex)));

    layout_champ_tags = new QHBoxLayout;
    layout_champ_tags->addWidget(champ_tags);
    layout_champ_tags->addWidget(bouton_liste_tags);

    QVBoxLayout *layout_principal = new QVBoxLayout;
    layout_principal->addLayout(layout_champ_tags);
    layout_principal->addWidget(vue_liste_tags);
    layout_principal->setMargin(0);

    setLayout(layout_principal);
}

void Widget_tags::from_string_list(QStringList liste_tags_a_metre)
{
    QString chaine_tags;
    for(int i = 0 ; i < liste_tags_a_metre.size() ; ++i)
    {
        chaine_tags += ", " + liste_tags_a_metre.at(i);
    }
    chaine_tags.remove(QRegExp("^, "));
    champ_tags->setText(chaine_tags);//widget_tags s'occuppe tous seul d'actualiser le cochage
}

QStringList Widget_tags::to_string_list()
{
    valider_champ_tags();
    return champ_tags->text().split(", ");
}

QString Widget_tags::valider_chaine(QString chaine_a_valider)
{
    return chaine_a_valider;
}

void Widget_tags::ajouter_tag(QString nouveau_tag)
{
    liste_tags->prepend(nouveau_tag);
    QStandardItem *item_liste_tags = new QStandardItem;
    item_liste_tags->setFlags(Qt::ItemIsEnabled | Qt::ItemIsUserCheckable);
    item_liste_tags->setText(nouveau_tag);
    model_liste_tags->appendRow(item_liste_tags);
    model_liste_tags->sort(0, Qt::AscendingOrder);
    if(!liste_tags->isEmpty())
    {
        bouton_liste_tags->show();// si la liste n'est plus vide, on affiche le bouton pour la prochaine fois(sinon, il reste masqué)
    }
}

void Widget_tags::afficher_liste_tags(bool enfonce)
{
    model_modification_autorise = false;
    vue_liste_tags->setVisible(enfonce);
    setFixedHeight(sizeHint().height());
    if(parent())
    {
        static_cast<QWidget*>(parent())->setFixedHeight(static_cast<QWidget*>(parent())->sizeHint().height());
    }
    emit liste_tags_afficher(enfonce);// Pour qu le widget parent puisse adapter sa taille
    model_modification_autorise = true;
}

void Widget_tags::champ_tags_modifie()
{
    if(champ_tags_modification_autorise)
    {
        model_modification_autorise = false;

        for(int i = 0 ; i < model_liste_tags->rowCount(QModelIndex()) ; ++i)
        {
            model_liste_tags->item(i)->setCheckState(Qt::Unchecked);// On décoche tous les cases
        }

        QStringList liste_tags_champ = champ_tags->text().split(", ", QString::SkipEmptyParts);
        for(int i = 0 ; i < liste_tags_champ.size() ; ++i)
        {
            for(int j = 0 ; j < model_liste_tags->rowCount(QModelIndex()) ; ++j)
            {
                if(liste_tags_champ.at(i) == model_liste_tags->item(j)->text())
                {
                    model_liste_tags->item(j)->setCheckState(Qt::Checked);
                }
            }
        }
        // 2eme moyen, sans doute plus performant pour une grande liste(on ne parcours qu'une fois la liste
        /*
        for(int i = 0 ; i < model_liste_tags->rowCount(QModelIndex()) ; ++i)
        {
            bool possede_ce_tag = false;

            for(int j = 0 ; j < question_a_ajouter->tags.size() ; ++j)
            {
                if(model_liste_tags->item(i)->text() == question_a_ajouter->tags.at(j))
                {
                    possede_ce_tag = true;
                    break;
                }
            }

            if(possede_ce_tag)
            {
                model_liste_tags->item(i)->setCheckState(Qt::Checked);
            }
            else
            {
                model_liste_tags->item(i)->setCheckState(Qt::Unchecked);
            }
        }
        */

        model_modification_autorise = true;
    }
}

void Widget_tags::vue_tag_clique(QModelIndex index)
{
    if(!model_tag_change_premier)// si le model n'a pas encore été changé, cette fonction à été appelé en premier
    {
        vue_tag_clique_premier = true;// On change cette variable en premier

        QStandardItem *item = model_liste_tags->itemFromIndex(index);
        if(item->checkState() == Qt::Unchecked)
        {
            item->setCheckState(Qt::Checked);
        }
        else
        {
            item->setCheckState(Qt::Unchecked);// On inverse le cochage
        }
    }
    else
    {
        model_tag_change_premier = false;//on à déja changé donc on ne gere pas le clique
    }
}

void Widget_tags::model_tag_change(QStandardItem *item)
{
    if(model_modification_autorise)
    {
        // TODO BUG : Si on drap and drog sur une case : ca appelle cette fonction mais la vue ne détecte pas le clique
        if(!vue_tag_clique_premier)
        {
            model_tag_change_premier = true;
        }
        else
        {
            vue_tag_clique_premier = false;
        }
        // Maintenant que l'on à modifié un cochage, on actualise le QLineEdit

        champ_tags_modification_autorise = false;// On peut désormais faire toutes les modifications que l'on veut sans que champ_tags_modifie soit appelé à chaque fois
        valider_champ_tags();
        if(item->checkState() == Qt::Checked)
        {
            QStringList chaine_tags = champ_tags->text().split(", ");// Pas besoin de sauter les espaces vides vue que la chaine est passée dans le validateur
            bool existe_deja = false;
            for(int i = 0 ; i < chaine_tags.size() ; ++i)
            {
                if(item->text() == chaine_tags.at(i))
                {
                    existe_deja = true;
                }
            }
            if(!existe_deja)
            {
                if(champ_tags->text().isEmpty())
                {
                    champ_tags->setText(champ_tags->text() + item->text());
                }
                else
                {
                    champ_tags->setText(champ_tags->text() + ", " + item->text());
                }
            }
        }
        else
        {
            QStringList liste_tags_champ = champ_tags->text().split(", ");
            QString chaine_tags;
            for(int i = 0 ; i < liste_tags_champ.size() ; ++i)
            {
                if(liste_tags_champ.at(i) == item->text())
                {
                    liste_tags_champ.removeAt(i);
                    i--;
                }
                else
                {
                    chaine_tags += ", " + liste_tags_champ.at(i);
                }
            }
            chaine_tags.remove(QRegExp("(?:^, )"));
            champ_tags->setText(chaine_tags);
        }
        champ_tags_modification_autorise = true;
    }
}

void Widget_tags::valider_champ_tags()
{
    // TODO : Validation avec des regexp : suppression des espaces et virgules inutiles, rajout les espaces la ou on en à besoin, suppression des virgules à la fin...
    champ_tags_modification_autorise = false;
    champ_tags->setText(valider_chaine(champ_tags->text()));// On fait les modifications necessaires
    champ_tags_modification_autorise = true;
}
